//
//  AppDelegate.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/13.
//

#import "AppDelegate.h"
#import "BC_LoginViewController.h"
#import "MainTabbarViewController.h"
#import "TUILogin.h"
#import "GenerateTestUserSig.h"
#import "TUIConfig.h"
@interface AppDelegate ()

@end

@implementation AppDelegate

+ (AppDelegate *)sharedAppDelegate {
    return (AppDelegate *)[[UIApplication sharedApplication] delegate];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    // Override point for customization after application launch.
    [IQKeyboardManager sharedManager].enable = YES;
    [IQKeyboardManager sharedManager].shouldResignOnTouchOutside = YES;
    [IQKeyboardManager sharedManager].shouldShowToolbarPlaceholder = YES; // 是否显示占位文字
    [IQKeyboardManager sharedManager].enableAutoToolbar = YES;
    [[IQKeyboardManager sharedManager] registerTextFieldViewClass:[YYTextView class] didBeginEditingNotificationName:YYTextViewTextDidBeginEditingNotification didEndEditingNotificationName:YYTextViewTextDidEndEditingNotification];
    
    MJRefreshConfig.defaultConfig.languageCode = @"en";
    
//    [TUILogin initWithSdkAppID:SDKAPPID];
    [TUIConfig defaultConfig].defaultAvatarImage = kAvatarPlaceHolderImg;
    [TUIConfig defaultConfig].defaultGroupAvatarImage = kAvatarPlaceHolderImg;
    [TUIConfig defaultConfig].avatarType = TAvatarTypeRounded;
    
    
    if ([KStringIsNull([UserManager shareManager].getUserInfo.member_id) isEqualToString:@""]) {
        [UserManager shareManager].isLogin = NO;
        [self setLoginRoot];
    } else {
        BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
        NSString *iden = KStringIsNull(umodel.TUIIden);
        NSString *sig = KStringIsNull(umodel.TUISig);
        @weakify(self);
        [TUILogin login:SDKAPPID userID:iden userSig:sig succ:^{
            NSLog(@"-----> 登录成功");
            @strongify(self);
            [UserManager shareManager].isLogin = YES;
            BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
            [PPNetworkHelper setValue:kStringFormat(@"JWT %@", KStringIsNull(umodel.access)) forHTTPHeaderField:@"Authorization"];
            MainTabbarViewController *tabbar = [MainTabbarViewController new];
            if ([KStringIsNull(umodel.identity) isEqualToString:@"1"]) {
                //游客
                [tabbar setupGuestTab];
            } else if ([KStringIsNull(umodel.identity) isEqualToString:@"2"]) {
                //专业用户
                [tabbar setupProfessionTab];
            } else if ([KStringIsNull(umodel.identity) isEqualToString:@"3"]) {
                //执行者
                [tabbar setupExecutorTab];
            } else if ([KStringIsNull(umodel.identity) isEqualToString:@"4"]) {
                //督导
                [tabbar setupSupervisorTab];
            }
            self.window.backgroundColor = UIColor.whiteColor;
            self.window.rootViewController = tabbar;
            [self.window makeKeyAndVisible];
        } fail:^(int code, NSString *msg) {
            NSLog(@"-----> 登录失败===%@ ===%i", msg, code);
            @strongify(self);
            [UserManager shareManager].isLogin = NO;
            [self setLoginRoot];
        }];
    }
    return YES;
}

#pragma mark  登录页
- (void)setLoginRoot {
    BaseNavigationViewController *nav = [[BaseNavigationViewController alloc] initWithRootViewController:[BC_LoginViewController new]];
    self.window.backgroundColor = UIColor.whiteColor;
    self.window.rootViewController = nav;
    [self.window makeKeyAndVisible];
}

- (UIInterfaceOrientationMask)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(nullable UIWindow *)window{
    if (_allowRotation == YES) {
        return UIInterfaceOrientationMaskAllButUpsideDown;  // 这里是屏幕要旋转的方向
    } else {
        return (UIInterfaceOrientationMaskPortrait);
    }
}

- (UIViewController *)getCurrentVC{
    
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for (UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    return result;
}

#pragma mark  获取顶层控制器
- (UIViewController *)getCurrentUIVC
{
    UIViewController *superVC = [self getCurrentVC];
    if ([superVC isKindOfClass:[UITabBarController class]]) {
        UIViewController *tabSelectVC = ((UITabBarController*)superVC).selectedViewController;
        if ([tabSelectVC isKindOfClass:[UINavigationController class]]) {
            return ((UINavigationController*)tabSelectVC).viewControllers.lastObject;
        }
        return tabSelectVC;
    } else if ([superVC isKindOfClass:[UINavigationController class]]) {
        return ((UINavigationController*)superVC).viewControllers.lastObject;
    }
    return superVC;
}

@end
