//
//  BC_ProjectInformationViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/22.
//

#import "BC_ProjectInformationViewController.h"
#import "BC_ProjectMaterialViewController.h"
#import "BC_ProjectScheduleViewController.h"

#import "BC_ProjectDetailPicCollectionViewCell.h"
#import "BC_ProjectInfoFileCollectionViewCell.h"
#import "ZlinkPhotoBrowserView.h"

#import "BC_ExecuteModuleModel.h"

@interface BC_ProjectInformationViewController ()<UICollectionViewDelegate, UICollectionViewDataSource>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;

@property (nonatomic, strong) UILabel *timeL;
@property (nonatomic, strong) UILabel *cateL;
@property (nonatomic, strong) UILabel *nameL;
@property (nonatomic, strong) UILabel *descL;
@property (nonatomic, strong) UIView *areaView;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) UICollectionView *picCollectionView;
@property (nonatomic, strong) UICollectionView *contractCollectionView;
@property (nonatomic, strong) UICollectionView *agreementCollectionView;

@property (nonatomic, strong) BC_ExecuteModuleModel *infoModel;
@property (nonatomic, strong) NSMutableArray *proPicArr;
@property (nonatomic, strong) NSMutableArray *contractArr;
@property (nonatomic, strong) NSMutableArray *agreeArr;

@end

@implementation BC_ProjectInformationViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Project details";
    self.proPicArr = [NSMutableArray array];
    self.contractArr = [NSMutableArray array];
    self.agreeArr = [NSMutableArray array];

    [self setupUI];
    
    [self getPageData];
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(15);
    }];
    
    self.cateL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:UIColor.whiteColor];
    self.cateL.layer.masksToBounds = YES;
    self.cateL.layer.cornerRadius = 14;
    self.cateL.textAlignment = NSTextAlignmentCenter;
    self.cateL.backgroundColor = kColorWithHex(@"#05A833");
    [self.contentView addSubview:self.cateL];
    [self.cateL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.centerY.equalTo(self.timeL);
        make.size.mas_equalTo(CGSizeMake(30, 28));
    }];
    
    self.nameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(17) textColor:kMainTextColor];
    self.nameL.numberOfLines = 0;
    [self.contentView addSubview:self.nameL];
    [self.nameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.timeL.mas_bottom).offset(20);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.descL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kLightGrayTextColor];
    self.descL.numberOfLines = 0;
    [self.contentView addSubview:self.descL];
    [self.descL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.nameL.mas_bottom).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.areaView = [JLFactoryUtil view];
    self.areaView.backgroundColor = kColorWithHex(@"#ECF8EF");
    [self.areaView setCorner:12 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.areaView];
    [self.areaView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.descL.mas_bottom).offset(10);
        make.size.mas_equalTo(CGSizeMake(30, 24));
    }];
    
    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.areaView addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(self.areaView.mas_left).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.areaView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(areaImg.mas_right).offset(4);
        make.width.mas_lessThanOrEqualTo(kScreenWidth - 70);
    }];
    
    UILabel *picL = [JLFactoryUtil labelWithText:@"Picture" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [self.contentView addSubview:picL];
    [picL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.areaView.mas_bottom).offset(20);
    }];
    
    UICollectionViewFlowLayout *taglayout = [UICollectionViewFlowLayout new];
    taglayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    taglayout.minimumInteritemSpacing = 10;
    taglayout.minimumLineSpacing = 10;
    taglayout.itemSize = CGSizeMake((kScreenWidth-50)/4, (kScreenWidth-50)/4);
    taglayout.sectionInset = UIEdgeInsetsMake(10, 10, 10, 10);
    self.picCollectionView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:taglayout];
    self.picCollectionView.delegate = self;
    self.picCollectionView.dataSource = self;
    self.picCollectionView.backgroundColor = UIColor.whiteColor;
    self.picCollectionView.showsVerticalScrollIndicator = NO;
    self.picCollectionView.scrollEnabled = NO;
    [self.picCollectionView registerClass:[BC_ProjectDetailPicCollectionViewCell class] forCellWithReuseIdentifier:@"membercell"];
    [self.contentView addSubview:self.picCollectionView];
    [self.picCollectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView);
        make.left.equalTo(self.contentView);
        make.top.equalTo(picL.mas_bottom).offset(10);
        make.height.mas_equalTo(1);
    }];
    
    UILabel *fileL = [JLFactoryUtil labelWithText:@"File" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [self.contentView addSubview:fileL];
    [fileL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.picCollectionView.mas_bottom).offset(20);
    }];
        
    UIView *conView = [JLFactoryUtil view];
    conView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:conView];
    [conView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(fileL.mas_bottom).offset(8);
        make.height.mas_equalTo(32);
    }];
    
    UILabel *conL = [JLFactoryUtil labelWithText:@"Contract" font:REGULARFONT(14) textColor:kMainTextColor];
    [conView addSubview:conL];
    [conL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(conView.mas_left).offset(15);
        make.centerY.equalTo(conView);
    }];
    
    UICollectionViewFlowLayout *conlayout = [UICollectionViewFlowLayout new];
    conlayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    conlayout.minimumInteritemSpacing = 10;
    conlayout.minimumLineSpacing = 10;
    conlayout.itemSize = CGSizeMake((kScreenWidth-50)/4, 103);
    conlayout.sectionInset = UIEdgeInsetsMake(10, 10, 10, 10);
    self.contractCollectionView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:conlayout];
    self.contractCollectionView.delegate = self;
    self.contractCollectionView.dataSource = self;
    self.contractCollectionView.backgroundColor = UIColor.whiteColor;
    self.contractCollectionView.showsVerticalScrollIndicator = NO;
    self.contractCollectionView.scrollEnabled = NO;
    [self.contractCollectionView registerClass:[BC_ProjectInfoFileCollectionViewCell class] forCellWithReuseIdentifier:@"contractcell"];
    [self.contentView addSubview:self.contractCollectionView];
    [self.contractCollectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView);
        make.left.equalTo(self.contentView);
        make.top.equalTo(conView.mas_bottom).offset(10);
        make.height.mas_equalTo(1);
    }];
    
    UIView *agreeView = [JLFactoryUtil view];
    agreeView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:agreeView];
    [agreeView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.contractCollectionView.mas_bottom).offset(10);
        make.height.mas_equalTo(32);
    }];
    
    UILabel *agreeL = [JLFactoryUtil labelWithText:@"Agreement" font:REGULARFONT(14) textColor:kMainTextColor];
    [agreeView addSubview:agreeL];
    [agreeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(agreeView.mas_left).offset(15);
        make.centerY.equalTo(agreeView);
    }];
    
    UICollectionViewFlowLayout *agrlayout = [UICollectionViewFlowLayout new];
    agrlayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    agrlayout.minimumInteritemSpacing = 10;
    agrlayout.minimumLineSpacing = 10;
    agrlayout.itemSize = CGSizeMake((kScreenWidth-50)/4, 103);
    agrlayout.sectionInset = UIEdgeInsetsMake(10, 10, 10, 10);
    self.agreementCollectionView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:agrlayout];
    self.agreementCollectionView.delegate = self;
    self.agreementCollectionView.dataSource = self;
    self.agreementCollectionView.backgroundColor = UIColor.whiteColor;
    self.agreementCollectionView.showsVerticalScrollIndicator = NO;
    self.agreementCollectionView.scrollEnabled = NO;
    [self.agreementCollectionView registerClass:[BC_ProjectInfoFileCollectionViewCell class] forCellWithReuseIdentifier:@"agreecell"];
    [self.contentView addSubview:self.agreementCollectionView];
    [self.agreementCollectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView);
        make.left.equalTo(self.contentView);
        make.top.equalTo(agreeView.mas_bottom).offset(10);
        make.height.mas_equalTo(1);
    }];
    
    UIView *matreView = [JLFactoryUtil view];
    matreView.backgroundColor = UIColor.whiteColor;
    [self.contentView addSubview:matreView];
    [matreView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.agreementCollectionView.mas_bottom).offset(20);
        make.height.mas_equalTo(60);
    }];
    CALayer *layer1 = [CALayer layer];
    layer1.frame = CGRectMake(15, 0, kScreenWidth - 30, 1);
    layer1.backgroundColor = kColorWithHex(@"#F2F2F2").CGColor;
    [matreView.layer addSublayer:layer1];
    CALayer *layer2 = [CALayer layer];
    layer2.frame = CGRectMake(15, 59, kScreenWidth - 30, 1);
    layer2.backgroundColor = kColorWithHex(@"#F2F2F2").CGColor;
    [matreView.layer addSublayer:layer2];
    UITapGestureRecognizer *tapMaterial = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushProjectMaterial)];
    [matreView addGestureRecognizer:tapMaterial];
    
    UILabel *matreL = [JLFactoryUtil labelWithText:@"Material" font:REGULARFONT(15) textColor:kMainTextColor];
    matreL.userInteractionEnabled = YES;
    [matreView addSubview:matreL];
    [matreL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(matreView);
        make.left.equalTo(matreView.mas_left).offset(15);
    }];
    
    UIImageView *matreArrow = [JLFactoryUtil imageViewWithImg:@"更多"];
    matreArrow.userInteractionEnabled = YES;
    [matreView addSubview:matreArrow];
    [matreArrow mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(matreView);
        make.right.equalTo(matreView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    UIView *scheduleView = [JLFactoryUtil view];
    scheduleView.backgroundColor = UIColor.whiteColor;
    [self.contentView addSubview:scheduleView];
    [scheduleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(matreView.mas_bottom).offset(0);
        make.height.mas_equalTo(60);
    }];
    CALayer *layer3 = [CALayer layer];
    layer3.frame = CGRectMake(15, 59, kScreenWidth - 30, 1);
    layer3.backgroundColor = kColorWithHex(@"#F2F2F2").CGColor;
    [scheduleView.layer addSublayer:layer3];
    UITapGestureRecognizer *tapSchedule = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushProjectSchedule)];
    [scheduleView addGestureRecognizer:tapSchedule];
    
    UILabel *scheL = [JLFactoryUtil labelWithText:@"Schedule" font:REGULARFONT(15) textColor:kMainTextColor];
    scheL.userInteractionEnabled = YES;
    [scheduleView addSubview:scheL];
    [scheL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(scheduleView);
        make.left.equalTo(scheduleView.mas_left).offset(15);
    }];
    
    UIImageView *scheArrow = [JLFactoryUtil imageViewWithImg:@"更多"];
    scheArrow.userInteractionEnabled = YES;
    [scheduleView addSubview:scheArrow];
    [scheArrow mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(scheduleView);
        make.right.equalTo(scheduleView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(scheduleView.mas_bottom).offset(20);
    }];
    
}

#pragma mark  UICollectionView Delegate
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {

    if (collectionView == self.picCollectionView) {
        BC_ProjectDetailPicCollectionViewCell *cell = (BC_ProjectDetailPicCollectionViewCell *)[self.picCollectionView cellForItemAtIndexPath:indexPath];
        NSMutableArray *itemarr = [NSMutableArray array];
        for (NSString *url in self.proPicArr) {
            ZlinkPhotoGroupItem *item = [ZlinkPhotoGroupItem new];
            item.thumbView = cell;
            item.largeImageURL = [NSURL URLWithString:KStringIsNull(url)];
            [itemarr addObject:item];
        }
        ZlinkPhotoBrowserView *browser = [[ZlinkPhotoBrowserView alloc] initWithGroupItems:itemarr];
        [browser presentFromImageView:cell toContainer:self.navigationController.view animated:YES index:indexPath.item completion:^{
        }];
    } else if (collectionView == self.contractCollectionView) {
        NSDictionary *dic = self.contractArr[indexPath.item];
        BC_ProjectMaterialViewController *mater = [BC_ProjectMaterialViewController new];
        mater.linkurl = KStringIsNull(dic[@"url"]);
        [self.navigationController pushViewController:mater animated:YES];
    } else {
        NSDictionary *dic = self.agreeArr[indexPath.item];
        BC_ProjectMaterialViewController *mater = [BC_ProjectMaterialViewController new];
        mater.linkurl = KStringIsNull(dic[@"url"]);
        [self.navigationController pushViewController:mater animated:YES];
    }
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    if (collectionView == self.picCollectionView) {
        BC_ProjectDetailPicCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"membercell" forIndexPath:indexPath];
        [cell setPicurl:_proPicArr[indexPath.item]];
        return cell;
    } else if (collectionView == self.contractCollectionView) {
        BC_ProjectInfoFileCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"contractcell" forIndexPath:indexPath];
        [cell setDic:self.contractArr[indexPath.item]];
        return cell;
    } else {
        BC_ProjectInfoFileCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"agreecell" forIndexPath:indexPath];
        [cell setDic:self.agreeArr[indexPath.item]];
        return cell;
    }
    
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if (collectionView == self.picCollectionView) {
        return self.proPicArr.count;
    } else if (collectionView == self.contractCollectionView) {
        return self.contractArr.count;
    } else {
        return self.agreeArr.count;
    }
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

#pragma mark  项目材料
- (void)pushProjectMaterial {
    BC_ProjectMaterialViewController *mater = [BC_ProjectMaterialViewController new];
    mater.contentStr = KStringIsNull(self.infoModel.stuffs);
    [self.navigationController pushViewController:mater animated:YES];
}

#pragma mark  项目进度
- (void)pushProjectSchedule {
    BC_ProjectScheduleViewController *schedu = [BC_ProjectScheduleViewController new];
    schedu.pid = KStringIsNull(self.infoModel.itemid);
    [self.navigationController pushViewController:schedu animated:YES];
}

- (void)getPageData {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/programdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.pid)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dataDic[@"data"];

            self.infoModel = [BC_ExecuteModuleModel mj_objectWithKeyValues:dataDic[@"data"]];
            
            self.areaL.text = kStringFormat(@"%@,%@", KStringIsNull(self.infoModel.city), KStringIsNull(self.infoModel.province));
            CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
            if (size.width >= (kScreenWidth - 70)) {
                size.width = kScreenWidth - 70;
            }
            [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
                make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
            }];
            
            self.cateL.text = KStringIsNull(self.infoModel.service_type);
            CGSize csize = [self.cateL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
            [self.cateL mas_updateConstraints:^(MASConstraintMaker *make) {
                make.size.mas_equalTo(CGSizeMake(24+csize.width, 28));
            }];
            
            self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
            self.nameL.text = KStringIsNull(self.infoModel.name);
            self.descL.text = KStringIsNull(self.infoModel.desc);
            
            if (self.infoModel.images.count > 0) {
                self.proPicArr = [NSMutableArray arrayWithArray:self.infoModel.images];
            }
            if (self.infoModel.contract_files.count > 0) {
                self.contractArr = [NSMutableArray arrayWithArray:self.infoModel.contract_files];
            }
            if (self.infoModel.sideagreement_files.count > 0) {
                self.agreeArr = [NSMutableArray arrayWithArray:self.infoModel.sideagreement_files];
            }
            
            [self.picCollectionView reloadData];
            [self.picCollectionView layoutIfNeeded];
            [self.picCollectionView mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.picCollectionView.collectionViewLayout.collectionViewContentSize.height);
            }];
            
            [self.contractCollectionView reloadData];
            [self.contractCollectionView layoutIfNeeded];
            [self.contractCollectionView mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.contractCollectionView.collectionViewLayout.collectionViewContentSize.height);
            }];
            
            [self.agreementCollectionView reloadData];
            [self.agreementCollectionView layoutIfNeeded];
            [self.agreementCollectionView mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.agreementCollectionView.collectionViewLayout.collectionViewContentSize.height);
            }];
            
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
