//
//  BC_ProjectRateViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/22.
//

#import "BC_ProjectRateViewController.h"
#import "BC_RateSuccessViewController.h"

#import "BC_ProjectRateTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_ProjectRateViewController ()<UITableViewDelegate, UITableViewDataSource, BC_ProjectRateTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIButton *submitBtn;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) BOOL canEdit;
@end

@implementation BC_ProjectRateViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Score";
    self.dataArr = [NSMutableArray array];
    self.canEdit = YES;
    [self setupUI];
    [self getScore];
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - HOME_INDICATOR_HEIGHT - 60)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    
    UIButton *submitBtn = [JLFactoryUtil buttonWithTitle:@"Submit" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
    [submitBtn setCorner:24 withType:JLCornerTypeAllCorner];
    submitBtn.backgroundColor = kMainThemeColor;
    [submitBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
    [submitBtn addTarget:self action:@selector(submitBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:submitBtn];
    [submitBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(30);
        make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT-6);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
    }];
    _submitBtn = submitBtn;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_ProjectRateTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_ProjectRateTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.delegate = self;
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setCanedit:self.canEdit];
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 100;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)starValue:(BC_ExecuteModuleModel *)model {
    KLog(@"%f", model.starvalue);
}

- (void)submitBtnClick {
        
    NSMutableArray *temp = [NSMutableArray array];
    for (BC_ExecuteModuleModel *model in self.dataArr) {
        NSString *score = kStringFormat(@"%.0f", model.starvalue*10/2);
        NSDictionary *dic = @{@"id":KStringIsNull(model.itemid), @"score":KStringIsNull(score)};
        [temp addObject:dic];
    }
    NSString *scoreStr = [KStringIsNull([temp mj_JSONString]) stringByReplacingOccurrencesOfString:@"\\" withString:@""];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/programownuserscore/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(self.pid), @"score":scoreStr} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"refreshProDetailAfterEva" object:nil];
            [self.navigationController pushViewController:[BC_RateSuccessViewController new] animated:YES];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

//- (void)getPeopleList {
//    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
//    NSString *url = [NSString stringWithFormat:@"%@/api/app/getprogramchargeperson/", KMainUrl];
//    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.pid)} success:^(id responseObject) {
//        KLog(@"评分人物成功======%@",responseObject);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//        NSDictionary *resdic = (NSDictionary *)responseObject;
//        if ([resdic[@"code"] integerValue] == 2000) {
//            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
//            self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
//            for (BC_ExecuteModuleModel *m in self.dataArr) {
//                m.starvalue = 1;
//            }
//        }
//        [self.tableView reloadData];
//    } failure:^(NSError *error) {
//        KLog(@"失败======%@",error);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//    }];
//}

- (void)getScore {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/programownuserscore/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.pid)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic1 = (NSDictionary *)resdic[@"data"];
            NSDictionary *dataDic2 = (NSDictionary *)dataDic1[@"data"];
            self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic2[@"score"]];
            if (self.dataArr.count > 0) {
                BC_ExecuteModuleModel *fmodel = self.dataArr[0];
                if ([KStringIsNull(fmodel.score) isEqualToString:@"0"]) {
                    //未评分
                    self.canEdit = YES;
                    self.submitBtn.hidden = NO;
                } else {
                    //已评分
                    self.canEdit = NO;
                    self.submitBtn.hidden = YES;
                }
                for (BC_ExecuteModuleModel *model in self.dataArr) {
                    if ([KStringIsNull(model.score) isEqualToString:@"0"]) {
                        model.starvalue = 1;
                    } else {
                        model.starvalue = [KStringIsNull(model.score) floatValue]*2/10;
                    }
                }
            } else {
                self.canEdit = NO;
                self.submitBtn.hidden = YES;
            }
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
