//
//  BC_ProjectScheduleTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/22.
//

#import "BC_ProjectScheduleTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@implementation BC_ProjectScheduleTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.contentView.backgroundColor = UIColor.whiteColor;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:24 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(20);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_centerX).offset(20);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-2);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(12) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.nicknameL);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
    }];
    
    self.idenL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.idenL];
    [self.idenL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-10);
        make.top.equalTo(self.avatar.mas_centerY).offset(2);
    }];
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"Work content" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [self.contentView addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_left);
        make.top.equalTo(self.avatar.mas_bottom).offset(20);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kLightGrayTextColor];
    self.contentL.numberOfLines = 0;
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(label1.mas_bottom).offset(10);
    }];
    
    UILabel *label2 = [JLFactoryUtil labelWithText:@"Description" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [self.contentView addSubview:label2];
    [label2 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_left);
        make.top.equalTo(self.contentL.mas_bottom).offset(20);
    }];
    
    self.descL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kLightGrayTextColor];
    self.descL.numberOfLines = 0;
    [self.contentView addSubview:self.descL];
    [self.descL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(label2.mas_bottom).offset(10);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(-15);
    }];
    
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"");
    [self.contentView addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.bottom.equalTo(self.contentView.mas_bottom);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
}

- (void)setModel:(BC_ExecuteModuleModel *)model {
    _model = model;
    BC_ExecuteModuleModel *infoModel = [BC_ExecuteModuleModel mj_objectWithKeyValues:model.userinfo];
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(infoModel.avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(infoModel.name);
    if ([KStringIsNull(infoModel.identity) isEqualToString:@"3"]) {
        self.idenL.text = @"Executor";
    } else {
        self.idenL.text = @"-";
    }
    self.contentL.text = KStringIsNull(model.content);
    self.descL.text = KStringIsNull(model.desc);
    self.timeL.text = KStringIsNull(model.create_datetime);
}
@end
