//
//  BC_ExecutorHomePageViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/23.
//

#import "BC_ExecutorHomePageViewController.h"
#import "BC_SearchViewController.h"
#import "BC_NoticeListViewController.h"
#import "BC_ExecutorProjectDetailViewController.h"
#import "BC_NoticeDetailViewController.h"
#import "BC_BannerDetailViewController.h"

#import "BC_ExecutorHomeHeaderView.h"
#import "JXPagerView.h"
#import "JXPagerListRefreshView.h"
#import <JXCategoryView.h>
#import "BC_ExecutorHomeListView.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_ExecutorHomePageViewController ()<JXPagerViewDelegate, JXCategoryViewDelegate, BC_ExecutorHomeHeaderViewDelegate, BC_ExecutorHomeListViewDelegate>
@property (nonatomic, assign) CGFloat JXTableHeaderViewHeight;
@property (nonatomic, strong) BC_ExecutorHomeHeaderView *userHeaderView;
//@property (nonatomic, strong) JXPagerView *pagingView;
@property (nonatomic, strong) JXPagerListRefreshView *pagingView;
@property (nonatomic, strong) JXCategoryNumberView *categoryView;

@end

static const CGFloat JXheightForHeaderInSection = 60;

@implementation BC_ExecutorHomePageViewController
- (void)dealloc {
    [kNotificationCenter removeObserver:self];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.userHeaderView headerRefreshData];
    [self getProNum];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    self.JXTableHeaderViewHeight = (kScreenWidth - 32)*160/340+90;
    [kNotificationCenter addObserver:self selector:@selector(getProNum) name:@"refreshCateNum" object:nil];
    [self setupUI];
}

- (void)setupUI {
    UIView *topView = [JLFactoryUtil view];
    topView.backgroundColor = UIColor.whiteColor;
    topView.frame = CGRectMake(0, 0, kScreenWidth, kTopHeight);
    [self.view addSubview:topView];
    
    UIView *searchView = [JLFactoryUtil view];
    searchView.backgroundColor = kColorWithHex(@"#F7F7F7");
    [searchView setCorner:12 withType:JLCornerTypeAllCorner];
    [topView addSubview:searchView];
    [searchView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(topView.mas_left).offset(16);
        make.bottom.equalTo(topView.mas_bottom).offset(-2);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 44));
    }];
    UITapGestureRecognizer *tapSearch = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushSearchVC)];
    [searchView addGestureRecognizer:tapSearch];
    
    UILabel *sLabel = [JLFactoryUtil labelWithText:@"Search" font:REGULARFONT(14) textColor:kColorWithHex(@"#AAAAAA")];
    sLabel.userInteractionEnabled = YES;
    [searchView addSubview:sLabel];
    [sLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.left.equalTo(searchView.mas_left).offset(20);
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [searchView addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.right.equalTo(searchView.mas_right).offset(-20);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    _userHeaderView = [[BC_ExecutorHomeHeaderView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, self.JXTableHeaderViewHeight)];
    _userHeaderView.delegate = self;
    
    self.categoryView = [[JXCategoryNumberView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, 60)];
    self.categoryView.delegate = self;
    self.categoryView.backgroundColor = UIColor.clearColor;
    self.categoryView.cellSpacing = 30;
    self.categoryView.titleColor = kMainTextColor;
    self.categoryView.titleSelectedColor = kMainTextColor;
    self.categoryView.titleFont = REGULARFONT(17);
    self.categoryView.titleSelectedFont = MEDIUMFONT(17);
    self.categoryView.defaultSelectedIndex = 0;
    self.categoryView.titleColorGradientEnabled = YES;
    self.categoryView.averageCellSpacingEnabled = NO;
    self.categoryView.titles = @[@"New task", @"Assign Renew"];
    self.categoryView.numberLabelFont = MEDIUMFONT(12);
    self.categoryView.numberTitleColor = UIColor.whiteColor;
    self.categoryView.numberBackgroundColor = kMainThemeColor;
    self.categoryView.shouldMakeRoundWhenSingleNumber = YES;
    
    _pagingView = [[JXPagerListRefreshView alloc] initWithDelegate:self listContainerType:JXPagerListContainerType_ScrollView];
    _pagingView.frame = CGRectMake(0, kTopHeight, kScreenWidth, kScreenHeight - kTabBarHeight - kTopHeight);
    [self.view addSubview:_pagingView];
    self.categoryView.listContainer = (id<JXCategoryViewListContainer>)self.pagingView.listContainerView;
    self.pagingView.pinSectionHeaderVerticalOffset = 0;
    self.pagingView.automaticallyDisplayListVerticalScrollIndicator = NO;
    if (@available(iOS 15.0, *)) {
        self.pagingView.mainTableView.sectionHeaderTopPadding = 0;
    } else {
        // Fallback on earlier versions
    }
    self.pagingView.mainTableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    self.pagingView.listContainerView.scrollView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
}

#pragma mark - JXPagingViewDelegate
- (UIView *)tableHeaderViewInPagerView:(JXPagerView *)pagerView {
    return self.userHeaderView;
}

- (NSUInteger)tableHeaderViewHeightInPagerView:(JXPagerView *)pagerView {
    return self.JXTableHeaderViewHeight;
}

- (NSUInteger)heightForPinSectionHeaderInPagerView:(JXPagerView *)pagerView {
    return JXheightForHeaderInSection;
}

- (UIView *)viewForPinSectionHeaderInPagerView:(JXPagerView *)pagerView {
    return self.categoryView;
}

- (NSInteger)numberOfListsInPagerView:(JXPagerView *)pagerView {
    return 2;
}

- (id<JXPagerViewListViewDelegate>)pagerView:(JXPagerView *)pagerView initListAtIndex:(NSInteger)index {
    BC_ExecutorHomeListView *list = [[BC_ExecutorHomeListView alloc] initWithIndex:index];
    list.delegate = self;
    return list;
}

- (void)mainTableViewDidScroll:(UIScrollView *)scrollView {
    [self.userHeaderView scrollViewDidScroll:scrollView.contentOffset.y];
}

#pragma mark - JXCategoryViewDelegate
- (void)categoryView:(JXCategoryBaseView *)categoryView didSelectedItemAtIndex:(NSInteger)index {
    [self.pagingView.listContainerView didClickSelectedItemAtIndex:index];
}

- (void)pushSearchVC {
    [self.navigationController pushViewController:[BC_SearchViewController new] animated:NO];
}

- (void)moreNoticeClick {
    [self.navigationController pushViewController:[BC_NoticeListViewController new] animated:YES];
}

- (void)clickNoticeItem:(NSDictionary *)dic {
    BC_NoticeDetailViewController *detail = [BC_NoticeDetailViewController new];
    detail.titleStr = KStringIsNull(dic[@"title"]);
    detail.timeStr = KStringIsNull(dic[@"create_datetime"]);
    detail.contentStr = KStringIsNull(dic[@"content"]);
    [self.navigationController pushViewController:detail animated:YES];
}

- (void)clickBannerItem:(NSDictionary *)dic {
    if ([KStringIsNull(dic[@"link"]) isEqualToString:@""]) return;
    BC_BannerDetailViewController *detail = [BC_BannerDetailViewController new];
    detail.url = KStringIsNull(dic[@"link"]);
    [self.navigationController pushViewController:detail animated:YES];
}

- (void)projectItemClick:(BC_ExecuteModuleModel *)model index:(NSInteger)index {
    BC_ExecutorProjectDetailViewController *detial = [BC_ExecutorProjectDetailViewController new];
    detial.isFromHome = YES;
    if (index == 0) {
        detial.isFromGrab = YES;
    } else {
        detial.isFromGrab = NO;
    }
    detial.pid = KStringIsNull(model.itemid);
    [self.navigationController pushViewController:detial animated:YES];
}

- (void)execuShield:(BC_ExecuteModuleModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_ExecuteModuleModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_ExecuteModuleModel *)model {
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"2"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.pagingView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
    
}

- (void)getProNum {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getnewprogrammessage/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{} success:^(id responseObject) {
        KLog(@"未读成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *dataArr = (NSArray *)dataDic[@"data"];
            if (dataArr.count == 0) {
                self.categoryView.counts = @[@(0), @(0)];
            } else {
                NSInteger num = dataArr.count;
                self.categoryView.counts = @[@(num), @(0)];
            }
            [self.categoryView reloadDataWithoutListContainer];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}

@end
