//
//  BC_ExecutorHomeHeaderView.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/23.
//

#import "BC_ExecutorHomeHeaderView.h"
#import "SDCycleScrollView.h"
#import "UUMarqueeView.h"

@interface BC_ExecutorHomeHeaderView ()<SDCycleScrollViewDelegate, UUMarqueeViewDelegate>
@property (nonatomic, strong) SDCycleScrollView *bannerScrollView;
@property (nonatomic, strong) UUMarqueeView *marqueeView;

@property (nonatomic, strong) NSMutableArray *noticeArr;
@property (nonatomic, strong) NSMutableArray *bannerArr;
@end

@implementation BC_ExecutorHomeHeaderView
- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.noticeArr = [NSMutableArray array];
        self.bannerArr = [NSMutableArray array];
        [self setupUI];
    }
    return self;
}

- (void)headerRefreshData {
    [self getHomeNotice];
    [self getHomeBanner];
}

- (void)setupUI {
    self.bannerScrollView= [SDCycleScrollView cycleScrollViewWithFrame:CGRectMake(16, 15, kScreenWidth - 32, (kScreenWidth - 32)*160/340) delegate:self placeholderImage:nil];
    self.bannerScrollView.backgroundColor = UIColor.whiteColor;
    self.bannerScrollView.pageControlDotSize = CGSizeMake(5, 5);
    self.bannerScrollView.titlesGroup = nil;
    self.bannerScrollView.currentPageDotColor = kMainThemeColor;
    self.bannerScrollView.pageDotColor = UIColor.whiteColor;
    self.bannerScrollView.pageControlAliment = SDCycleScrollViewPageContolAlimentCenter;
    self.bannerScrollView.showPageControl = YES;
    self.bannerScrollView.pageControlStyle = SDCycleScrollViewPageContolStyleAnimated;
    self.bannerScrollView.autoScrollTimeInterval = 4;
    self.bannerScrollView.pageControlBottomOffset = 10;
    self.bannerScrollView.pageControlAliment = SDCycleScrollViewPageContolAlimentRight;
    self.bannerScrollView.bannerImageViewContentMode = UIViewContentModeScaleAspectFill;
    self.bannerScrollView.layer.masksToBounds = YES;
    self.bannerScrollView.layer.cornerRadius = 14;
    [self addSubview:self.bannerScrollView];
    
    UIView *notiView = [JLFactoryUtil view];
    notiView.backgroundColor = kColorWithHex(@"#FFF5F6");
    [notiView setCorner:14 withType:JLCornerTypeAllCorner];
    [self addSubview:notiView];
    [notiView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.mas_left).offset(16);
        make.top.equalTo(self.bannerScrollView.mas_bottom).offset(25);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 30));
    }];
    
    self.marqueeView = [[UUMarqueeView alloc] initWithFrame:CGRectMake(42, 3, kScreenWidth - 134, 24)];
    self.marqueeView.delegate = self;
    self.marqueeView.timeIntervalPerScroll = 2.0f;    // 条目滑动间隔
    self.marqueeView.timeDurationPerScroll = 0.5;    // 条目滑动时间
    self.marqueeView.touchEnabled = YES;    // 设置为YES可监听点击事件，默认值为NO
    [notiView addSubview:self.marqueeView];
    [self.marqueeView reloadData];
    
    UIImageView *notiImg = [JLFactoryUtil imageViewWithImg:@"通知"];
    [notiView addSubview:notiImg];
    [notiImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(notiView);
        make.left.equalTo(notiView.mas_left).offset(6);
        make.size.mas_equalTo(CGSizeMake(30, 28));
    }];
    
    UIButton *moreBtn = [JLFactoryUtil buttonWithTitle:@"More" font:MEDIUMFONT(14) textColor:kMainThemeColor];
    [moreBtn addTarget:self action:@selector(moreNotiClick) forControlEvents:UIControlEventTouchUpInside];
    [notiView addSubview:moreBtn];
    [moreBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(notiView);
        make.right.equalTo(notiView.mas_right).offset(0);
        make.size.mas_equalTo(CGSizeMake(60, 22));
    }];
    
}

- (void)cycleScrollView:(SDCycleScrollView *)cycleScrollView didSelectItemAtIndex:(NSInteger)index {
    if (self.bannerArr.count > 0) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(clickBannerItem:)]) {
            [self.delegate clickBannerItem:self.bannerArr[index]];
        }
    }
}

- (void)moreNotiClick {
    if (self.delegate && [self.delegate respondsToSelector:@selector(moreNoticeClick)]) {
        [self.delegate moreNoticeClick];
    }
}

- (void)scrollViewDidScroll:(CGFloat)contentOffsetY {
    
}

- (NSUInteger)numberOfDataForMarqueeView:(UUMarqueeView *)marqueeView {
    return self.noticeArr.count;
}

- (NSUInteger)numberOfVisibleItemsForMarqueeView:(UUMarqueeView *)marqueeView {
    return 1;
}

- (void)createItemView:(UIView *)itemView forMarqueeView:(UUMarqueeView *)marqueeView {
    UILabel *content = [[UILabel alloc] initWithFrame:itemView.bounds];
    content.font = REGULARFONT(14);
    content.textColor = kColorWithHex(@"#333333");
    content.tag = 1001;
    [itemView addSubview:content];
}

- (void)updateItemView:(UIView *)itemView atIndex:(NSUInteger)index forMarqueeView:(UUMarqueeView *)marqueeView {
    if (self.noticeArr.count > 0) {
        NSDictionary *dic = self.noticeArr[index];
        UILabel *content = [itemView viewWithTag:1001];
        content.text = KStringIsNull(dic[@"title"]);
    } else {
        UILabel *content = [itemView viewWithTag:1001];
        content.text = @"";
    }
}

- (void)didTouchItemViewAtIndex:(NSUInteger)index forMarqueeView:(UUMarqueeView *)marqueeView {
    if (self.noticeArr.count > 0) {
        if (self.delegate && [self.delegate respondsToSelector:@selector(clickNoticeItem:)]) {
            [self.delegate clickNoticeItem:self.noticeArr[index]];
        }
    }
}

- (void)getHomeNotice {
    [self.noticeArr removeAllObjects];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/message2listdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@"1", @"limit":@"10"} success:^(id responseObject) {
        KLog(@"头条成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            [self.noticeArr addObjectsFromArray:arr];
            [self.marqueeView reloadData];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}

- (void)getHomeBanner {
    [self.bannerArr removeAllObjects];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getrotationimgs/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"type":@"1"} success:^(id responseObject) {
        KLog(@"banner成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            [self.bannerArr addObjectsFromArray:arr];
            NSMutableArray *temparr = [NSMutableArray array];
            for (NSDictionary *dic in arr) {
                [temparr addObject:KStringIsNull(dic[@"image"])];
            }
            self.bannerScrollView.imageURLStringsGroup = temparr;
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}

@end
