//
//  BC_ExecutorHomeListView.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/23.
//

#import "BC_ExecutorHomeListView.h"
#import "BC_ExecutorHomeListTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_ExecutorHomeListView ()<UITableViewDelegate, UITableViewDataSource, BC_ExecutorHomeListTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, copy) void(^scrollCallback)(UIScrollView *scrollView);
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@end

@implementation BC_ExecutorHomeListView
- (id)initWithIndex:(NSInteger)index {
    if (self = [super init]) {
        _index = index;
        self.dataArr = [NSMutableArray array];
        self.pageNum = 1;
        [self setupUI];
        [self loadData];
    }
    return self;
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - kTabBarHeight - 60)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    [self addSubview:self.tableView];
    [self.tableView registerClass:[BC_ExecutorHomeListTableViewCell class] forCellReuseIdentifier:@"cellid"];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;

    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [kNotificationCenter postNotificationName:@"refreshCateNum" object:nil];
    [self getMyItemList];
}

- (void)loadMoreData {
    self.pageNum ++;
    [self getMyItemList];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_ExecutorHomeListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_ExecutorHomeListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [cell setPageIndex:self.index];
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_ExecutorHomeListTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"cellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if (self.delegate && [self.delegate respondsToSelector:@selector(projectItemClick:index:)]) {
        [self.delegate projectItemClick:self.dataArr[indexPath.row] index:_index];
    }
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_ExecuteModuleModel *)model {
    if (self.delegate && [self.delegate respondsToSelector:@selector(execuShield:)]) {
        [self.delegate execuShield:model];
    }
    
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    self.scrollCallback(scrollView);
}

#pragma mark - JXPagingViewListViewDelegate

- (UIScrollView *)listScrollView {
    return self.tableView;
}

- (void)listViewDidScrollCallback:(void (^)(UIScrollView *))callback {
    self.scrollCallback = callback;
}

- (UIView *)listView {
    return self;
}


- (void)getMyItemList {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/programglist2/", KMainUrl];
    NSString *status = @"";
    if (_index == 0) {
        status = @"1";
    } else {
        status = @"2";
    }
    KLog(@"status===%@", status);
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10", @"type":status} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}

@end
