//
//  BC_ExecutorProjectSubListViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/24.
//

#import "BC_ExecutorProjectSubListViewController.h"
#import "BC_ExecutorProjectDetailViewController.h"
#import "BC_ExectorAddRateViewController.h"
#import "BC_ExectorProjectListTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_ExecutorProjectSubListViewController ()<UITableViewDelegate, UITableViewDataSource, DZNEmptyDataSetSource, DZNEmptyDataSetDelegate, BC_ExectorProjectListTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;

@end

@implementation BC_ExecutorProjectSubListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupListView];
    [self loadData];
}

- (void)setupListView {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - 50 - kTabBarHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.backgroundColor = UIColor.whiteColor;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_ExectorProjectListTableViewCell class] forCellReuseIdentifier:@"cellid"];
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_ExectorProjectListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_ExectorProjectListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_ExectorProjectListTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    cell.pageIndex = self.index;
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"cellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    //执行者
    BC_ExecuteModuleModel *model = _dataArr[indexPath.row];
    BC_ExecutorProjectDetailViewController *detail = [BC_ExecutorProjectDetailViewController new];
    detail.isFromHome = NO;
    detail.pid = KStringIsNull(model.itemid);
    [self.navigationController pushViewController:detail animated:YES];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    return [UIImage imageNamed:@"空数据"];
}

- (BOOL)emptyDataSetShouldAllowScroll:(UIScrollView *)scrollView {
    return YES;
}

#pragma mark - JXCategoryListContentViewDelegate
- (UIView *)listView {
    return self.view;
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_ExecuteModuleModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_ExecuteModuleModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_ExecuteModuleModel *)model {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"2"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.dataArr removeObject:model];
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

#pragma mark  完成项目
- (void)finishTheProject {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Confirm to complete the project" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmCompleteAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)confirmCompleteAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)cancelAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

#pragma mark  去评价
- (void)goToRateTheProject {
    BC_ExectorAddRateViewController *rate = [BC_ExectorAddRateViewController new];
    [self.navigationController pushViewController:rate animated:YES];
}

- (void)getPageData {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getmyprogramlistdetail/", KMainUrl];
    NSString *status = @"";
    if (_index == 0) {
        status = @"20";
    } else if (_index == 1) {
        status = @"30";
    } else {
        status = @"40";
    }
    KLog(@"status===%@", status);
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10", @"status":status} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}

@end
