//
//  BC_HomePageViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/13.
//

#import "BC_HomePageViewController.h"
#import "BC_SearchViewController.h"
#import "BC_NoticeListViewController.h"
#import "BC_PostDetailViewController.h"
#import "BC_VideoPostDetailViewController.h"
#import "BC_CreatePostViewController.h"
#import "BC_ManageMyGroupViewController.h"
#import "BC_NoticeDetailViewController.h"
#import "BC_BannerDetailViewController.h"

#import "BC_PostItemListTableViewCell.h"
#import "BC_PostItemListHasPicTableViewCell.h"
#import "BC_PostItemListHasVideoTableViewCell.h"

#import "SDCycleScrollView.h"
#import "UUMarqueeView.h"
#import "BC_PostListModel.h"

@import ImSDK_Plus;

@interface BC_HomePageViewController ()<UITableViewDelegate, UITableViewDataSource, SDCycleScrollViewDelegate, UUMarqueeViewDelegate, BC_PostItemListTableViewCellDelegate, BC_PostItemListHasPicTableViewCellDelegate, BC_PostItemListHasVideoTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) SDCycleScrollView *bannerScrollView;
@property (nonatomic, strong) UUMarqueeView *marqueeView;
@property (nonatomic, strong) UIButton *pubBtn;
@property (nonatomic, strong) UIImageView *pubPostBgImg;

@property (nonatomic, strong) NSMutableArray *noticeArr;
@property (nonatomic, strong) NSMutableArray *bannerArr;
@property (nonatomic, assign) NSInteger pageNum;
@property (nonatomic, strong) NSMutableArray *dataArr;

@end

@implementation BC_HomePageViewController

- (void)dealloc {
    KLog(@"homepage dealloc");
    [kNotificationCenter removeObserver:self];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self.bannerScrollView adjustWhenControllerViewWillAppera];
    [self getHomeNotice];
    [self getHomeBanner];
    [self getUserInfo];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    self.dataArr = [NSMutableArray array];
    self.noticeArr = [NSMutableArray array];
    self.bannerArr = [NSMutableArray array];
    self.pageNum = 1;
    [kNotificationCenter addObserver:self selector:@selector(loadData) name:@"refreshAfterDel" object:nil];
    [self setupUI];
    [self loadData];
        
}

- (void)setupUI {
    
    UIView *topView = [JLFactoryUtil view];
    topView.backgroundColor = UIColor.whiteColor;
    topView.frame = CGRectMake(0, 0, kScreenWidth, kTopHeight);
    [self.view addSubview:topView];
    
    UIView *searchView = [JLFactoryUtil view];
    searchView.backgroundColor = kColorWithHex(@"#F7F7F7");
    [searchView setCorner:12 withType:JLCornerTypeAllCorner];
    [topView addSubview:searchView];
    [searchView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(topView.mas_left).offset(16);
        make.bottom.equalTo(topView.mas_bottom).offset(-2);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 44));
    }];
    UITapGestureRecognizer *tapSearch = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushSearchVC)];
    [searchView addGestureRecognizer:tapSearch];
    
    UILabel *sLabel = [JLFactoryUtil labelWithText:@"Search" font:REGULARFONT(14) textColor:kColorWithHex(@"#AAAAAA")];
    sLabel.userInteractionEnabled = YES;
    [searchView addSubview:sLabel];
    [sLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.left.equalTo(searchView.mas_left).offset(20);
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [searchView addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.right.equalTo(searchView.mas_right).offset(-20);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, kTopHeight, kScreenWidth, kScreenHeight - kTabBarHeight - kTopHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_PostItemListTableViewCell class] forCellReuseIdentifier:@"postcell"];
    [self.tableView registerClass:[BC_PostItemListHasPicTableViewCell class] forCellReuseIdentifier:@"postpiccell"];
    [self.tableView registerClass:[BC_PostItemListHasVideoTableViewCell class] forCellReuseIdentifier:@"postvideocell"];
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
    
    CGFloat bannerH = (kScreenWidth - 32)*160/340;
    
    UIView *header = [[UIView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, bannerH + 90)];
    self.tableView.tableHeaderView = header;
    
    self.bannerScrollView= [SDCycleScrollView cycleScrollViewWithFrame:CGRectMake(16, 15, kScreenWidth - 32, bannerH) delegate:self placeholderImage:nil];
    self.bannerScrollView.backgroundColor = UIColor.whiteColor;
    self.bannerScrollView.pageControlDotSize = CGSizeMake(5, 5);
    self.bannerScrollView.titlesGroup = nil;
    self.bannerScrollView.currentPageDotColor = kMainThemeColor;
    self.bannerScrollView.pageDotColor = UIColor.whiteColor;
    self.bannerScrollView.pageControlAliment = SDCycleScrollViewPageContolAlimentCenter;
    self.bannerScrollView.showPageControl = YES;
    self.bannerScrollView.pageControlStyle = SDCycleScrollViewPageContolStyleAnimated;
    self.bannerScrollView.autoScrollTimeInterval = 4;
    self.bannerScrollView.pageControlBottomOffset = 10;
    self.bannerScrollView.pageControlAliment = SDCycleScrollViewPageContolAlimentRight;
    self.bannerScrollView.bannerImageViewContentMode = UIViewContentModeScaleAspectFill;
    self.bannerScrollView.layer.masksToBounds = YES;
    self.bannerScrollView.layer.cornerRadius = 14;
    [header addSubview:self.bannerScrollView];
    
    UIView *notiView = [JLFactoryUtil view];
    notiView.backgroundColor = kColorWithHex(@"#FFF5F6");
    [notiView setCorner:14 withType:JLCornerTypeAllCorner];
    [header addSubview:notiView];
    [notiView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(header.mas_left).offset(16);
        make.top.equalTo(self.bannerScrollView.mas_bottom).offset(25);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 30));
    }];
    
    self.marqueeView = [[UUMarqueeView alloc] initWithFrame:CGRectMake(42, 3, kScreenWidth - 134, 24)];
    self.marqueeView.delegate = self;
    self.marqueeView.timeIntervalPerScroll = 2.0f;    // 条目滑动间隔
    self.marqueeView.timeDurationPerScroll = 0.5;    // 条目滑动时间
    self.marqueeView.touchEnabled = YES;    // 设置为YES可监听点击事件，默认值为NO
    [notiView addSubview:self.marqueeView];
    
    UIImageView *notiImg = [JLFactoryUtil imageViewWithImg:@"通知"];
    [notiView addSubview:notiImg];
    [notiImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(notiView);
        make.left.equalTo(notiView.mas_left).offset(6);
        make.size.mas_equalTo(CGSizeMake(30, 28));
    }];
    
    UIButton *moreBtn = [JLFactoryUtil buttonWithTitle:@"More" font:MEDIUMFONT(14) textColor:kMainThemeColor];
    [moreBtn addTarget:self action:@selector(moreNotiClick) forControlEvents:UIControlEventTouchUpInside];
    [notiView addSubview:moreBtn];
    [moreBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(notiView);
        make.right.equalTo(notiView.mas_right).offset(0);
        make.size.mas_equalTo(CGSizeMake(60, 22));
    }];
    
    UIButton *pubBtn = [JLFactoryUtil buttonWithImage:@"发布帖子"];
    [pubBtn addTarget:self action:@selector(publishPostClick:) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:pubBtn];
    [pubBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.view.mas_right).offset(-5);
        make.bottom.equalTo(self.view.mas_bottom).offset(-kTabBarHeight - 150);
        make.size.mas_equalTo(CGSizeMake(90, 90));
    }];
    _pubBtn = pubBtn;
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    BC_PostListModel *model = self.dataArr[indexPath.row];
    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        //视频
        static NSString *cellid = @"postvideocell";
        BC_PostItemListHasVideoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
        if (!cell) {
            cell = [[BC_PostItemListHasVideoTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
        }
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.delegate = self;
        [self configVideoCell:cell andIndexP:indexPath];
        return cell;
    } else {
        if (model.files.count == 0) {
            //无图
            static NSString *cellid = @"postcell";
            BC_PostItemListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
            if (!cell) {
                cell = [[BC_PostItemListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
            }
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            cell.delegate = self;
            [self configCell:cell andIndexP:indexPath];
            return cell;
        } else {
            static NSString *cellid = @"postpiccell";
            BC_PostItemListHasPicTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
            if (!cell) {
                cell = [[BC_PostItemListHasPicTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
            }
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            cell.delegate = self;
            [self configPicCell:cell andIndexP:indexPath];
            return cell;
        }
    }
}

- (void)configCell:(BC_PostItemListTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (void)configPicCell:(BC_PostItemListHasPicTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (void)configVideoCell:(BC_PostItemListHasVideoTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    BC_PostListModel *model = self.dataArr[indexPath.row];
    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        //视频
        return [tableView fd_heightForCellWithIdentifier:@"postvideocell" cacheByIndexPath:indexPath configuration:^(id cell) {
            [self configVideoCell:cell andIndexP:indexPath];
        }];
    } else {
        if (model.files.count == 0) {
            //无图
            return [tableView fd_heightForCellWithIdentifier:@"postcell" cacheByIndexPath:indexPath configuration:^(id cell) {
                [self configCell:cell andIndexP:indexPath];
            }];
        } else {
            return [tableView fd_heightForCellWithIdentifier:@"postpiccell" cacheByIndexPath:indexPath configuration:^(id cell) {
                [self configPicCell:cell andIndexP:indexPath];
            }];
        }
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    UIView *header = [[UIView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, 55)];
    header.backgroundColor = UIColor.whiteColor;
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"COMUNITI" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [header addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(header);
        make.left.equalTo(header.mas_left).offset(16);
    }];
        
    return header;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    BC_PostListModel *model = self.dataArr[indexPath.row];
    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        BC_VideoPostDetailViewController *detail = [BC_VideoPostDetailViewController new];
        detail.postid = KStringIsNull(model.itemid);
        [self.navigationController pushViewController:detail animated:YES];
    } else {
        BC_PostDetailViewController *detail = [BC_PostDetailViewController new];
        detail.postid = KStringIsNull(model.itemid);
        [self.navigationController pushViewController:detail animated:YES];
    }
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)shieldPicItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)shieldVideoItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_PostListModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_PostListModel *)model {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"1"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.dataArr removeObject:model];
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    if (self.pubPostBgImg) {
        [self hidePopView];
    }
}

- (NSUInteger)numberOfDataForMarqueeView:(UUMarqueeView *)marqueeView {
    return self.noticeArr.count;
}

- (NSUInteger)numberOfVisibleItemsForMarqueeView:(UUMarqueeView *)marqueeView {
    return 1;
}

- (void)createItemView:(UIView *)itemView forMarqueeView:(UUMarqueeView *)marqueeView {
    UILabel *content = [[UILabel alloc] initWithFrame:itemView.bounds];
    content.font = REGULARFONT(14);
    content.textColor = kColorWithHex(@"#333333");
    content.tag = 1001;
    [itemView addSubview:content];
}

- (void)updateItemView:(UIView *)itemView atIndex:(NSUInteger)index forMarqueeView:(UUMarqueeView *)marqueeView {
    if (self.noticeArr.count > 0) {
        NSDictionary *dic = self.noticeArr[index];
        UILabel *content = [itemView viewWithTag:1001];
        content.text = KStringIsNull(dic[@"title"]);
    } else {
        UILabel *content = [itemView viewWithTag:1001];
        content.text = @"";
    }
}

- (void)didTouchItemViewAtIndex:(NSUInteger)index forMarqueeView:(UUMarqueeView *)marqueeView {
    if (self.noticeArr.count > 0) {
        NSDictionary *dic = self.noticeArr[index];
        BC_NoticeDetailViewController *detail = [BC_NoticeDetailViewController new];
        detail.titleStr = KStringIsNull(dic[@"title"]);
        detail.timeStr = KStringIsNull(dic[@"create_datetime"]);
        detail.contentStr = KStringIsNull(dic[@"content"]);
        [self.navigationController pushViewController:detail animated:YES];
    }
}

- (void)cycleScrollView:(SDCycleScrollView *)cycleScrollView didSelectItemAtIndex:(NSInteger)index {
    NSDictionary *dic = self.bannerArr[index];
    if ([KStringIsNull(dic[@"link"]) isEqualToString:@""]) return;
    BC_BannerDetailViewController *detail = [BC_BannerDetailViewController new];
    detail.url = KStringIsNull(dic[@"link"]);
    [self.navigationController pushViewController:detail animated:YES];
}

#pragma mark  Action
#pragma mark  头条
- (void)moreNotiClick {
    [self.navigationController pushViewController:[BC_NoticeListViewController new] animated:YES];
}

#pragma mark  发布帖子
- (void)publishPostClick:(UIButton *)sender {
    sender.selected = !sender.selected;
    if (sender.selected) {
        self.pubPostBgImg = [JLFactoryUtil imageViewWithImg:@"发帖背景"];
        self.pubPostBgImg.userInteractionEnabled = YES;
        [self.view addSubview:self.pubPostBgImg];
        [self.pubPostBgImg mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.pubBtn.mas_bottom).offset(-20);
            make.right.equalTo(self.view.mas_right).offset(-15);
            make.size.mas_equalTo(CGSizeMake(122, 140));
        }];
        
        UIView *item1 = [JLFactoryUtil view];
        [self.pubPostBgImg addSubview:item1];
        [item1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.equalTo(self.pubPostBgImg);
            make.bottom.equalTo(self.pubPostBgImg.mas_centerY).offset(-1);
            make.size.mas_equalTo(CGSizeMake(80, 45));
        }];
        UITapGestureRecognizer *tapItem1 = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(publishPicPost)];
        [item1 addGestureRecognizer:tapItem1];

        UIImageView *img1 = [JLFactoryUtil imageViewWithImg:@"发帖图片"];
        img1.userInteractionEnabled = YES;
        [item1 addSubview:img1];
        [img1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(item1);
            make.left.equalTo(item1.mas_left).offset(1);
            make.size.mas_equalTo(CGSizeMake(18, 18));
        }];
        
        UILabel *label1 = [JLFactoryUtil labelWithText:@"Picture" font:REGULARFONT(16) textColor:kMainTextColor];
        label1.userInteractionEnabled = YES;
        [item1 addSubview:label1];
        [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(item1);
            make.left.equalTo(img1.mas_right).offset(6);
        }];
        
        UIView *item2 = [JLFactoryUtil view];
        [self.pubPostBgImg addSubview:item2];
        [item2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.equalTo(self.pubPostBgImg);
            make.top.equalTo(self.pubPostBgImg.mas_centerY).offset(1);
            make.size.mas_equalTo(CGSizeMake(80, 45));
        }];
        UITapGestureRecognizer *tapItem2 = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(publishVideoPost)];
        [item2 addGestureRecognizer:tapItem2];
        
        UIImageView *img2 = [JLFactoryUtil imageViewWithImg:@"发帖视频"];
        img2.userInteractionEnabled = YES;
        [item2 addSubview:img2];
        [img2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(item2);
            make.left.equalTo(item2.mas_left).offset(1);
            make.size.mas_equalTo(CGSizeMake(18, 18));
        }];
        
        UILabel *label2 = [JLFactoryUtil labelWithText:@"Video" font:REGULARFONT(16) textColor:kMainTextColor];
        label2.userInteractionEnabled = YES;
        [item2 addSubview:label2];
        [label2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(item2);
            make.left.equalTo(img2.mas_right).offset(6);
        }];
        
        @weakify(self);
        self.pubPostBgImg.alpha = 0;
        [UIView animateWithDuration:0.15 delay:0 options:UIViewAnimationOptionCurveEaseIn animations:^{
            @strongify(self);
            self.pubPostBgImg.alpha = 1;
        } completion:nil];
    } else {
        [self hidePopView];
    }
}

- (void)hidePopView {
    self.pubBtn.selected = NO;
    @weakify(self);
    [UIView animateWithDuration:0.15 delay:0 options:UIViewAnimationOptionCurveEaseOut animations:^{
        @strongify(self);
        self.pubPostBgImg.alpha = 0;
    } completion:^(BOOL finished) {
        @strongify(self);
        [self.pubPostBgImg removeFromSuperview];
        self.pubPostBgImg = nil;
    }];
}

#pragma mark  视频帖子
- (void)publishVideoPost {
    [self hidePopView];
    BC_CreatePostViewController *cpost = [BC_CreatePostViewController new];
    cpost.type = @"2";
    BaseNavigationViewController *nav = [[BaseNavigationViewController alloc] initWithRootViewController:cpost];
    nav.modalPresentationStyle = UIModalPresentationFullScreen;
    [self.navigationController presentViewController:nav animated:YES completion:nil];
}

#pragma mark  图片帖子
- (void)publishPicPost {
    [self hidePopView];
    BC_CreatePostViewController *cpost = [BC_CreatePostViewController new];
    cpost.type = @"1";
    BaseNavigationViewController *nav = [[BaseNavigationViewController alloc] initWithRootViewController:cpost];
    nav.modalPresentationStyle = UIModalPresentationFullScreen;
    [self.navigationController presentViewController:nav animated:YES completion:nil];
}

- (void)pushSearchVC {
    [self.navigationController pushViewController:[BC_SearchViewController new] animated:NO];
}

- (void)getHomeNotice {
    [self.noticeArr removeAllObjects];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/message2listdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@"1", @"limit":@"10"} success:^(id responseObject) {
        KLog(@"头条成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            [self.noticeArr addObjectsFromArray:arr];
            [self.marqueeView reloadData];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}

- (void)getHomeBanner {
    [self.bannerArr removeAllObjects];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getrotationimgs/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"type":@"1"} success:^(id responseObject) {
        KLog(@"banner成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            [self.bannerArr addObjectsFromArray:arr];
            NSMutableArray *temparr = [NSMutableArray array];
            for (NSDictionary *dic in arr) {
                [temparr addObject:KStringIsNull(dic[@"image"])];
            }
            self.bannerScrollView.imageURLStringsGroup = temparr;
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPostList];
}

- (void)loadMoreData {
    self.pageNum ++;
    [self getPostList];
}

- (void)getPostList {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/posts/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10"} success:^(id responseObject) {
        KLog(@"帖子成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
    }];
}

- (void)getUserInfo {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getmyowninfo/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dataDic[@"data"];
            V2TIMUserFullInfo *info = [[V2TIMUserFullInfo alloc] init];
            info.faceURL = KStringIsNull(infoDic[@"user_avatar"]);
            info.nickName = KStringIsNull(infoDic[@"user_nickname"]);
            [[V2TIMManager sharedInstance] setSelfInfo:info succ:^{
                KLog(@"设置im头像成功");
            } fail:^(int code, NSString *desc) {
            }];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
    }];
}
@end
