//
//  BC_SearchViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/14.
//

#import "BC_SearchViewController.h"
#import "BC_SearchPostListViewController.h"
#import "BC_SearchGroupChatListViewController.h"
#import "BC_SearchProjectListViewController.h"

#import <JXCategoryView.h>
#import "JXCategoryTitleBackgroundView.h"

@interface BC_SearchViewController ()<UITextFieldDelegate, JXCategoryViewDelegate, JXCategoryListContainerViewDelegate>
@property (nonatomic, strong) UITextField *searchTF;
@property (nonatomic, strong) JXCategoryTitleBackgroundView *categoryView;
@property (nonatomic, strong) JXCategoryListContainerView *listContainerView;
@end

@implementation BC_SearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    [self setupUI];
}

- (void)setupUI {
    UIView *sview = [JLFactoryUtil view];
    sview.backgroundColor = kColorWithHex(@"#F7F7F7");
    [sview setCorner:12 withType:JLCornerTypeAllCorner];
    [self.view addSubview:sview];
    [sview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(15);
        make.top.equalTo(self.view.mas_top).offset(kStatusBarHeight);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 95, 44));
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [sview addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.searchTF = [JLFactoryUtil textFieldWithFont:REGULARFONT(14) place:@"Search"];
    self.searchTF.textColor = kMainTextColor;
    self.searchTF.delegate = self;
    self.searchTF.returnKeyType = UIReturnKeySearch;
    [sview addSubview:self.searchTF];
    [self.searchTF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sImg.mas_right).offset(10);
        make.right.equalTo(sview.mas_right).offset(-10);
        make.height.mas_equalTo(30);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainTextColor];
    [cancelBtn addTarget:self action:@selector(cancelBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_right);
        make.right.equalTo(self.view.mas_right);
        make.height.mas_equalTo(30);
    }];
    
    self.categoryView = [[JXCategoryTitleBackgroundView alloc] initWithFrame:CGRectMake(0, kTopHeight+7, kScreenWidth, 36)];
    self.categoryView.delegate = self;
    self.categoryView.backgroundColor = UIColor.whiteColor;
    self.categoryView.cellSpacing = 20;
    self.categoryView.titleColor = kMainTextColor;
    self.categoryView.titleSelectedColor = UIColor.whiteColor;
    self.categoryView.titleFont = REGULARFONT(15);
    self.categoryView.titleSelectedFont = REGULARFONT(15);
    self.categoryView.defaultSelectedIndex = 0;
    self.categoryView.titleColorGradientEnabled = NO;
    self.categoryView.averageCellSpacingEnabled = NO;
    self.categoryView.titles = @[@"Project", @"Post"];
    self.categoryView.cellWidth = 100;
    self.categoryView.normalBackgroundColor = kColorWithHex(@"#F5F5F5");
    self.categoryView.selectedBackgroundColor = kMainThemeColor;
    self.categoryView.backgroundCornerRadius = 18;
    self.categoryView.backgroundWidth = 100;
    self.categoryView.backgroundHeight = 36;
//    self.categoryView.cellBackgroundColorGradientEnabled = YES;
//    self.categoryView.cellBackgroundUnselectedColor = kColorWithHex(@"#F5F5F5");//
//    self.categoryView.cellBackgroundSelectedColor = UIColor.whiteColor;
    
//    JXCategoryIndicatorBackgroundView *backgroundView = [[JXCategoryIndicatorBackgroundView alloc] init];
//    backgroundView.indicatorHeight = 36;
//    backgroundView.indicatorCornerRadius = JXCategoryViewAutomaticDimension;
//    backgroundView.indicatorColor = kMainThemeColor;
//    self.categoryView.indicators = @[backgroundView];
    [self.view addSubview:self.categoryView];
    
    self.listContainerView = [[JXCategoryListContainerView alloc] initWithType:JXCategoryListContainerType_ScrollView delegate:self];
    self.listContainerView.frame = CGRectMake(0, kTopHeight+50, kScreenWidth, kScreenHeight - kTopHeight - 50);
    [self.view addSubview:self.listContainerView];
    self.categoryView.listContainer = self.listContainerView;
    
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    
    if ([KStringIsNull(textField.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Please input content"];
        return NO;
    }
    [textField resignFirstResponder];
    [self.categoryView reloadData];
    return YES;
}

- (void)cancelBtnClick {
    [self.navigationController popViewControllerAnimated:NO];
}

#pragma mark - JXCategoryViewDelegate
- (id<JXCategoryListContentViewDelegate>)listContainerView:(JXCategoryListContainerView *)listContainerView initListForIndex:(NSInteger)index {
    if (index == 0) {
        BC_SearchProjectListViewController *list = [BC_SearchProjectListViewController new];
        list.keyword = KStringIsNull(self.searchTF.text);
        return list;
    }
//    else if (index == 1) {
//        BC_SearchGroupChatListViewController *list = [BC_SearchGroupChatListViewController new];
//        list.keyword = KStringIsNull(self.searchTF.text);
//        return list;
//    }
    else {
        BC_SearchPostListViewController *list = [BC_SearchPostListViewController new];
        list.keyword = KStringIsNull(self.searchTF.text);
        return list;
    }
}

- (NSInteger)numberOfListsInlistContainerView:(JXCategoryListContainerView *)listContainerView {
    return 2;
}

@end
