//
//  BC_GroupInfoViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/14.
//

#import "BC_GroupInfoViewController.h"
#import "BC_JoinGroupSubmitSuccViewController.h"
#import "BC_ChangeMyNameInGroupViewController.h"
#import "BC_SearchUserViewController.h"

#import "BC_GroupMemberCollectionViewCell.h"
#import "BC_GroupInfoModel.h"

@import ImSDK_Plus;
@interface BC_GroupInfoViewController ()<UICollectionViewDelegate, UICollectionViewDataSource>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *groupNameL;
@property (nonatomic, strong) UILabel *groupNumL;
@property (nonatomic, strong) UICollectionView *memberCV;
@property (nonatomic, strong) UIImageView *logoImg;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) UILabel *disL;
@property (nonatomic, strong) UILabel *addressL;
@property (nonatomic, strong) UILabel *nicknameL;

@property (nonatomic, strong) BC_GroupInfoModel *infoModel;
@property (nonatomic, strong) NSMutableArray *memberArr;
@end

@implementation BC_GroupInfoViewController
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self getMyNameInGroup];
}

- (void)dealloc {
    [kNotificationCenter removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Group management";
    self.memberArr = [NSMutableArray array];
    [kNotificationCenter addObserver:self selector:@selector(getMemberList) name:@"updateMemberList" object:nil];
    [self setupUI];
    [self getGroupInfo];
    [self getMemberList];
}

#pragma mark  群名片
- (void)getMyNameInGroup {
    [[V2TIMManager sharedInstance] getGroupMembersInfo:KStringIsNull(self.groupid) memberList:@[KStringIsNull([UserManager shareManager].getUserInfo.TUIIden)] succ:^(NSArray<V2TIMGroupMemberFullInfo *> *memberList) {
        if (memberList.count > 0) {
            V2TIMGroupMemberFullInfo *info = memberList[0];
            self.nicknameL.text = KStringIsNull(info.nameCard).length==0?KStringIsNull(info.nickName):KStringIsNull(info.nameCard);
        }
    } fail:^(int code, NSString *desc) {
        
    }];
}

#pragma mark  成员列表
- (void)getMemberList {
    [[V2TIMManager sharedInstance] getGroupMemberList:KStringIsNull(self.groupid) filter:V2TIM_GROUP_MEMBER_FILTER_ALL nextSeq:0 succ:^(uint64_t nextSeq, NSArray<V2TIMGroupMemberFullInfo *> *memberList) {
        [self.memberArr removeAllObjects];
        for (V2TIMGroupMemberFullInfo *model in memberList) {
            NSString *name = KStringIsNull(model.nameCard).length == 0 ? KStringIsNull(model.nickName):KStringIsNull(model.nameCard);
            NSDictionary *dic = @{@"name":KStringIsNull(name), @"avatar":KStringIsNull(model.faceURL), @"type":@""};
            [self.memberArr addObject:dic];
        }
//        NSDictionary *dic = @{@"name":@"", @"avatar":@"", @"type":@"add"};
//        [self.memberArr addObject:dic];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.memberCV reloadData];
            [self.memberCV layoutIfNeeded];
            [self.memberCV mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.memberCV.collectionViewLayout.collectionViewContentSize.height);
            }];
        });
    } fail:^(int code, NSString *desc) {
        
    }];
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:26 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.contentView.mas_top).offset(15);
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(52, 52));
    }];
    
    self.groupNameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.groupNameL];
    [self.groupNameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.groupNumL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.groupNumL];
    [self.groupNumL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.avatar.mas_centerY).offset(2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    UIView *memberView = [JLFactoryUtil view];
    memberView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:memberView];
    [memberView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.avatar.mas_bottom).offset(15);
        make.height.mas_equalTo(36);
    }];
    
    UILabel *memL = [JLFactoryUtil labelWithText:@"Group member" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [memberView addSubview:memL];
    [memL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(memberView);
        make.left.equalTo(memberView.mas_left).offset(15);
    }];
    
    UICollectionViewFlowLayout *taglayout = [UICollectionViewFlowLayout new];
    taglayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    taglayout.minimumInteritemSpacing = 10;
    taglayout.minimumLineSpacing = 10;
    taglayout.itemSize = CGSizeMake((kScreenWidth-60)/5, 85);
    taglayout.sectionInset = UIEdgeInsetsMake(10, 10, 10, 10);
    self.memberCV = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:taglayout];
    self.memberCV.delegate = self;
    self.memberCV.dataSource = self;
    self.memberCV.backgroundColor = UIColor.whiteColor;
    self.memberCV.showsVerticalScrollIndicator = NO;
    self.memberCV.scrollEnabled = NO;
    [self.memberCV registerClass:[BC_GroupMemberCollectionViewCell class] forCellWithReuseIdentifier:@"membercell"];
    [self.contentView addSubview:self.memberCV];
    [self.memberCV mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView);
        make.left.equalTo(self.contentView);
        make.top.equalTo(memberView.mas_bottom).offset(10);
        make.height.mas_equalTo(1);
    }];
    
    UIView *realView = [JLFactoryUtil view];
    realView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:realView];
    [realView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.memberCV.mas_bottom).offset(5);
        make.height.mas_equalTo(36);
    }];
    
    UILabel *realestateL = [JLFactoryUtil labelWithText:@"Real estate" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [realView addSubview:realestateL];
    [realestateL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(realView);
        make.left.equalTo(realView.mas_left).offset(15);
    }];
    
    self.logoImg = [JLFactoryUtil imageView];
    [self.logoImg setCorner:5 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.logoImg];
    [self.logoImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(realView.mas_bottom).offset(15);
        make.size.mas_equalTo(CGSizeMake(88, 88));
    }];
    
    UIImageView *disImg = [JLFactoryUtil imageViewWithImg:@"价格"];
    [self.contentView addSubview:disImg];
    [disImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.logoImg);
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.disL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.disL];
    [self.disL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(disImg.mas_right).offset(5);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(self.logoImg);
    }];
    
    self.areaL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.bottom.equalTo(disImg.mas_top).offset(-10);
    }];
    
    UIImageView *addrImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.contentView addSubview:addrImg];
    [addrImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(disImg.mas_bottom).offset(10);
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.addressL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.addressL];
    [self.addressL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(addrImg.mas_right).offset(5);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(addrImg);
    }];
    
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"#F2F2F2");
    [self.contentView addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(self.logoImg.mas_bottom).offset(15);
        make.height.mas_equalTo(1);
    }];
    
    UILabel *nameL = [JLFactoryUtil labelWithText:@"My nickname" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [self.contentView addSubview:nameL];
    [nameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(line.mas_bottom).offset(15);
    }];
    
    UIImageView *arrow = [JLFactoryUtil imageViewWithImg:@"更多"];
    [self.contentView addSubview:arrow];
    [arrow mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(nameL);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@"" font:MEDIUMFONT(13) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(nameL);
        make.right.equalTo(arrow.mas_left).offset(-10);
    }];
    
    UIView *nicknameClear = [JLFactoryUtil view];
    nicknameClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:nicknameClear];
    [nicknameClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(nameL);
        make.height.mas_equalTo(40);
    }];
    UITapGestureRecognizer *tapnick = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(changeNickname)];
    [nicknameClear addGestureRecognizer:tapnick];
    
    if (_ifHasJoin) {
        UIButton *exitBtn = [JLFactoryUtil buttonWithTitle:@"Exit group chat" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
        [exitBtn setCorner:24 withType:JLCornerTypeAllCorner];
        exitBtn.backgroundColor = kMainThemeColor;
        [exitBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
        [exitBtn addTarget:self action:@selector(showLogoutAlert) forControlEvents:UIControlEventTouchUpInside];
        [self.contentView addSubview:exitBtn];
        [exitBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(30);
            make.top.equalTo(nameL.mas_bottom).offset(80);
            make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
        }];
        
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(exitBtn.mas_bottom).offset(30);
        }];
    } else {
        UIButton *joinBtn = [JLFactoryUtil buttonWithTitle:@"Join group chat" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
        [joinBtn setCorner:24 withType:JLCornerTypeAllCorner];
        joinBtn.backgroundColor = kMainThemeColor;
        [joinBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
        [joinBtn addTarget:self action:@selector(joinBtnClick) forControlEvents:UIControlEventTouchUpInside];
        [self.contentView addSubview:joinBtn];
        [joinBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(30);
            make.top.equalTo(nameL.mas_bottom).offset(80);
            make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
        }];
        
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(joinBtn.mas_bottom).offset(30);
        }];
    }
}

#pragma mark  UICollectionView Delegate
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
//    NSDictionary *dic = self.memberArr[indexPath.item];
//    if ([KStringIsNull(dic[@"type"]) isEqualToString:@"add"]) {
//        BC_SearchUserViewController *search = [BC_SearchUserViewController new];
//        search.groupid = KStringIsNull(self.groupid);
//        [self.navigationController pushViewController:search animated:YES];
//    }
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    BC_GroupMemberCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"membercell" forIndexPath:indexPath];
    [cell setDic:_memberArr[indexPath.item]];
    return cell;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return _memberArr.count;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

#pragma mark  修改昵称
- (void)changeNickname {
    BC_ChangeMyNameInGroupViewController *group = [BC_ChangeMyNameInGroupViewController new];
    group.groupid = KStringIsNull(self.groupid);
    group.myAvatar = KStringIsNull([UserManager shareManager].getUserInfo.member_avatar);
    [self.navigationController pushViewController:group animated:YES];
}

- (void)joinBtnClick {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    [[V2TIMManager sharedInstance] joinGroup:KStringIsNull(self.groupid) msg:@"Apply to join the group" succ:^{
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [self.navigationController pushViewController:[BC_JoinGroupSubmitSuccViewController new] animated:YES];
        });
    } fail:^(int code, NSString *desc) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [BaseFunction showAlert:KStringIsNull(desc)];
        });
    }];
}

- (void)showLogoutAlert {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Are you sure to leave the group chat" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelQuitAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

#pragma mark  确认退出
- (void)confirmAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    [[V2TIMManager sharedInstance] quitGroup:KStringIsNull(self.groupid) succ:^{
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    } fail:^(int code, NSString *desc) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [BaseFunction showAlert:KStringIsNull(desc)];
        });
    }];
}

- (void)cancelQuitAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)configUIData {
    
    BC_GroupInfoModel *detainfo = [BC_GroupInfoModel mj_objectWithKeyValues:_infoModel.datainfo];
    self.areaL.text = KStringIsNull(detainfo.name);
    if ([KStringIsNull(detainfo.price) isEqualToString:@""]) {
        self.disL.text = kStringFormat(@"%@", KStringIsNull(detainfo.service_type));
    } else {
        self.disL.text = kStringFormat(@"%@", KStringIsNull(detainfo.price));
    }
    self.addressL.text = KStringIsNull(detainfo.province);
    if (detainfo.images.count > 0) {
        [self.logoImg sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(detainfo.images[0])]];
    }
    if (_infoModel.groupinfo.count > 0) {
        BC_GroupInfoModel *groupModel = _infoModel.groupinfo.firstObject;
        self.groupNameL.text = KStringIsNull(groupModel.groupName);
        self.groupNumL.text = KStringIsNull(self.groupid);
        [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(groupModel.FaceUrl)]];
    }
        
}

- (void)getGroupInfo {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getimdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"groupid":KStringIsNull(self.groupid)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dic1 = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dic1[@"data"];
            self.infoModel = [BC_GroupInfoModel mj_objectWithKeyValues:infoDic];
            [self configUIData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

@end
