//
//  BC_VideoPostDetailViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/17.
//

#import "BC_VideoPostDetailViewController.h"
#import "BC_ReportPostViewController.h"

#import "BC_PostDetailCommentTableViewCell.h"
#import "FullScreenHelperViewController.h"
#import "LandscapeRightViewController.h"
#import "LandscapeLeftViewController.h"
#import "EnterFullScreenTransition.h"
#import "ExitFullScreenTransition.h"
#import "WMPlayer.h"

#import "BC_PostListModel.h"

@interface BC_VideoPostDetailViewController ()<WMPlayerDelegate, UIViewControllerTransitioningDelegate, UITableViewDelegate, UITableViewDataSource, UITextFieldDelegate, BC_PostDetailCommentTableViewCellDelegate>
@property (nonatomic, strong)WMPlayer  *wmPlayer;
@property (nonatomic, strong) UIView *header;
@property (nonatomic, strong) UILabel *titleL;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) UILabel *timeL;
@property (nonatomic, strong) UILabel *contentL;
@property (nonatomic, strong) UIView *areaView;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) UILabel *commentNumL;
@property (nonatomic, assign) CGFloat headerHeight;
@property (nonatomic, strong) UITextField *commentTF;
@property (nonatomic, strong) UIView *bottomView;
@property (nonatomic, strong) UIImageView *coverImg;

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@property (nonatomic, strong) BC_PostListModel *infoModel;
@property (nonatomic, strong) BC_PostListModel *replyModel;

@end

@implementation BC_VideoPostDetailViewController

- (void)dealloc{
    [self.wmPlayer pause];
    [self.wmPlayer removeFromSuperview];
    self.wmPlayer = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    KLog(@"DetailViewController dealloc");
}

//全屏的时候hidden底部homeIndicator
- (BOOL)prefersHomeIndicatorAutoHidden{
    return self.wmPlayer.isFullscreen;
}
- (UIStatusBarStyle)preferredStatusBarStyle{
    return UIStatusBarStyleDefault;
}
- (BOOL)prefersStatusBarHidden{
    return NO;
}
//视图控制器实现的方法
- (BOOL)shouldAutorotate{
     return YES;
}
- (UIInterfaceOrientationMask)supportedInterfaceOrientations{
    return UIInterfaceOrientationMaskAllButUpsideDown;
}

-(UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    return UIInterfaceOrientationPortrait;
}
//播放器CloseButton
- (void)wmplayer:(WMPlayer *)wmplayer clickedCloseButton:(UIButton *)closeBtn{
    if (wmplayer.isFullscreen) {
        [self exitFullScreen];
    }else{
        if (self.presentingViewController) {
            [self dismissViewControllerAnimated:YES completion:^{

            }];
        }else{
            [self.navigationController popViewControllerAnimated:YES];
        }
    }
}

///全屏按钮
- (void)wmplayer:(WMPlayer *)wmplayer clickedFullScreenButton:(UIButton *)fullScreenBtn{
    if (self.wmPlayer.viewState==PlayerViewStateSmall) {
         [self enterFullScreen];
    } else {
        [self exitFullScreen];
    }
}

- (void)enterFullScreen {
    if (self.wmPlayer.viewState!=PlayerViewStateSmall) {
        return;
    }
    LandscapeRightViewController *rightVC = [[LandscapeRightViewController alloc] init];
    [self presentToVC:rightVC];
}

- (void)exitFullScreen {
    if (self.wmPlayer.viewState!=PlayerViewStateFullScreen) {
        return;
    }
    self.wmPlayer.isFullscreen = NO;
    self.wmPlayer.viewState = PlayerViewStateAnimating;
    [self dismissViewControllerAnimated:YES completion:^{
       self.wmPlayer.viewState  = PlayerViewStateSmall;
    }];
}

- (void)presentToVC:(FullScreenHelperViewController *)aHelperVC{
     self.wmPlayer.viewState = PlayerViewStateAnimating;
       self.wmPlayer.beforeBounds = self.wmPlayer.bounds;
       self.wmPlayer.beforeCenter = self.wmPlayer.center;
       self.wmPlayer.parentView = self.wmPlayer.superview;
       self.wmPlayer.isFullscreen = YES;

       aHelperVC.wmPlayer = self.wmPlayer;
       aHelperVC.modalPresentationStyle = UIModalPresentationFullScreen;
       aHelperVC.transitioningDelegate = self;
       [self presentViewController:aHelperVC animated:YES completion:^{
           self.wmPlayer.viewState = PlayerViewStateFullScreen;
       }];
}

/**
 *  旋转屏幕通知
 */
- (void)onDeviceOrientationChange:(NSNotification *)notification{
    if (self.wmPlayer.viewState!=PlayerViewStateSmall) {
        return;
    }
    if (self.wmPlayer.isLockScreen){
        return;
    }
    UIDeviceOrientation orientation = [UIDevice currentDevice].orientation;
    UIInterfaceOrientation interfaceOrientation = (UIInterfaceOrientation)orientation;
    switch (interfaceOrientation) {
        case UIInterfaceOrientationPortraitUpsideDown:{
        }
            break;
        case UIInterfaceOrientationPortrait:{

        }
            break;
        case UIInterfaceOrientationLandscapeLeft:{
            [self presentToVC:[LandscapeLeftViewController new]];
        }
            break;
        case UIInterfaceOrientationLandscapeRight:{
            [self presentToVC:[LandscapeRightViewController new]];
        }
            break;
        default:
            break;
    }
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
     self.wmPlayer.delegate = self;
    [IQKeyboardManager sharedManager].enable = NO;
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [IQKeyboardManager sharedManager].enable = YES;
}

- (void)wmplayer:(WMPlayer *)wmplayer clickedPlayOrPauseButton:(UIButton *)playOrPauseBtn {
    
}

- (void)wmplayer:(WMPlayer *)wmplayer doubleTaped:(UITapGestureRecognizer *)doubleTap {
    if (wmplayer.state == WMPlayerStatePlaying) {
        [wmplayer pause];
    } else {
        [wmplayer play];
    }
}

- (void)backBtnClicked {
    [self.wmPlayer pause];
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.wmPlayer = [[WMPlayer alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, 224)];
    self.wmPlayer.tintColor = UIColor.whiteColor;
    self.wmPlayer.backBtnStyle = BackBtnStylePop;
    self.wmPlayer.loopPlay = NO;
    self.wmPlayer.delegate = self;
    self.wmPlayer.enableAirPlay = NO;
    [self.view addSubview:self.wmPlayer];
    
    self.coverImg = [JLFactoryUtil imageView];
    self.coverImg.userInteractionEnabled = YES;
    self.coverImg.frame = CGRectMake(0, 0, kScreenWidth, 224);
    [self.view addSubview:self.coverImg];
    UITapGestureRecognizer *tapCover = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(playVideo)];
    [self.coverImg addGestureRecognizer:tapCover];
    UIImageView *playimg = [JLFactoryUtil imageViewWithImg:@"播放icon"];
    playimg.userInteractionEnabled = YES;
    [self.coverImg addSubview:playimg];
    [playimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.centerX.equalTo(self.coverImg);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    //旋转屏幕通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onDeviceOrientationChange:) name:UIDeviceOrientationDidChangeNotification object:nil];
    [kNotificationCenter addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillChangeFrameNotification object:nil];
    
    self.headerHeight = 150;
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupUI];
    [self setupHeader];
    [self getPageData];
}

- (void)keyboardWillShow:(NSNotification *)notification {
    CGFloat duration = [notification.userInfo[UIKeyboardAnimationDurationUserInfoKey] floatValue];
    CGRect keyboardFrame = [notification.userInfo[UIKeyboardFrameEndUserInfoKey] CGRectValue];
    if (keyboardFrame.origin.y == kScreenHeight) {
        //执行动画
        [UIView animateWithDuration:duration animations:^{
            self.bottomView.frame = CGRectMake(0, kScreenHeight-kTopHeight-HOME_INDICATOR_HEIGHT-50, kScreenWidth, 50+HOME_INDICATOR_HEIGHT);
        }];
    } else {
        //执行动画
        [UIView animateWithDuration:duration animations:^{
            self.bottomView.frame = CGRectMake(0,kScreenHeight-kTopHeight-keyboardFrame.size.height-50, kScreenWidth, 50+HOME_INDICATOR_HEIGHT);
        }];
    }
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 224, kScreenWidth, kScreenHeight - kTopHeight - 50 - HOME_INDICATOR_HEIGHT - 224)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_PostDetailCommentTableViewCell class] forCellReuseIdentifier:@"cellid"];
//    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
//    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
    
    UIView *bottomView = [JLFactoryUtil view];
    bottomView.backgroundColor = kColorWithHex(@"#E6E6E6");
    bottomView.frame = CGRectMake(0, kScreenHeight-kTopHeight-HOME_INDICATOR_HEIGHT-50, kScreenWidth, 50+HOME_INDICATOR_HEIGHT);
    [self.view addSubview:bottomView];
    _bottomView = bottomView;
    
    UIView *cview = [JLFactoryUtil view];
    [cview setCorner:18 withType:JLCornerTypeAllCorner];
    cview.backgroundColor = UIColor.whiteColor;
    [bottomView addSubview:cview];
    [cview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(bottomView.mas_top).offset(7);
        make.left.equalTo(bottomView.mas_left).offset(15);
        make.right.equalTo(bottomView.mas_right).offset(-65);
        make.height.mas_equalTo(36);
    }];
    
    self.commentTF = [JLFactoryUtil textFieldWithFont:REGULARFONT(14) place:@"Please enter a comment"];
    self.commentTF.textColor = kMainTextColor;
    self.commentTF.returnKeyType = UIReturnKeySend;
    self.commentTF.delegate = self;
    [cview addSubview:self.commentTF];
    [self.commentTF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(cview);
        make.left.equalTo(cview.mas_left).offset(12);
        make.height.mas_equalTo(35);
        make.right.equalTo(cview.mas_right).offset(-10);
    }];
    
    UIButton *sendBtn = [JLFactoryUtil buttonWithImage:@"发送评论"];
    [sendBtn addTarget:self action:@selector(sendCommentAction) forControlEvents:UIControlEventTouchUpInside];
    [bottomView addSubview:sendBtn];
    [sendBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(bottomView);
        make.right.equalTo(bottomView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(35, 35));
    }];
    
}

- (void)setupHeader {
    self.header = [[UIView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, self.headerHeight)];
    self.tableView.tableHeaderView = self.header;
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:18 withType:JLCornerTypeAllCorner];
    [self.header addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.header.mas_left).offset(15);
        make.top.equalTo(self.header.mas_top).offset(15);
        make.size.mas_equalTo(CGSizeMake(36, 36));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kMainTextColor];
    [self.header addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.header.mas_centerX).offset(-2);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-1);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(11) textColor:kLightGrayTextColor];
    [self.header addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.header.mas_centerX).offset(-2);
        make.top.equalTo(self.avatar.mas_centerY).offset(1);
    }];
    
    self.areaView = [JLFactoryUtil view];
    self.areaView.backgroundColor = kColorWithHex(@"#ECF8EF");
    [self.areaView setCorner:12 withType:JLCornerTypeAllCorner];
    [self.header addSubview:self.areaView];
    [self.areaView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.header.mas_right).offset(-15);
        make.centerY.equalTo(self.avatar);
        make.size.mas_equalTo(CGSizeMake(30, 24));
    }];

    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.areaView addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(self.areaView.mas_left).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];

    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.areaView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(areaImg.mas_right).offset(4);
        make.width.mas_lessThanOrEqualTo(kScreenWidth/2 - 57);
    }];
    
    self.titleL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(18) textColor:kMainTextColor];
    self.titleL.numberOfLines = 0;
    [self.header addSubview:self.titleL];
    [self.titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.header.mas_left).offset(15);
        make.top.equalTo(self.avatar.mas_bottom).offset(20);
        make.right.lessThanOrEqualTo(self.header.mas_right).offset(-15);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    self.contentL.numberOfLines = 0;
    [self.header addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.header.mas_left).offset(15);
        make.top.equalTo(self.titleL.mas_bottom).offset(12);
        make.right.lessThanOrEqualTo(self.header.mas_right).offset(-15);
    }];
    
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"#F2F2F2");
    [self.header addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.header.mas_left).offset(15);
        make.top.equalTo(self.contentL.mas_bottom).offset(20);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"Comments" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [self.header addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.header.mas_left).offset(15);
        make.top.equalTo(line.mas_bottom).offset(15);
    }];
    
    self.commentNumL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kColorWithHex(@"#777777")];
    [self.header addSubview:self.commentNumL];
    [self.commentNumL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(label1.mas_right).offset(3);
        make.centerY.equalTo(label1);
    }];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_PostDetailCommentTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_PostDetailCommentTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_PostDetailCommentTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"cellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
}

#pragma mark  Action
#pragma mark  举报
- (void)reportAction {
    BC_ReportPostViewController *report = [BC_ReportPostViewController new];
    report.postid = KStringIsNull(self.postid);
    [self.navigationController pushViewController:report animated:YES];
}

#pragma mark  删除帖子
- (void)delMyPostAction {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Are you sure you want to delete this post?" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)cancelAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)confirmAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postdelete/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.postid)} success:^(id responseObject) {
        KLog(@"删帖成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Delete success"];
            [kNotificationCenter postNotificationName:@"refreshAfterDel" object:nil];
            [self.navigationController popViewControllerAnimated:YES];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (nullable id <UIViewControllerAnimatedTransitioning>)animationControllerForPresentedController:(UIViewController *)presented presentingController:(UIViewController *)presenting sourceController:(UIViewController *)source{
    return [[EnterFullScreenTransition alloc] initWithPlayer:self.wmPlayer];
}

- (nullable id <UIViewControllerAnimatedTransitioning>)animationControllerForDismissedController:(UIViewController *)dismissed{
    return [[ExitFullScreenTransition alloc] initWithPlayer:self.wmPlayer];
}

//准备播放的代理方法
- (void)wmplayerReadyToPlay:(WMPlayer *)wmplayer WMPlayerStatus:(WMPlayerState)state {
    
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if ([KStringIsNull(textField.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Comment cannot be empty"];
        return NO;
    }
    [self sendCommentAction];
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    self.commentTF.placeholder = @"Please enter a comment";
}

- (void)replyComment:(BC_PostListModel *)model {
    self.replyModel = model;
    self.commentTF.placeholder = kStringFormat(@"reply @%@", KStringIsNull(model.comment_author_name));
    [self.commentTF becomeFirstResponder];
}

- (void)playVideo {
    [UIView animateWithDuration:0.3 animations:^{
        self.coverImg.alpha = 0;
    } completion:^(BOOL finished) {
        [self.coverImg removeFromSuperview];
        [self.wmPlayer play];
    }];
}

#pragma mark  发送评论
- (void)sendCommentAction {
    if ([KStringIsNull(self.commentTF.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Comment cannot be empty"];
        return;
    }
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postscomment/", KMainUrl];
    NSDictionary *para = @{@"id":KStringIsNull(self.postid), @"content":KStringIsNull(self.commentTF.text), @"pid":KStringIsNull(self.replyModel.itemid)};
    KLog(@"评论=====%@", para);
    [PPNetworkHelper POST:url parameters:para success:^(id responseObject) {
        KLog(@"评论成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [self.commentTF resignFirstResponder];
            self.replyModel = nil;
            [BaseFunction showAlert:@"Send success"];
            self.commentTF.text = @"";
            [self updateCommentList];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)getPageData {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getpostsdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.postid)} success:^(id responseObject) {
        KLog(@"详情成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            self.infoModel = [BC_PostListModel mj_objectWithKeyValues:dataDic[@"data"]];
            self.dataArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:self.infoModel.conmment_list];
            
            self.areaL.text = KStringIsNull(self.infoModel.city);
            CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
            if (size.width >= kScreenWidth/2 - 57) {
                size.width = kScreenWidth/2 - 57;
            }
            [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
                make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
            }];
            
            [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(self.infoModel.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
            self.nicknameL.text = KStringIsNull(self.infoModel.user_name).length == 0 ? @"User":KStringIsNull(self.infoModel.user_name);
            self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
            self.titleL.text = KStringIsNull(self.infoModel.name);
            self.contentL.text = KStringIsNull(self.infoModel.desc);
            NSInteger cnum = 0;
            for (BC_PostListModel *m in self.dataArr) {
                cnum++;
                NSInteger subnum = m.childlist.count;
                cnum = cnum + subnum;
            }
            self.commentNumL.text = kStringFormat(@"(%zd)", cnum);
                        
            CGSize tsize = [self.titleL sizeThatFits:CGSizeMake(kScreenWidth - 30, CGFLOAT_MAX)];
            CGSize csize = [self.contentL sizeThatFits:CGSizeMake(kScreenWidth - 30, CGFLOAT_MAX)];
            self.headerHeight = tsize.height + csize.height + 160;
            self.header.frame = CGRectMake(0, 0, kScreenWidth, self.headerHeight);
            self.tableView.tableHeaderView = self.header;
            
            if ([KStringIsNull(self.infoModel.isown) isEqualToString:@"1"]) {
                UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithImage:[[UIImage imageNamed:@"删除帖子"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]  style:UIBarButtonItemStylePlain target:self action:@selector(delMyPostAction)];
                self.navigationItem.rightBarButtonItem = item;
            } else {
                UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithImage:[[UIImage imageNamed:@"举报"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]  style:UIBarButtonItemStylePlain target:self action:@selector(reportAction)];
                self.navigationItem.rightBarButtonItem = item;
            }
            
            if (self.infoModel.files.count > 0) {
                WMPlayerModel *playerModel = [WMPlayerModel new];
                NSString *encodedString = [KStringIsNull(self.infoModel.files[0]) stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
                playerModel.videoURL = [NSURL URLWithString:KStringIsNull(encodedString)];
            
//                playerModel.videoURL = [NSURL URLWithString:@"http://static.tripbe.com/videofiles/20121214/9533522808.f4v.mp4"];
                playerModel.title = @"";
                self.wmPlayer.playerModel = playerModel;
                
                dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                    UIImage *cover = [BaseFunction thumbnailImageForVideo:[NSURL URLWithString:KStringIsNull(encodedString)] atTime:0.1];
                    dispatch_async(dispatch_get_main_queue(), ^{
                        self.coverImg.image = cover;
                    });
                });
                
            }
                
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
            
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)updateCommentList {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getpostsdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.postid)} success:^(id responseObject) {
        KLog(@"列表成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            BC_PostListModel *model = [BC_PostListModel mj_objectWithKeyValues:dataDic[@"data"]];
            self.dataArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:model.conmment_list];
            NSInteger cnum = 0;
            for (BC_PostListModel *m in self.dataArr) {
                cnum++;
                cnum = cnum + m.childlist.count;
            }
            self.commentNumL.text = kStringFormat(@"(%zd)", cnum);
            
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
