//
//  BC_SearchProjectListViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/14.
//

#import "BC_SearchProjectListViewController.h"
#import "BC_ProjectDetailViewController.h"

#import "BC_SearchProjectListTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_SearchProjectListViewController ()<UITableViewDelegate, UITableViewDataSource, DZNEmptyDataSetSource, DZNEmptyDataSetDelegate, BC_SearchProjectListTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@end

@implementation BC_SearchProjectListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupListView];
    [self loadData];
}

- (void)setupListView {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - 50)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.backgroundColor = UIColor.whiteColor;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_SearchProjectListTableViewCell class] forCellReuseIdentifier:@"cellid"];
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_SearchProjectListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_SearchProjectListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_SearchProjectListTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"cellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    BC_ExecuteModuleModel *model = self.dataArr[indexPath.row];
    BC_ProjectDetailViewController *detail = [BC_ProjectDetailViewController new];
    detail.pid = KStringIsNull(model.itemid);
    [self.navigationController pushViewController:detail animated:YES];
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_ExecuteModuleModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_ExecuteModuleModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_ExecuteModuleModel *)model {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"2"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.dataArr removeObject:model];
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    return [UIImage imageNamed:@"空数据"];
}

- (BOOL)emptyDataSetShouldAllowScroll:(UIScrollView *)scrollView {
    return YES;
}

#pragma mark - JXCategoryListContentViewDelegate
- (UIView *)listView {
    return self.view;
}

- (void)getPageData {
    if ([KStringIsNull(self.keyword) isEqualToString:@""]) return;
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/myprogramlist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10", @"status":@"", @"search":KStringIsNull(self.keyword)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}


@end
