//
//  BC_PostDetailCommentTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/15.
//

#import "BC_PostDetailCommentTableViewCell.h"
#import "BC_PostDetailSubCommentTableViewCell.h"
#import "BC_PostListModel.h"
@interface BC_PostDetailCommentTableViewCell()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UITableView *subTB;
@end

@implementation BC_PostDetailCommentTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.contentView.backgroundColor = UIColor.whiteColor;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:18 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(15);
        make.size.mas_equalTo(CGSizeMake(36, 36));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-1);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kMainTextColor];
    self.contentL.numberOfLines = 0;
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.avatar.mas_centerY).offset(1);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(12) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.contentL.mas_bottom).offset(5);
    }];
    
    self.replyBtn = [JLFactoryUtil buttonWithTitle:@"Reply" font:MEDIUMFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.replyBtn addTarget:self action:@selector(replyBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:self.replyBtn];
    [self.replyBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.timeL.mas_right).offset(12);
        make.centerY.equalTo(self.timeL);
        make.size.mas_equalTo(CGSizeMake(40, 20));
    }];
    
    self.subTB = [[UITableView alloc] initWithFrame:CGRectZero];
    self.subTB.delegate = self;
    self.subTB.dataSource = self;
    self.subTB.tableFooterView = [UIView new];
    self.subTB.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.subTB.scrollEnabled = NO;
    [self.contentView addSubview:self.subTB];
    [self.subTB registerClass:[BC_PostDetailSubCommentTableViewCell class] forCellReuseIdentifier:@"subcellid"];
    [self.subTB mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.nicknameL.mas_left);
        make.top.equalTo(self.timeL.mas_bottom).offset(1);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.height.mas_equalTo(1);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(-15);
    }];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"subcellid";
    BC_PostDetailSubCommentTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_PostDetailSubCommentTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_PostDetailSubCommentTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_model.childlist[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"subcellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _model.childlist.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)replyBtnClick {
    if (self.delegate && [self.delegate respondsToSelector:@selector(replyComment:)]) {
        [self.delegate replyComment:_model];
    }
}

- (void)setModel:(BC_PostListModel *)model {
    _model = model;
    [self.subTB reloadData];
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(model.comment_author_avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(model.comment_author_name).length == 0 ? @"User":KStringIsNull(model.comment_author_name);
    self.contentL.text = KStringIsNull(model.comment_content);
    self.timeL.text = KStringIsNull(model.create_datetime);
    
    CGFloat rowH = 65 * _model.childlist.count;
    CGFloat cH = 0;
    for (BC_PostListModel *c in _model.childlist) {
        CGSize size = [KStringIsNull(c.comment_content) boundingRectWithSize:CGSizeMake(kScreenWidth - 114, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin|NSStringDrawingUsesFontLeading attributes:@{NSFontAttributeName: REGULARFONT(15)} context:nil].size;
        cH = cH + size.height;
    }
    [self.subTB mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(rowH + cH);
    }];
}
@end
