//
//  BC_HouseGroupChatListViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2022/1/19.
//

#import "BC_HouseGroupChatListViewController.h"
#import "BC_GroupChatViewController.h"
#import "TUIChatConversationModel.h"
#import "BC_HouseGroupListTableViewCell.h"
@import ImSDK_Plus;

@interface BC_HouseGroupChatListViewController ()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@end

@implementation BC_HouseGroupChatListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Real estate";
    self.dataArr = [NSMutableArray array];
    [self setupUI];
    [self getGroupList];
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - kTabBarHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_HouseGroupListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_HouseGroupListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 80;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    V2TIMGroupInfo *gmodel = self.dataArr[indexPath.row];
    
    TUIChatConversationModel *model = [[TUIChatConversationModel alloc] init];
    model.groupID = KStringIsNull(gmodel.groupID);
    model.groupType = KStringIsNull(gmodel.groupType);
    model.title = KStringIsNull(gmodel.groupName);
    BC_GroupChatViewController *chat = [[BC_GroupChatViewController alloc] init];
    chat.conversationData = model;
    [self.navigationController pushViewController:chat animated:YES];
    
}

- (void)getGroupList {
    [self.dataArr removeAllObjects];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    [[V2TIMManager sharedInstance] getJoinedGroupList:^(NSArray<V2TIMGroupInfo *> *groupList) {
        for (V2TIMGroupInfo *info in groupList) {
            NSDictionary *dic = info.customInfo;
            NSData *type = dic[@"groupcategory"];
            
            if (type.length != 0) {
                NSString *typestr = [self stringFromHexString:[self hexStringFromString:type]];
                KLog(@"typestr===%@", typestr);
                if ([KStringIsNull(typestr) isEqualToString:@"1"]) {
                    [self.dataArr addObject:info];
                }
            }
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [self.tableView reloadData];
        });
    } fail:^(int code, NSString *desc) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
        });
    }];
}

- (NSString *)hexStringFromString:(NSData *)data{
    Byte *bytes = (Byte *)[data bytes];
    //下面是Byte转换为16进制。
    NSString *hexStr=@"";
    for(int i=0;i<[data length];i++){
        NSString *newHexStr = [NSString stringWithFormat:@"%x",bytes[i]&0xff];///16进制数
        if([newHexStr length]==1)
            hexStr = [NSString stringWithFormat:@"%@0%@",hexStr,newHexStr];
        else
            hexStr = [NSString stringWithFormat:@"%@%@",hexStr,newHexStr];
    }
    return hexStr;
}

// 十六进制转换为普通字符串的。
- (NSString *)stringFromHexString:(NSString *)hexString {
    
    char *myBuffer = (char *)malloc((int)[hexString length] / 2 + 1);
    bzero(myBuffer, [hexString length] / 2 + 1);
    for (int i = 0; i < [hexString length] - 1; i += 2) {
        unsigned int anInt;
        NSString * hexCharStr = [hexString substringWithRange:NSMakeRange(i, 2)];
        NSScanner * scanner = [[NSScanner alloc] initWithString:hexCharStr];
        [scanner scanHexInt:&anInt];
        myBuffer[i / 2] = (char)anInt;
    }
    NSString *unicodeString = [NSString stringWithCString:myBuffer encoding:4];
    return unicodeString;
}

@end
