//
//  BC_SearchUserViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2022/3/1.
//

#import "BC_SearchUserViewController.h"
#import "BC_SearchUserTableViewCell.h"
#import "BC_GroupInfoModel.h"

@import ImSDK_Plus;
@interface BC_SearchUserViewController ()<UITableViewDelegate, UITableViewDataSource, BC_SearchUserTableViewCellDelegate, UITextFieldDelegate>
@property (nonatomic, strong) UITextField *searchTF;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@end

@implementation BC_SearchUserViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Invitation";
    self.dataArr = [NSMutableArray array];
    [self setupUI];
}

- (void)setupUI {
    
    UIView *sview = [JLFactoryUtil view];
    sview.backgroundColor = kColorWithHex(@"#F7F7F7");
    [sview setCorner:12 withType:JLCornerTypeAllCorner];
    [self.view addSubview:sview];
    [sview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(15);
        make.top.equalTo(self.view.mas_top).offset(15);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 95, 44));
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [sview addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.searchTF = [JLFactoryUtil textFieldWithFont:REGULARFONT(14) place:@"Search"];
    self.searchTF.textColor = kMainTextColor;
    self.searchTF.delegate = self;
    self.searchTF.returnKeyType = UIReturnKeySearch;
    [sview addSubview:self.searchTF];
    [self.searchTF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sImg.mas_right).offset(10);
        make.right.equalTo(sview.mas_right).offset(-10);
        make.height.mas_equalTo(30);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainTextColor];
    [cancelBtn addTarget:self action:@selector(cancelBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_right);
        make.right.equalTo(self.view.mas_right);
        make.height.mas_equalTo(30);
    }];
    
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 75, kScreenWidth, kScreenHeight - kTopHeight - 75)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_SearchUserTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_SearchUserTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [cell setModel:self.dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 80;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    
    if ([KStringIsNull(textField.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Please input content"];
        return NO;
    }
    [textField resignFirstResponder];
    [self searchUser];
    return YES;
}

- (void)cancelBtnClick {
    [self.navigationController popViewControllerAnimated:NO];
}

- (void)searchUser {
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/searchuser/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"mobile":KStringIsNull(self.searchTF.text)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            self.dataArr = [BC_GroupInfoModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"服务器错误"];
    }];
}

- (void)inviteUser:(BC_GroupInfoModel *)model {
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/addgroupmember/", KMainUrl];
    NSArray *arr = @[KStringIsNull(model.itemid)];
    NSDictionary *para = @{@"ids":KStringIsNull([arr mj_JSONString]), @"groupid":KStringIsNull(self.groupid)};
    KLog(@"para====%@", para);
    [PPNetworkHelper POST:url parameters:para success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"updateMemberList" object:nil];
            [BaseFunction showAlert:@"Invitation succeeded"];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
    
//    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
//    [[V2TIMManager sharedInstance] inviteUserToGroup:KStringIsNull(self.groupid) userList:@[] succ:^(NSArray<V2TIMGroupMemberOperationResult *> *resultList) {
//        if (resultList.count > 0) {
//            V2TIMGroupMemberOperationResult *res = resultList[0];
//            KLog(@"res.result===%zd", res.result);
//            if (res.result == 1) {
//                //成功
//                [kNotificationCenter postNotificationName:@"updateMemberList" object:nil];
//                dispatch_async(dispatch_get_main_queue(), ^{
//                    [MBProgressHUD hideHUDForView:self.view animated:YES];
//                    [BaseFunction showAlert:@"Invitation succeeded"];
//                });
//            } else {
//                dispatch_async(dispatch_get_main_queue(), ^{
//                    [MBProgressHUD hideHUDForView:self.view animated:YES];
//                    [BaseFunction showAlert:@"Invitation failed"];
//                });
//            }
//        } else {
//            dispatch_async(dispatch_get_main_queue(), ^{
//                [MBProgressHUD hideHUDForView:self.view animated:YES];
//                [BaseFunction showAlert:@"Invitation failed"];
//            });
//        }
//    } fail:^(int code, NSString *desc) {
//        dispatch_async(dispatch_get_main_queue(), ^{
//            [MBProgressHUD hideHUDForView:self.view animated:YES];
//            [BaseFunction showAlert:@"Invitation failed"];
//        });
//    }];
}
@end
