//
//  BC_HouseGroupListTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2022/1/19.
//

#import "BC_HouseGroupListTableViewCell.h"
@import ImSDK_Plus;

@implementation BC_HouseGroupListTableViewCell
- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.contentView.backgroundColor = UIColor.whiteColor;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:24 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.contentView);
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    self.nameL = [JLFactoryUtil labelWithText:@"" font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.nameL];
    [self.nameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-100);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-2);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@"" font:REGULARFONT(12) textColor:kColorWithHex(@"#AAAAAA")];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(self.nameL);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@"" font:REGULARFONT(13) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(self.avatar.mas_centerY).offset(2);
    }];
}

- (void)setModel:(V2TIMGroupInfo *)model {
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(model.faceURL)] placeholderImage:kAvatarPlaceHolderImg];
    self.nameL.text = KStringIsNull(model.groupName);
    self.contentL.text = KStringIsNull(model.introduction);
    
    NSTimeInterval time = model.joinTime;
    if (time > 0) {
        NSDate *date = [NSDate dateWithTimeIntervalSince1970:time];
        NSString *str = [date stringWithFormat:@"yyyy-MM-dd"];
        self.timeL.text = KStringIsNull(str);
    } else {
        self.timeL.text = @"";
    }
        
}

@end
