//
//  BC_UserCommentListTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/15.
//

#import "BC_UserCommentListTableViewCell.h"
#import "BC_MessageVCModel.h"
@implementation BC_UserCommentListTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.contentView.backgroundColor = UIColor.whiteColor;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:24 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.contentView);
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    self.timeL = [YYLabel new];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.centerY.equalTo(self.avatar);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-75);
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.bottom.equalTo(self.timeL.mas_top).offset(-2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-75);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kMainTextColor];
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.timeL.mas_bottom).offset(2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-75);
    }];
    
    self.contentImg = [JLFactoryUtil imageView];
    [self.contentImg setCorner:4 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.contentImg];
    [self.contentImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.contentView);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(56, 56));
    }];
        
}

- (void)setModel:(BC_MessageVCModel *)model {
    BC_MessageVCModel *umodel = [BC_MessageVCModel mj_objectWithKeyValues:model.userinfo];
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(umodel.avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(umodel.nickname);
    self.contentL.text = KStringIsNull(model.content);
    
    BC_MessageVCModel *fmodel = [BC_MessageVCModel mj_objectWithKeyValues:model.mypostinfo];
    if (fmodel.files.count > 0) {
        NSString *file = KStringIsNull(fmodel.files[0]);
        if ([file hasSuffix:@"mp4"]) {
            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                UIImage *img = [BaseFunction thumbnailImageForVideo:[NSURL URLWithString:KStringIsNull(file)] atTime:0.1];
                dispatch_async(dispatch_get_main_queue(), ^{
                    self.contentImg.image = img;
                });
            });
        } else {
            [self.contentImg sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(file)]];
        }
    } else {
        self.contentImg.image = [UIImage imageNamed:@"none"];
    }
    NSString *timeStr = KStringIsNull(KStringIsNull(model.create_datetime));
    NSMutableAttributedString *att = [[NSMutableAttributedString alloc] initWithString:kStringFormat(@"Comment on the %@", timeStr)];
    [att setYy_font:REGULARFONT(12)];
    [att yy_setColor:kColorWithHex(@"#777777") range:NSMakeRange(0, 15)];
    [att yy_setColor:kColorWithHex(@"#AAAAAA") range:NSMakeRange(15, timeStr.length)];
    self.timeL.attributedText = att;
}
@end
