//
//  BC_MineViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/13.
//

#import "BC_MineViewController.h"
#import "BC_PersonInfoViewController.h"
#import "BC_IntegralListViewController.h"
#import "BC_MyPostViewController.h"
#import "BC_InviteUserViewController.h"
#import "BC_MyRealEstateViewController.h"
#import "BC_ExecutorInComeViewController.h"
#import "BC_ExecutorRecommendViewController.h"
#import "BC_HouseDetailViewController.h"

#import "TUILogin.h"
@import ImSDK_Plus;

@interface BC_MineViewController ()<UIScrollViewDelegate>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) UILabel *phoneL;
@property (nonatomic, strong) UILabel *integralL;
@property (nonatomic, strong) UILabel *incomeL;
@property (nonatomic, strong) UILabel *estateNumL;
@property (nonatomic, strong) UIView *statusView;
@property (nonatomic, strong) UIView *inteView;
@property (nonatomic, strong) UIButton *signoutBtn;
@property (nonatomic, strong) UILabel *estateL;
@property (nonatomic, strong) NSMutableArray *houseArr;
@property (nonatomic, strong) NSMutableArray *houseItemArr;
@end

@implementation BC_MineViewController
- (void)dealloc {
    KLog(@"BC_MessageViewController dealloc");
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self getUserInfo];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    self.houseArr = [NSMutableArray array];
    self.houseItemArr = [NSMutableArray array];
    [self setupUI];
    BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
    if ([KStringIsNull(umodel.identity) isEqualToString:@"1"]) {
        //游客身份
        [self setupGuestModule];
    } else if ([KStringIsNull(umodel.identity) isEqualToString:@"2"]) {
        //专业用户身份
        [self setupProfessionModule];
    } else if ([KStringIsNull(umodel.identity) isEqualToString:@"3"]) {
        //执行者身份
        [self setupExecutorModule];
    }
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTabBarHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    self.scrollView.delegate = self;
    [self.view addSubview:self.scrollView];
    self.scrollView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;

    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    UIImageView *topImg = [JLFactoryUtil imageViewWithImg:@"我的背景"];
    topImg.userInteractionEnabled = YES;
    [self.contentView addSubview:topImg];
    [topImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.equalTo(self.contentView);
        make.height.mas_equalTo(kTopHeight+112);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:33 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(kStatusBarHeight+20);
        make.size.mas_equalTo(CGSizeMake(66, 66));
    }];
    
    UIView *infoClear = [JLFactoryUtil view];
    infoClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:infoClear];
    [infoClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(self.avatar);
        make.height.mas_equalTo(70);
    }];
    UITapGestureRecognizer *tapInfo = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushInfo)];
    [infoClear addGestureRecognizer:tapInfo];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(20) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(18);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-3);
    }];
    
    self.phoneL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.phoneL];
    [self.phoneL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(18);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(self.avatar.mas_centerY).offset(3);
    }];
    
    UIView *inteView = [JLFactoryUtil view];
    inteView.backgroundColor = UIColor.whiteColor;
    [inteView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [inteView setCorner:8 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:inteView];
    [inteView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(topImg.mas_bottom).offset(-50);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 88));
    }];
    UITapGestureRecognizer *tapInte = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushIntegral)];
    [inteView addGestureRecognizer:tapInte];
    _inteView = inteView;
    
    UILabel *inteL = [JLFactoryUtil labelWithText:@"Integral" font:REGULARFONT(15) textColor:kMainTextColor];
    inteL.userInteractionEnabled = YES;
    [inteView addSubview:inteL];
    [inteL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(inteView.mas_left).offset(15);
        make.top.equalTo(inteView.mas_top).offset(12);
    }];
    
    self.integralL = [JLFactoryUtil labelWithText:@"0" font:SEMIBOLDFONT(24) textColor:kMainTextColor];
    self.integralL.userInteractionEnabled = YES;
    [inteView addSubview:self.integralL];
    [self.integralL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(inteView.mas_left).offset(15);
        make.top.equalTo(inteL.mas_bottom).offset(10);
    }];
        
    //如果是执行者，加入收入展示
    if ([KStringIsNull([UserManager shareManager].getUserInfo.identity) isEqualToString:@"3"]) {
        [inteView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(topImg.mas_bottom).offset(-50);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 45)/2, 88));
        }];
        
        UIView *incomeView = [JLFactoryUtil view];
        incomeView.backgroundColor = UIColor.whiteColor;
        [incomeView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
        [incomeView setCorner:8 withType:JLCornerTypeAllCorner];
        [self.contentView addSubview:incomeView];
        [incomeView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(inteView.mas_right).offset(15);
            make.top.equalTo(topImg.mas_bottom).offset(-50);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 45)/2, 88));
        }];
        UITapGestureRecognizer *tapIncome = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushIncome)];
        [incomeView addGestureRecognizer:tapIncome];
        
        UILabel *incomeL = [JLFactoryUtil labelWithText:@"Income" font:REGULARFONT(15) textColor:kMainTextColor];
        incomeL.userInteractionEnabled = YES;
        [incomeView addSubview:incomeL];
        [incomeL mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(incomeView.mas_left).offset(15);
            make.top.equalTo(incomeView.mas_top).offset(12);
        }];
        
        self.incomeL = [JLFactoryUtil labelWithText:@"0" font:SEMIBOLDFONT(24) textColor:kMainTextColor];
        self.incomeL.userInteractionEnabled = YES;
        [incomeView addSubview:self.incomeL];
        [self.incomeL mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(incomeView.mas_left).offset(15);
            make.top.equalTo(incomeL.mas_bottom).offset(10);
        }];
    }

    UILabel *label1 = [JLFactoryUtil labelWithText:@"Real estate" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [self.contentView addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(inteView.mas_bottom).offset(120);
    }];
    _estateL = label1;
    
    self.estateNumL = [JLFactoryUtil labelWithText:@"(0)" font:MEDIUMFONT(14) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.estateNumL];
    [self.estateNumL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(label1.mas_right).offset(5);
        make.centerY.equalTo(label1);
    }];
    
    UIButton *arrow = [JLFactoryUtil buttonWithImage:@"更多"];
    [arrow addTarget:self action:@selector(clickMore) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:arrow];
    [arrow mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.centerY.equalTo(label1);
        make.size.mas_equalTo(CGSizeMake(25, 25));
    }];
        
    //游客登录没有退出按钮
    CGFloat marginTop = 20;
    UIButton *signoutBtn = [JLFactoryUtil buttonWithTitle:@"Sign out" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
    [signoutBtn setCorner:24 withType:JLCornerTypeAllCorner];
    signoutBtn.backgroundColor = kMainThemeColor;
    [signoutBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
    [signoutBtn addTarget:self action:@selector(signoutBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:signoutBtn];
    [signoutBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(30);
        make.top.equalTo(label1.mas_bottom).offset(marginTop);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
    }];
    _signoutBtn = signoutBtn;
        
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(signoutBtn.mas_bottom).offset(30);
    }];
    
    self.statusView = [JLFactoryUtil view];
    self.statusView.frame = CGRectMake(0, 0, kScreenWidth, kStatusBarHeight);
    self.statusView.backgroundColor = [UIColor.whiteColor colorWithAlphaComponent:0];
    [self.view addSubview:self.statusView];
    
    self.avatar.image = kAvatarPlaceHolderImg;
    self.nicknameL.text = @" ";
    self.phoneL.text = @" ";
    self.integralL.text = @"0";
    self.estateNumL.text = @"(0)";
    
}

#pragma mark  游客功能
- (void)setupGuestModule {
    //游客有我的帖子和邀请模块
    UIView *postView = [JLFactoryUtil view];
    [postView setCorner:8 withType:JLCornerTypeAllCorner];
    [postView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:postView];
    [postView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 45)/2, 88));
    }];
    UITapGestureRecognizer *tapPost = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyPost)];
    [postView addGestureRecognizer:tapPost];
    
    UIImageView *postimg = [JLFactoryUtil imageViewWithImg:@"我发的帖子"];
    postimg.userInteractionEnabled = YES;
    [postView addSubview:postimg];
    [postimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(postView);
        make.bottom.equalTo(postView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *postL = [JLFactoryUtil labelWithText:@"My post" font:REGULARFONT(14) textColor:kMainTextColor];
    postL.userInteractionEnabled = YES;
    [postView addSubview:postL];
    [postL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(postView);
        make.top.equalTo(postimg.mas_bottom).offset(4);
    }];
    
    UIView *inviteView = [JLFactoryUtil view];
    [inviteView setCorner:8 withType:JLCornerTypeAllCorner];
    [inviteView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:inviteView];
    [inviteView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(postView.mas_right).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 45)/2, 88));
    }];
    UITapGestureRecognizer *tapInvi = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyInvite)];
    [inviteView addGestureRecognizer:tapInvi];
    
    UIImageView *inviteImg = [JLFactoryUtil imageViewWithImg:@"邀请好友"];
    inviteImg.userInteractionEnabled = YES;
    [inviteView addSubview:inviteImg];
    [inviteImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.bottom.equalTo(inviteView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *inviteL = [JLFactoryUtil labelWithText:@"Invite users" font:REGULARFONT(14) textColor:kMainTextColor];
    inviteL.userInteractionEnabled = YES;
    [inviteView addSubview:inviteL];
    [inviteL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.top.equalTo(inviteImg.mas_bottom).offset(4);
    }];
}

#pragma mark  专业用户功能
- (void)setupProfessionModule {
    UIView *proView = [JLFactoryUtil view];
    [proView setCorner:8 withType:JLCornerTypeAllCorner];
    [proView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:proView];
    [proView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapPro = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyProject)];
    [proView addGestureRecognizer:tapPro];
    
    UIImageView *proimg = [JLFactoryUtil imageViewWithImg:@"订单"];
    proimg.userInteractionEnabled = YES;
    [proView addSubview:proimg];
    [proimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(proView);
        make.bottom.equalTo(proView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *proL = [JLFactoryUtil labelWithText:@"My project" font:REGULARFONT(14) textColor:kMainTextColor];
    proL.userInteractionEnabled = YES;
    [proView addSubview:proL];
    [proL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(proView);
        make.top.equalTo(proimg.mas_bottom).offset(4);
    }];
    
    UIView *postView = [JLFactoryUtil view];
    [postView setCorner:8 withType:JLCornerTypeAllCorner];
    [postView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:postView];
    [postView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(proView.mas_right).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapPost = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyPost)];
    [postView addGestureRecognizer:tapPost];
    
    UIImageView *postimg = [JLFactoryUtil imageViewWithImg:@"我发的帖子"];
    postimg.userInteractionEnabled = YES;
    [postView addSubview:postimg];
    [postimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(postView);
        make.bottom.equalTo(postView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *postL = [JLFactoryUtil labelWithText:@"My post" font:REGULARFONT(14) textColor:kMainTextColor];
    postL.userInteractionEnabled = YES;
    [postView addSubview:postL];
    [postL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(postView);
        make.top.equalTo(postimg.mas_bottom).offset(4);
    }];
    
    UIView *inviteView = [JLFactoryUtil view];
    [inviteView setCorner:8 withType:JLCornerTypeAllCorner];
    [inviteView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:inviteView];
    [inviteView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(postView.mas_right).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapInvi = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyInvite)];
    [inviteView addGestureRecognizer:tapInvi];
    
    UIImageView *inviteImg = [JLFactoryUtil imageViewWithImg:@"邀请好友"];
    inviteImg.userInteractionEnabled = YES;
    [inviteView addSubview:inviteImg];
    [inviteImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.bottom.equalTo(inviteView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *inviteL = [JLFactoryUtil labelWithText:@"Invite users" font:REGULARFONT(14) textColor:kMainTextColor];
    inviteL.userInteractionEnabled = YES;
    [inviteView addSubview:inviteL];
    [inviteL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.top.equalTo(inviteImg.mas_bottom).offset(4);
    }];
}

#pragma mark  执行者功能
- (void)setupExecutorModule {
    UIView *proView = [JLFactoryUtil view];
    [proView setCorner:8 withType:JLCornerTypeAllCorner];
    [proView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:proView];
    [proView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapPro = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyProject)];
    [proView addGestureRecognizer:tapPro];
    
    UIImageView *proimg = [JLFactoryUtil imageViewWithImg:@"订单"];
    proimg.userInteractionEnabled = YES;
    [proView addSubview:proimg];
    [proimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(proView);
        make.bottom.equalTo(proView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *proL = [JLFactoryUtil labelWithText:@"My project" font:REGULARFONT(14) textColor:kMainTextColor];
    proL.userInteractionEnabled = YES;
    [proView addSubview:proL];
    [proL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(proView);
        make.top.equalTo(proimg.mas_bottom).offset(4);
    }];
    
    UIView *inviteView = [JLFactoryUtil view];
    [inviteView setCorner:8 withType:JLCornerTypeAllCorner];
    [inviteView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:inviteView];
    [inviteView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(proView.mas_right).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapInvi = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyInvite)];
    [inviteView addGestureRecognizer:tapInvi];
    
    UIImageView *inviteImg = [JLFactoryUtil imageViewWithImg:@"邀请好友"];
    inviteImg.userInteractionEnabled = YES;
    [inviteView addSubview:inviteImg];
    [inviteImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.bottom.equalTo(inviteView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *inviteL = [JLFactoryUtil labelWithText:@"Invite users" font:REGULARFONT(14) textColor:kMainTextColor];
    inviteL.userInteractionEnabled = YES;
    [inviteView addSubview:inviteL];
    [inviteL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(inviteView);
        make.top.equalTo(inviteImg.mas_bottom).offset(4);
    }];
    
    UIView *recView = [JLFactoryUtil view];
    [recView setCorner:8 withType:JLCornerTypeAllCorner];
    [recView setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.06] offset:CGSizeMake(0, 2) radius:8];
    [self.contentView addSubview:recView];
    [recView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(inviteView.mas_right).offset(15);
        make.top.equalTo(self.inteView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake((kScreenWidth - 60)/3, 88));
    }];
    UITapGestureRecognizer *tapRec = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushMyRecommend)];
    [recView addGestureRecognizer:tapRec];
    
    UIImageView *recImg = [JLFactoryUtil imageViewWithImg:@"推荐"];
    recImg.userInteractionEnabled = YES;
    [recView addSubview:recImg];
    [recImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(recView);
        make.bottom.equalTo(recView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(28, 28));
    }];
    
    UILabel *recL = [JLFactoryUtil labelWithText:@"Recommend" font:REGULARFONT(14) textColor:kMainTextColor];
    recL.userInteractionEnabled = YES;
    [recView addSubview:recL];
    [recL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(recView);
        make.top.equalTo(recImg.mas_bottom).offset(4);
    }];
}

#pragma mark  Action
- (void)pushInfo {
    if ([KStringIsNull([UserManager shareManager].getUserInfo.identity) isEqualToString:@"1"]) {
        [self showLoginViewController];
    } else {
        [self.navigationController pushViewController:[BC_PersonInfoViewController new] animated:YES];
    }
}

- (void)clickMore {
    [self.navigationController pushViewController:[BC_MyRealEstateViewController new] animated:YES];
}

#pragma mark  我的帖子
- (void)pushMyPost {
    [self.navigationController pushViewController:[BC_MyPostViewController new] animated:YES];
}

#pragma mark  我的项目
- (void)pushMyProject {
    [self.tabBarController setSelectedIndex:1];
}

#pragma mark  我邀请的
- (void)pushMyInvite {
    [self.navigationController pushViewController:[BC_InviteUserViewController new] animated:YES];
}

#pragma mark  推荐
- (void)pushMyRecommend {
    [self.navigationController pushViewController:[BC_ExecutorRecommendViewController new] animated:YES];
}

#pragma mark  退出
- (void)signoutBtnClick {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure to log out?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            
    }]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [TUILogin logout:^{
            dispatch_async(dispatch_get_main_queue(), ^{
                [[UserManager shareManager] cleanUserInfo];
                [PPNetworkHelper setValue:@"" forHTTPHeaderField:@"Authorization"];
                [UserManager shareManager].isLogin = NO;
                [BaseFunction showAlert:@"Success"];
                [kAppDelegate setLoginRoot];
            });
        } fail:^(int code, NSString *msg) {
            
        }];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

#pragma mark  积分
- (void)pushIntegral {
    [self.navigationController pushViewController:[BC_IntegralListViewController new] animated:YES];
}

#pragma mark  我的收益
- (void)pushIncome {
    [self.navigationController pushViewController:[BC_ExecutorInComeViewController new] animated:YES];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    CGFloat y = scrollView.contentOffset.y;
    CGFloat alpha = y/kTopHeight;
    if (alpha > 1) {
        alpha = 1;
    }
    if (alpha < 0) {
        alpha = 0;
    }
    self.statusView.backgroundColor = [UIColor.whiteColor colorWithAlphaComponent:alpha];
}

#pragma mark  房产详情
- (void)pushHouseDetail:(UIGestureRecognizer *)ges {
    if (self.houseArr.count == 0) return;
    NSInteger tag = [ges view].tag - 200;
    NSDictionary *dic = self.houseArr[tag];
    BC_HouseDetailViewController *detail = [BC_HouseDetailViewController new];
    detail.houseid = KStringIsNull(dic[@"id"]);
    [self.navigationController pushViewController:detail animated:YES];
}

- (void)getUserInfo {
    
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getmyowninfo/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dataDic[@"data"];
            
            V2TIMUserFullInfo *info = [[V2TIMUserFullInfo alloc] init];
            info.faceURL = KStringIsNull(infoDic[@"user_avatar"]);
            info.nickName = KStringIsNull(infoDic[@"user_nickname"]);
            [[V2TIMManager sharedInstance] setSelfInfo:info succ:^{
                KLog(@"设置im头像成功");
            } fail:^(int code, NSString *desc) {
            }];
            
            BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
            umodel.identity = KStringIsNull(infoDic[@"user_identity"]);
            umodel.member_nickname = KStringIsNull(infoDic[@"user_nickname"]);
            umodel.member_avatar = KStringIsNull(infoDic[@"user_avatar"]);
            umodel.user_income = KStringIsNull(infoDic[@"user_income"]);
            umodel.user_invitecode = KStringIsNull(infoDic[@"user_invitecode"]);
            umodel.member_phone = KStringIsNull(infoDic[@"user_mobile"]);
            umodel.user_integral = KStringIsNull(infoDic[@"user_integral"]);
            [[UserManager shareManager] saveUserInfoWithModel:umodel];
            
            self.integralL.text = KStringIsNull(infoDic[@"user_integral"]).length == 0 ? @"0":KStringIsNull(infoDic[@"user_integral"]);
            self.incomeL.text = KStringIsNull(infoDic[@"user_income"]).length == 0 ? @"0":KStringIsNull(infoDic[@"user_income"]);
            
            //游客
            if ([KStringIsNull([UserManager shareManager].getUserInfo.identity) isEqualToString:@"1"]) {
                self.nicknameL.text = @"User name";
                self.phoneL.text = @"Guest Login";
                self.avatar.image = kAvatarPlaceHolderImg;
            } else {
                self.nicknameL.text = KStringIsNull(umodel.member_nickname).length == 0 ? @"-":KStringIsNull(umodel.member_nickname);
                NSString *phone = KStringIsNull(umodel.member_phone);
                if (phone.length >= 5) {
                    self.phoneL.text = kStringFormat(@"%@***%@", [phone substringToIndex:2], [phone substringFromIndex:phone.length - 2]);
                } else {
                    self.phoneL.text = phone;
                }
                [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(umodel.member_avatar)] placeholderImage:kAvatarPlaceHolderImg];
            }
            
            CGFloat imgW = (kScreenWidth - 45)/2;
            CGFloat imgH = imgW*120/165;
            
            for (UIImageView *imgv in self.houseItemArr) {
                [imgv removeFromSuperview];
            }
            [self.houseItemArr removeAllObjects];
            [self.houseArr removeAllObjects];

            NSDictionary *hdic = (NSDictionary *)infoDic[@"house_list"];
            self.estateNumL.text = kStringFormat(@"(%@)", KStringIsNull(hdic[@"house_total"]).length == 0 ? @"0":KStringIsNull(hdic[@"house_total"]));

            NSMutableArray *harr = [NSMutableArray arrayWithArray:(NSArray *)hdic[@"house_list"]];
            if (harr.count > 4) {
                harr = [harr subarrayWithRange:NSMakeRange(0, 4)].mutableCopy;
            }
            [self.houseArr addObjectsFromArray:harr];
            for (int i = 0; i < harr.count; i++) {
                NSDictionary *dic = harr[i];
                UIImageView *bgimg = [JLFactoryUtil imageView];
                bgimg.tag = 200 + i;
                bgimg.userInteractionEnabled = YES;
                [bgimg setCorner:8 withType:JLCornerTypeAllCorner];
                [self.contentView addSubview:bgimg];
                [bgimg mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(self.contentView.mas_left).offset(i%2 * (imgW + 15) + 15);
                    make.top.equalTo(self.estateL.mas_bottom).offset(i/2 * (imgH + 10) + 10);
                    make.size.mas_equalTo(CGSizeMake(imgW, imgH));
                }];
                NSArray *imgarr = (NSArray *)dic[@"images"];
                if (imgarr.count > 0) {
                    [bgimg sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(imgarr[0])]];
                }
                
                UITapGestureRecognizer *tapHouse = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushHouseDetail:)];
                [bgimg addGestureRecognizer:tapHouse];
                [self.houseItemArr addObject:bgimg];
                
                UILabel *areaL = [JLFactoryUtil labelWithText:KStringIsNull(dic[@"thecity"]) font:REGULARFONT(13) textColor:UIColor.whiteColor];
                areaL.textAlignment = NSTextAlignmentCenter;
                areaL.backgroundColor = [UIColor.blackColor colorWithAlphaComponent:0.4];
                areaL.layer.masksToBounds = YES;
                [areaL setCorner:4 withType:JLCornerTypeTopRightAndBottomRight];
                [bgimg addSubview:areaL];
                [areaL mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(bgimg.mas_left);
                    make.bottom.equalTo(bgimg.mas_bottom).offset(-10);
                    make.height.mas_equalTo(20);
                    make.width.mas_equalTo(70);
                }];
            }
            
            CGFloat marginTop = harr.count/2 * (imgH + 10) + 10;
            if (harr.count%2 == 0) {
                marginTop = 20 + (imgH + 10) * harr.count/2;
            } else {
                marginTop = 20 + (imgH + 10) * (harr.count/2 + 1);
            }
            [self.signoutBtn mas_updateConstraints:^(MASConstraintMaker *make) {
                make.top.equalTo(self.estateL.mas_bottom).offset(marginTop);
            }];
            
            if ([KStringIsNull([UserManager shareManager].getUserInfo.identity) isEqualToString:@"1"]) {
                self.signoutBtn.hidden = YES;
            } else {
                self.signoutBtn.hidden = NO;
            }
        } else {
            self.nicknameL.text = @"-";
            self.phoneL.text = @"-";
            self.avatar.image = kAvatarPlaceHolderImg;
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        self.nicknameL.text = @"-";
        self.phoneL.text = @"-";
        self.avatar.image = kAvatarPlaceHolderImg;
    }];
}
@end
