//
//  BC_IntegralListViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_IntegralListViewController.h"
#import "BC_IntegralListTableViewCell.h"
#import "BC_MineModuleModel.h"
@interface BC_IntegralListViewController ()<UITableViewDelegate, UITableViewDataSource, DZNEmptyDataSetSource, DZNEmptyDataSetDelegate>
@property (nonatomic, strong) UILabel *integralL;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, assign) NSInteger pageNum;
@property (nonatomic, strong) NSMutableArray *dataArr;
@end

@implementation BC_IntegralListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Integral";
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupUI];
    [self loadData];
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 200, kScreenWidth, kScreenHeight - kTopHeight - 200)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
    
    UIImageView *bgimg = [JLFactoryUtil imageViewWithImg:@"积分矩形"];
    [self.view addSubview:bgimg];
    [bgimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(4);
        make.top.equalTo(self.view.mas_top).offset(6);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 8, 150));
    }];
    
    UILabel *label = [JLFactoryUtil labelWithText:@"My scores" font:REGULARFONT(15) textColor:UIColor.whiteColor];
    [bgimg addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(bgimg.mas_left).offset(28);
        make.top.equalTo(bgimg.mas_top).offset(22);
    }];
    
    self.integralL = [JLFactoryUtil labelWithText:@"0" font:SEMIBOLDFONT(40) textColor:UIColor.whiteColor];
    [bgimg addSubview:self.integralL];
    [self.integralL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(bgimg.mas_left).offset(28);
        make.top.equalTo(label.mas_bottom).offset(16);
    }];
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"Record" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [self.view addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(15);
        make.top.equalTo(bgimg.mas_bottom).offset(10);
    }];
    self.integralL.text = KStringIsNull([UserManager shareManager].getUserInfo.user_integral).length == 0 ? @"0":KStringIsNull([UserManager shareManager].getUserInfo.user_integral);
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_IntegralListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_IntegralListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 80;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    return [UIImage imageNamed:@"空数据"];
}

- (BOOL)emptyDataSetShouldAllowScroll:(UIScrollView *)scrollView {
    return YES;
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getListData];
}

- (void)loadMoreData {
    self.pageNum ++;
    [self getListData];
}

- (void)getListData {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getintegrallist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_MineModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_MineModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
