//
//  BC_ExecutorInComeViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/24.
//

#import "BC_ExecutorInComeViewController.h"

#import "BC_ExecutorIncomeTableViewCell.h"

#import "BC_MineModuleModel.h"
@interface BC_ExecutorInComeViewController ()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UIView *navBarView;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UILabel *incomeL;
@property (nonatomic, assign) CGFloat contentOffY;
@property (nonatomic, strong) UILabel *navL;
@property (nonatomic, strong) UIButton *cusBackBtn;

@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@end

@implementation BC_ExecutorInComeViewController
- (UIStatusBarStyle)preferredStatusBarStyle {
    return self.contentOffY > 20 ? UIStatusBarStyleDefault:UIStatusBarStyleLightContent;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    self.contentOffY = 0;
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupUI];
    [self loadData];
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    } else {
        // Fallback on earlier versions
    }
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
    
    self.navBarView = [JLFactoryUtil view];
    self.navBarView.backgroundColor = [UIColor.whiteColor colorWithAlphaComponent:0];
    [self.view addSubview:self.navBarView];
    [self.navBarView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.equalTo(self.view);
        make.height.mas_equalTo(kTopHeight);
    }];
    
    UIButton *bbtn = [JLFactoryUtil buttonWithImage:@"whiteback"];
    [bbtn addTarget:self action:@selector(backBtnClicked) forControlEvents:UIControlEventTouchUpInside];
    [self.navBarView addSubview:bbtn];
    [bbtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.navBarView.mas_left).offset(15);
        make.top.equalTo(self.navBarView.mas_top).offset(kStatusBarHeight + 6);
        make.size.mas_equalTo(CGSizeMake(30, 30));
    }];
    _cusBackBtn = bbtn;
    
    UILabel *navL = [JLFactoryUtil labelWithText:@"Income" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
    [self.navBarView addSubview:navL];
    [navL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.navBarView);
        make.centerY.equalTo(bbtn);
    }];
    _navL = navL;
    
    UIView *header = [JLFactoryUtil view];
    header.frame = CGRectMake(0, 0, kScreenWidth, 300);
    self.tableView.tableHeaderView = header;
    
    UIImageView *bgimg = [JLFactoryUtil imageViewWithImg:@"收益背景"];
    bgimg.userInteractionEnabled = YES;
    [header addSubview:bgimg];
    [bgimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.equalTo(header);
        make.height.mas_equalTo(240);
    }];
    
    self.incomeL = [JLFactoryUtil labelWithText:@"0" font:SEMIBOLDFONT(36) textColor:UIColor.whiteColor];
    [bgimg addSubview:self.incomeL];
    [self.incomeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(bgimg);
        make.bottom.equalTo(bgimg.mas_bottom).offset(-70);
    }];
    
    UILabel *mpL = [JLFactoryUtil labelWithText:@"My purse" font:REGULARFONT(15) textColor:UIColor.whiteColor];
    [bgimg addSubview:mpL];
    [mpL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(bgimg);
        make.bottom.equalTo(bgimg.mas_bottom).offset(-35);
    }];
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"Amount turnover" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [header addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(header.mas_left).offset(15);
        make.top.equalTo(bgimg.mas_bottom).offset(20);
    }];
    
    self.incomeL.text = KStringIsNull([UserManager shareManager].getUserInfo.user_income).length == 0 ? @"0":KStringIsNull([UserManager shareManager].getUserInfo.user_income);
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_ExecutorIncomeTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_ExecutorIncomeTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 80;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    CGFloat y = scrollView.contentOffset.y;
    self.contentOffY = y;
    CGFloat alpha = y/kTopHeight;
    if (alpha > 1) {
        alpha = 1;
    }
    if (alpha < 0) {
        alpha = 0;
    }
    if (self.contentOffY > 20) {
        [self.cusBackBtn setImage:[UIImage imageNamed:@"blackback"] forState:0];
        self.navL.textColor = kMainTextColor;
    } else {
        [self.cusBackBtn setImage:[UIImage imageNamed:@"whiteback"] forState:0];
        self.navL.textColor = UIColor.whiteColor;
    }
    self.navBarView.backgroundColor = [UIColor.whiteColor colorWithAlphaComponent:alpha];
    [self setNeedsStatusBarAppearanceUpdate];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (void)getPageData {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getincomelist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"20"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_MineModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_MineModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
