//
//  BC_AddMyGroupViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/21.
//

#import "BC_AddMyGroupViewController.h"
#import "BC_GroupChatViewController.h"

#import "BC_AddGroupHouseListTableViewCell.h"
#import "BC_HouseInfoModel.h"
#import "TUIChatConversationModel.h"
@import ImSDK_Plus;
@interface BC_AddMyGroupViewController ()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@property (nonatomic, strong) BC_HouseInfoModel *selModel;
@end

@implementation BC_AddMyGroupViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"New group chat";
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupUI];
    [self loadData];
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - HOME_INDICATOR_HEIGHT - 60)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
    
    UIButton *submitBtn = [JLFactoryUtil buttonWithTitle:@"Create" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
    [submitBtn setCorner:24 withType:JLCornerTypeAllCorner];
    submitBtn.backgroundColor = kMainThemeColor;
    [submitBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
    [submitBtn addTarget:self action:@selector(submitBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:submitBtn];
    [submitBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT-6);
        make.centerX.equalTo(self.view);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
    }];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_AddGroupHouseListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_AddGroupHouseListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 130;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    for (BC_HouseInfoModel *model in self.dataArr) {
        model.isSelect = NO;
    }
    BC_HouseInfoModel *selmodel = self.dataArr[indexPath.row];
    selmodel.isSelect = YES;
    self.selModel = selmodel;
    [self.tableView reloadData];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getListData];
}

- (void)loadMoreData {
    self.pageNum ++;
    [self getListData];
}

- (void)getListData {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/gethouselist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"20"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_HouseInfoModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
                self.selModel = nil;
            } else {
                NSMutableArray *tempArr = [BC_HouseInfoModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}

#pragma mark  创建群聊
- (void)submitBtnClick {
    BOOL ifselect = NO;
    NSString *hid = @"";
    for (BC_HouseInfoModel *model in self.dataArr) {
        if (model.isSelect) {
            ifselect = YES;
            hid = KStringIsNull(model.itemid);
        }
    }
    
    if (!ifselect) {
        [BaseFunction showAlert:@"Please select the real estate"];
        return;
    }
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/createhourseimgroup/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":hid} success:^(id responseObject) {
        KLog(@"创建群成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dataDic[@"data"];
            [BaseFunction showAlert:@"Create success"];
            TUIChatConversationModel *model = [[TUIChatConversationModel alloc] init];
            model.groupID = KStringIsNull(infoDic[@"im_groupid"]);
            model.title = KStringIsNull(infoDic[@"hourse_name"]);
            BC_GroupChatViewController *chat = [[BC_GroupChatViewController alloc] init];
            chat.conversationData = model;
            [self.navigationController pushViewController:chat animated:YES];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
