//
//  BC_HouseDetailViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_HouseDetailViewController.h"
#import "BC_AddHouseViewController.h"
#import "BC_GroupChatViewController.h"

#import "TUIChatConversationModel.h"
#import "ZlinkPhotoBrowserView.h"
#import "SDCycleScrollView.h"
#import "BC_HouseDetailTeneTableViewCell.h"
#import "BC_HouseInfoModel.h"

@interface BC_HouseDetailViewController ()<SDCycleScrollViewDelegate, UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) SDCycleScrollView *bannerScrollView;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UILabel *currentNumL;
@property (nonatomic, strong) YYLabel *priceL;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) NSMutableArray *bannerArr;

@property (nonatomic, strong) BC_HouseInfoModel *infoModel;
@end

@implementation BC_HouseDetailViewController

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self getPageData];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Property details";
    self.bannerArr = [NSMutableArray array];
    [self setupUI];
    
}

- (void)setupUI {
    
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - 81 - HOME_INDICATOR_HEIGHT)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    
    CGFloat imgH = kScreenWidth*272/375;
    UIView *header = [JLFactoryUtil view];
    header.frame = CGRectMake(0, 0, kScreenWidth, 130+imgH);
    self.tableView.tableHeaderView = header;
    
    self.bannerScrollView= [SDCycleScrollView cycleScrollViewWithFrame:CGRectMake(0, 0, kScreenWidth, imgH) delegate:self placeholderImage:nil];
    self.bannerScrollView.backgroundColor = UIColor.whiteColor;
    self.bannerScrollView.titlesGroup = nil;
    self.bannerScrollView.showPageControl = NO;
    self.bannerScrollView.autoScrollTimeInterval = 3;
    self.bannerScrollView.bannerImageViewContentMode = UIViewContentModeScaleAspectFill;
    [header addSubview:self.bannerScrollView];
    
    self.currentNumL = [JLFactoryUtil labelWithText:@"0/0" font:MEDIUMFONT(14) textColor:UIColor.whiteColor];
    self.currentNumL.textAlignment = NSTextAlignmentCenter;
    self.currentNumL.backgroundColor = [UIColor.blackColor colorWithAlphaComponent:0.5];
    self.currentNumL.layer.masksToBounds = YES;
    self.currentNumL.layer.cornerRadius = 10;
    [header addSubview:self.currentNumL];
    [self.currentNumL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.right.equalTo(self.bannerScrollView).offset(-10);
        make.size.mas_equalTo(CGSizeMake(55, 20));
    }];
    
    self.priceL = [YYLabel new];
    [header addSubview:self.priceL];
    [self.priceL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(header.mas_left).offset(15);
        make.top.equalTo(self.bannerScrollView.mas_bottom).offset(10);
    }];
    
    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [header addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.priceL);
        make.top.equalTo(self.priceL.mas_bottom).offset(10);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kMainTextColor];
    [header addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(areaImg);
        make.left.equalTo(areaImg.mas_right).offset(5);
        make.right.equalTo(header.mas_right).offset(-10);
    }];
    
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"#F8F8F8");
    [header addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(header);
        make.top.equalTo(areaImg.mas_bottom).offset(20);
        make.height.mas_equalTo(10);
    }];
    
    UILabel *label = [JLFactoryUtil labelWithText:@"Tenant Information" font:MEDIUMFONT(17) textColor:kMainTextColor];
    [header addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(header.mas_left).offset(15);
        make.top.equalTo(line.mas_bottom).offset(15);
    }];
    
    UIImageView *menuImg = [JLFactoryUtil imageViewWithImg:@"房产详情菜单"];
    menuImg.userInteractionEnabled = YES;
    [self.view addSubview:menuImg];
    [menuImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.view);
        make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT);
        make.height.mas_equalTo(79);
    }];
    
    UIButton *delBtn = [JLFactoryUtil buttonWithTitle:@"Delete" font:MEDIUMFONT(14) textColor:kMainTextColor];
    [delBtn addTarget:self action:@selector(delBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [menuImg addSubview:delBtn];
    [delBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(menuImg);
        make.bottom.equalTo(menuImg.mas_bottom).offset(-10);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth/3, 50));
    }];
    
    UIView *groupView = [JLFactoryUtil view];
    groupView.backgroundColor = UIColor.clearColor;
    [menuImg addSubview:groupView];
    [groupView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(menuImg);
        make.bottom.equalTo(menuImg.mas_bottom).offset(0);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth/3, 75));
    }];
    UITapGestureRecognizer *tapgroup = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushChat)];
    [groupView addGestureRecognizer:tapgroup];
    
    UIImageView *groupImg = [JLFactoryUtil imageViewWithImg:@"群聊选中"];
    groupImg.userInteractionEnabled = YES;
    [groupView addSubview:groupImg];
    [groupImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(groupView);
        make.bottom.equalTo(groupView.mas_centerY).offset(-1);
        make.size.mas_equalTo(CGSizeMake(22, 22));
    }];
    
    UILabel *groupL = [JLFactoryUtil labelWithText:@"Group" font:MEDIUMFONT(14) textColor:kMainTextColor];
    groupL.userInteractionEnabled = YES;
    [groupView addSubview:groupL];
    [groupL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(groupView);
        make.top.equalTo(groupView.mas_centerY).offset(1);
    }];
    
    UIButton *editBtn = [JLFactoryUtil buttonWithTitle:@"Edit" font:MEDIUMFONT(14) textColor:kMainTextColor];
    [editBtn addTarget:self action:@selector(editBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [menuImg addSubview:editBtn];
    [editBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(groupView.mas_right).offset(0);
        make.bottom.equalTo(menuImg.mas_bottom).offset(-10);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth/3, 50));
    }];
        
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_HouseDetailTeneTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_HouseDetailTeneTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_infoModel.tenant[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 145;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _infoModel.tenant.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)cycleScrollView:(SDCycleScrollView *)cycleScrollView didSelectItemAtIndex:(NSInteger)index {
    NSMutableArray *itemarr = [NSMutableArray array];
    for (NSString *url in self.bannerArr) {
        ZlinkPhotoGroupItem *item = [ZlinkPhotoGroupItem new];
        item.thumbView = cycleScrollView;
        item.largeImageURL = [NSURL URLWithString:KStringIsNull(url)];
        [itemarr addObject:item];
    }
    ZlinkPhotoBrowserView *browser = [[ZlinkPhotoBrowserView alloc] initWithGroupItems:itemarr];
    [browser presentFromImageView:cycleScrollView toContainer:self.navigationController.view animated:YES index:index completion:^{
    }];
}

- (void)cycleScrollView:(SDCycleScrollView *)cycleScrollView didScrollToIndex:(NSInteger)index {
    NSString *idx = kStringFormat(@"%zd", index+1);
    self.currentNumL.text = kStringFormat(@"%@/%zd", idx, self.bannerArr.count);
}

- (void)pushChat {
    TUIChatConversationModel *model = [[TUIChatConversationModel alloc] init];
    model.groupID = KStringIsNull(_infoModel.im_groupid);
    model.title = KStringIsNull(_infoModel.name);
    BC_GroupChatViewController *chat = [[BC_GroupChatViewController alloc] init];
    chat.conversationData = model;
    [self.navigationController pushViewController:chat animated:YES];
}

#pragma mark  删除房产
- (void)delBtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Confirm to delete the property?" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)cancelAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

#pragma mark  删除房产
- (void)confirmAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/gethousedelete/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(_infoModel.itemid)} success:^(id responseObject) {
        KLog(@"删除成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Delete success"];
            [kNotificationCenter postNotificationName:@"refreshListAfterSubmit" object:nil];
            [self.navigationController popViewControllerAnimated:YES];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

#pragma mark  编辑房产
- (void)editBtnClick {
    BC_AddHouseViewController *edit = [BC_AddHouseViewController new];
    edit.isedit = YES;
    edit.editModel = self.infoModel;
    [self.navigationController pushViewController:edit animated:YES];

}

- (void)configUIData {
    [self.bannerArr removeAllObjects];
    [self.bannerArr addObjectsFromArray:_infoModel.images];
    self.bannerScrollView.imageURLStringsGroup = self.bannerArr;
    self.currentNumL.text = kStringFormat(@"1/%zd", self.bannerArr.count);

    NSMutableAttributedString *price = [[NSMutableAttributedString alloc] initWithString:kStringFormat(@"$%@", KStringIsNull(_infoModel.price))];
    [price setYy_font:MEDIUMFONT(20)];
    [price setYy_color:kMainTextColor];
    self.priceL.attributedText = price;
    self.areaL.text = kStringFormat(@"%@ %@ %@", KStringIsNull(_infoModel.province), KStringIsNull(_infoModel.city), KStringIsNull(_infoModel.place));
    [self.tableView reloadData];
    
}

- (void)getPageData {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/gethousedetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.houseid)} success:^(id responseObject) {
        KLog(@"详情成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            self.infoModel = [BC_HouseInfoModel mj_objectWithKeyValues:dataDic[@"data"]];
            [self configUIData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}
@end
