//
//  BC_ManageMyGroupViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/21.
//

#import "BC_ManageMyGroupViewController.h"
#import "BC_ChangeGroupNameViewController.h"
#import "BC_ChangeMyNameInGroupViewController.h"
#import "BC_ManageMyGroupMemberViewController.h"
#import "BC_SearchUserViewController.h"

#import <TZImagePickerController.h>
#import "BC_GroupMemberCollectionViewCell.h"

#import "BC_GroupInfoModel.h"
@import ImSDK_Plus;
@interface BC_ManageMyGroupViewController ()<UICollectionViewDelegate, UICollectionViewDataSource, TZImagePickerControllerDelegate>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *groupNameL;
@property (nonatomic, strong) UILabel *groupNumL;
@property (nonatomic, strong) UICollectionView *memberCV;
@property (nonatomic, strong) UIImageView *logoImg;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) UILabel *disL;
@property (nonatomic, strong) UILabel *addressL;

@property (nonatomic, strong) UILabel *rowGroupNameL;
@property (nonatomic, strong) UIImageView *rowGroupLogo;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) NSMutableArray *memberArr;
@property (nonatomic, strong) BC_GroupInfoModel *infoModel;

@end

@implementation BC_ManageMyGroupViewController

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self getGroupInfo];
    [self getMyNameInGroup];
}

- (void)dealloc {
    [kNotificationCenter removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Group management";
    [self setupUI];
    self.memberArr = [NSMutableArray array];
    [kNotificationCenter addObserver:self selector:@selector(getMemberList) name:@"updateMemberList" object:nil];
    [self getMemberList];
}

#pragma mark  群名片
- (void)getMyNameInGroup {
    [[V2TIMManager sharedInstance] getGroupMembersInfo:KStringIsNull(self.groupid) memberList:@[KStringIsNull([UserManager shareManager].getUserInfo.TUIIden)] succ:^(NSArray<V2TIMGroupMemberFullInfo *> *memberList) {
        if (memberList.count > 0) {
            V2TIMGroupMemberFullInfo *info = memberList[0];
            self.nicknameL.text = KStringIsNull(info.nameCard).length==0?KStringIsNull(info.nickName):KStringIsNull(info.nameCard);
        }
    } fail:^(int code, NSString *desc) {
        
    }];
}

#pragma mark  成员列表
- (void)getMemberList {
    [[V2TIMManager sharedInstance] getGroupMemberList:KStringIsNull(self.groupid) filter:V2TIM_GROUP_MEMBER_FILTER_ALL nextSeq:0 succ:^(uint64_t nextSeq, NSArray<V2TIMGroupMemberFullInfo *> *memberList) {
        [self.memberArr removeAllObjects];
        for (V2TIMGroupMemberFullInfo *model in memberList) {
            NSString *name = KStringIsNull(model.nameCard).length == 0 ? KStringIsNull(model.nickName):KStringIsNull(model.nameCard);
            NSDictionary *dic = @{@"name":KStringIsNull(name), @"avatar":KStringIsNull(model.faceURL), @"type":@""};
            [self.memberArr addObject:dic];
        }
        NSDictionary *dic = @{@"name":@"", @"avatar":@"", @"type":@"add"};
        [self.memberArr addObject:dic];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.memberCV reloadData];
            [self.memberCV layoutIfNeeded];
            [self.memberCV mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.memberCV.collectionViewLayout.collectionViewContentSize.height);
            }];
        });
    } fail:^(int code, NSString *desc) {
        
    }];
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:26 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.contentView.mas_top).offset(15);
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(52, 52));
    }];
    
    self.groupNameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.groupNameL];
    [self.groupNameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.groupNumL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.groupNumL];
    [self.groupNumL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.avatar.mas_centerY).offset(2);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    UIView *memberView = [JLFactoryUtil view];
    memberView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:memberView];
    [memberView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.avatar.mas_bottom).offset(15);
        make.height.mas_equalTo(36);
    }];
    
    UILabel *memL = [JLFactoryUtil labelWithText:@"Group member" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [memberView addSubview:memL];
    [memL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(memberView);
        make.left.equalTo(memberView.mas_left).offset(15);
    }];
    
    UICollectionViewFlowLayout *taglayout = [UICollectionViewFlowLayout new];
    taglayout.scrollDirection = UICollectionViewScrollDirectionVertical;
    taglayout.minimumInteritemSpacing = 10;
    taglayout.minimumLineSpacing = 10;
    taglayout.itemSize = CGSizeMake((kScreenWidth-60)/5, 85);
    taglayout.sectionInset = UIEdgeInsetsMake(10, 10, 10, 10);
    self.memberCV = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:taglayout];
    self.memberCV.delegate = self;
    self.memberCV.dataSource = self;
    self.memberCV.backgroundColor = UIColor.whiteColor;
    self.memberCV.showsVerticalScrollIndicator = NO;
    self.memberCV.scrollEnabled = NO;
    [self.memberCV registerClass:[BC_GroupMemberCollectionViewCell class] forCellWithReuseIdentifier:@"membercell"];
    [self.contentView addSubview:self.memberCV];
    [self.memberCV mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView);
        make.left.equalTo(self.contentView);
        make.top.equalTo(memberView.mas_bottom).offset(10);
        make.height.mas_equalTo(1);
    }];
    
    UIView *realView = [JLFactoryUtil view];
    realView.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:realView];
    [realView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.top.equalTo(self.memberCV.mas_bottom).offset(5);
        make.height.mas_equalTo(36);
    }];
    
    UILabel *realestateL = [JLFactoryUtil labelWithText:@"Real estate" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [realView addSubview:realestateL];
    [realestateL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(realView);
        make.left.equalTo(realView.mas_left).offset(15);
    }];
    
    self.logoImg = [JLFactoryUtil imageView];
    [self.logoImg setCorner:5 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.logoImg];
    [self.logoImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(realView.mas_bottom).offset(15);
        make.size.mas_equalTo(CGSizeMake(88, 88));
    }];
    
    UIImageView *disImg = [JLFactoryUtil imageViewWithImg:@"价格"];
    [self.contentView addSubview:disImg];
    [disImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.logoImg);
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.disL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.disL];
    [self.disL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(disImg.mas_right).offset(5);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(self.logoImg);
    }];
    
    self.areaL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.bottom.equalTo(disImg.mas_top).offset(-10);
    }];
    
    UIImageView *addrImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.contentView addSubview:addrImg];
    [addrImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(disImg.mas_bottom).offset(10);
        make.left.equalTo(self.logoImg.mas_right).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.addressL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.addressL];
    [self.addressL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(addrImg.mas_right).offset(5);
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(addrImg);
    }];
    
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"#F8F8F8");
    [self.contentView addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView);
        make.right.equalTo(self.contentView);
        make.top.equalTo(self.logoImg.mas_bottom).offset(15);
        make.height.mas_equalTo(36);
    }];
    
    UILabel *gsetting = [JLFactoryUtil labelWithText:@"Group settings" font:MEDIUMFONT(16) textColor:kMainTextColor];
    [line addSubview:gsetting];
    [gsetting mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(line);
        make.left.equalTo(line.mas_left).offset(15);
    }];
    
    //如果是群主，展示群名 群logo 和昵称，如果是成员只展示昵称
    UILabel *nameL;
//    if (群主) {
        UILabel *geL1 = [JLFactoryUtil labelWithText:@"Group name" font:REGULARFONT(15) textColor:kMainTextColor];
        [self.contentView addSubview:geL1];
        [geL1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(line.mas_bottom).offset(20);
        }];

        UIImageView *arrow1 = [JLFactoryUtil imageViewWithImg:@"更多"];
        [self.contentView addSubview:arrow1];
        [arrow1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(geL1);
            make.right.equalTo(self.contentView.mas_right).offset(-15);
            make.size.mas_equalTo(CGSizeMake(16, 16));
        }];

        self.rowGroupNameL = [JLFactoryUtil labelWithText:@"" font:MEDIUMFONT(15) textColor:kColorWithHex(@"#777777")];
        [self.contentView addSubview:self.rowGroupNameL];
        [self.rowGroupNameL mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(geL1);
            make.right.equalTo(arrow1.mas_left).offset(-10);
        }];

        UIView *line1 = [JLFactoryUtil view];
        line1.backgroundColor = kColorWithHex(@"#F2F2F2");
        [self.contentView addSubview:line1];
        [line1 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(geL1.mas_bottom).offset(20);
            make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
        }];

        UILabel *geL2 = [JLFactoryUtil labelWithText:@"Group avatar" font:REGULARFONT(15) textColor:kMainTextColor];
        [self.contentView addSubview:geL2];
        [geL2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(line1.mas_bottom).offset(20);
        }];

        UIImageView *arrow2 = [JLFactoryUtil imageViewWithImg:@"更多"];
        [self.contentView addSubview:arrow2];
        [arrow2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(geL2);
            make.right.equalTo(self.contentView.mas_right).offset(-15);
            make.size.mas_equalTo(CGSizeMake(16, 16));
        }];

        self.rowGroupLogo = [JLFactoryUtil imageView];
        [self.rowGroupLogo setCorner:18 withType:JLCornerTypeAllCorner];
        [self.contentView addSubview:self.rowGroupLogo];
        [self.rowGroupLogo mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.equalTo(geL2);
            make.right.equalTo(arrow2.mas_left).offset(-10);
            make.size.mas_equalTo(CGSizeMake(36, 36));
        }];

        UIView *line2 = [JLFactoryUtil view];
        line2.backgroundColor = kColorWithHex(@"#F2F2F2");
        [self.contentView addSubview:line2];
        [line2 mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(geL2.mas_bottom).offset(20);
            make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
        }];

        nameL = [JLFactoryUtil labelWithText:@"My nickname" font:REGULARFONT(15) textColor:kMainTextColor];
        [self.contentView addSubview:nameL];
        [nameL mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView.mas_left).offset(15);
            make.top.equalTo(line2.mas_bottom).offset(20);
        }];
//    } else {
//        nameL = [JLFactoryUtil labelWithText:@"My nickname" font:REGULARFONT(15) textColor:kMainTextColor];
//        [self.contentView addSubview:nameL];
//        [nameL mas_makeConstraints:^(MASConstraintMaker *make) {
//            make.left.equalTo(self.contentView.mas_left).offset(15);
//            make.top.equalTo(line.mas_bottom).offset(20);
//        }];
//    }
    
    UIImageView *arrow = [JLFactoryUtil imageViewWithImg:@"更多"];
    [self.contentView addSubview:arrow];
    [arrow mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(nameL);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];

    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(nameL);
        make.right.equalTo(arrow.mas_left).offset(-10);
    }];

//    UIView *boldLine = [JLFactoryUtil view];
//    boldLine.backgroundColor = kColorWithHex(@"#F8F8F8");
//    [self.contentView addSubview:boldLine];
//    [boldLine mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.left.equalTo(self.contentView);
//        make.right.equalTo(self.contentView);
//        make.top.equalTo(nameL.mas_bottom).offset(20);
//        make.height.mas_equalTo(36);
//    }];
//
//    UILabel *ginfo = [JLFactoryUtil labelWithText:@"Group information" font:MEDIUMFONT(16) textColor:kMainTextColor];
//    [boldLine addSubview:ginfo];
//    [ginfo mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.centerY.equalTo(boldLine);
//        make.left.equalTo(boldLine.mas_left).offset(15);
//    }];
//
//    UILabel *recordL = [JLFactoryUtil labelWithText:@"Chat record" font:REGULARFONT(15) textColor:kMainTextColor];
//    [self.contentView addSubview:recordL];
//    [recordL mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.left.equalTo(self.contentView.mas_left).offset(15);
//        make.top.equalTo(boldLine.mas_bottom).offset(20);
//    }];
//
//    UIImageView *arrow3 = [JLFactoryUtil imageViewWithImg:@"更多"];
//    [self.contentView addSubview:arrow3];
//    [arrow3 mas_makeConstraints:^(MASConstraintMaker *make) {
//        make.centerY.equalTo(recordL);
//        make.right.equalTo(self.contentView.mas_right).offset(-15);
//        make.size.mas_equalTo(CGSizeMake(16, 16));
//    }];

    //群主展示解散按钮 群员展示退出群聊
    UIButton *disbandBtn = [JLFactoryUtil buttonWithTitle:@"Disband group" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
    [disbandBtn setCorner:24 withType:JLCornerTypeAllCorner];
    disbandBtn.backgroundColor = kMainThemeColor;
    [disbandBtn setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 4) radius:8];
    [disbandBtn addTarget:self action:@selector(disbandBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:disbandBtn];
    [disbandBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(30);
        make.top.equalTo(nameL.mas_bottom).offset(20);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 60, 48));
    }];

    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(disbandBtn.mas_bottom).offset(30);
    }];
    
    UIView *gnameClear = [JLFactoryUtil view];
    gnameClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:gnameClear];
    [gnameClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(geL1);
        make.height.mas_equalTo(40);
    }];
    UITapGestureRecognizer *tapName = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(changeGroupName)];
    [gnameClear addGestureRecognizer:tapName];

    UIView *glogoClear = [JLFactoryUtil view];
    glogoClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:glogoClear];
    [glogoClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(geL2);
        make.height.mas_equalTo(40);
    }];
    UITapGestureRecognizer *tapLogo = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(changeGroupLogo)];
    [glogoClear addGestureRecognizer:tapLogo];

    UIView *nicknameClear = [JLFactoryUtil view];
    nicknameClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:nicknameClear];
    [nicknameClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(nameL);
        make.height.mas_equalTo(40);
    }];
    UITapGestureRecognizer *tapnick = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(changeNickname)];
    [nicknameClear addGestureRecognizer:tapnick];
    
}

#pragma mark  UICollectionView Delegate
- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    NSDictionary *dic = self.memberArr[indexPath.item];
    if ([KStringIsNull(dic[@"type"]) isEqualToString:@"add"]) {
        BC_SearchUserViewController *search = [BC_SearchUserViewController new];
        search.groupid = KStringIsNull(self.groupid);
        [self.navigationController pushViewController:search animated:YES];
    } else {
        BC_ManageMyGroupMemberViewController *member = [BC_ManageMyGroupMemberViewController new];
        member.groupid = KStringIsNull(self.groupid);
        [self.navigationController pushViewController:member animated:YES];
    }
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    BC_GroupMemberCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"membercell" forIndexPath:indexPath];
    [cell setDic:_memberArr[indexPath.item]];
    return cell;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.memberArr.count;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView {
    return 1;
}

#pragma mark  修改logo
- (void)changeGroupLogo {
    TZImagePickerController *imagePickerVc = [[TZImagePickerController alloc] initWithMaxImagesCount:1 delegate:self];
    imagePickerVc.barItemTextColor = [UIColor whiteColor];
    [imagePickerVc.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName : [UIColor whiteColor]}];
    imagePickerVc.navigationBar.tintColor = [UIColor whiteColor];
    if (@available(iOS 15.0, *)) {
        UINavigationBarAppearance *barApp = [UINavigationBarAppearance new];
        barApp.backgroundColor = [UIColor whiteColor];
        barApp.shadowColor = [UIColor clearColor];
        imagePickerVc.navigationController.navigationBar.scrollEdgeAppearance = barApp;
        imagePickerVc.navigationController.navigationBar.standardAppearance = barApp;
    } else {
        imagePickerVc.navigationBar.barTintColor = [UIColor whiteColor];
    }
    imagePickerVc.oKButtonTitleColorDisabled = [UIColor lightGrayColor];
    imagePickerVc.navigationBar.translucent = NO;
    imagePickerVc.iconThemeColor = kMainThemeColor;
    imagePickerVc.showSelectedIndex = YES;
    imagePickerVc.allowTakePicture = YES;
    imagePickerVc.allowTakeVideo = NO;   // 在内部显示拍视频按
    imagePickerVc.showPhotoCannotSelectLayer = YES;
    imagePickerVc.cannotSelectLayerColor = [[UIColor whiteColor] colorWithAlphaComponent:0.8];
    imagePickerVc.allowPickingVideo = NO;
    imagePickerVc.allowPickingOriginalPhoto = YES;
    imagePickerVc.allowPickingImage = YES;
    imagePickerVc.allowPickingGif = NO;
    imagePickerVc.allowCrop = YES;
    imagePickerVc.cropRect = CGRectMake(0, (kScreenHeight - kScreenWidth)/2, kScreenWidth, kScreenWidth);
    imagePickerVc.statusBarStyle = UIStatusBarStyleDefault;
    imagePickerVc.preferredLanguage = @"en";
//    imagePickerVc.modalPresentationStyle = UIModalPresentationFullScreen;
    [self presentViewController:imagePickerVc animated:YES completion:nil];
}

- (void)tz_imagePickerControllerDidCancel:(TZImagePickerController *)picker {
}

- (void)imagePickerController:(TZImagePickerController *)picker didFinishPickingPhotos:(NSArray<UIImage *> *)photos sourceAssets:(NSArray *)assets isSelectOriginalPhoto:(BOOL)isSelectOriginalPhoto {
    if (photos.count > 0) {
        [self uploadImg:photos[0]];
    }
}

#pragma mark  修改群名称
- (void)changeGroupName {
    BC_ChangeGroupNameViewController *group = [BC_ChangeGroupNameViewController new];
    group.groupid = KStringIsNull(self.groupid);
    if (_infoModel.groupinfo.count > 0) {
        BC_GroupInfoModel *groupModel = _infoModel.groupinfo.firstObject;
        group.groupAvatar = KStringIsNull(groupModel.FaceUrl);
    }
    [self.navigationController pushViewController:group animated:YES];
}

#pragma mark  修改昵称
- (void)changeNickname {
    BC_ChangeMyNameInGroupViewController *group = [BC_ChangeMyNameInGroupViewController new];
    group.groupid = KStringIsNull(self.groupid);
    group.myAvatar = KStringIsNull([UserManager shareManager].getUserInfo.member_avatar);
    [self.navigationController pushViewController:group animated:YES];
}

#pragma mark  解散
- (void)disbandBtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"After deletion, this group message will no longer be received" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelQuitAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmDisbandAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)cancelQuitAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)confirmDisbandAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    [[V2TIMManager sharedInstance] dismissGroup:KStringIsNull(self.groupid) succ:^{
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    } fail:^(int code, NSString *desc) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            [BaseFunction showAlert:KStringIsNull(desc)];
        });
    }];
}

- (void)configUIData {
    
    BC_GroupInfoModel *detainfo = [BC_GroupInfoModel mj_objectWithKeyValues:_infoModel.datainfo];
    self.areaL.text = KStringIsNull(detainfo.name);
    if ([KStringIsNull(detainfo.price) isEqualToString:@""]) {
        self.disL.text = kStringFormat(@"%@", KStringIsNull(detainfo.service_type));
    } else {
        self.disL.text = kStringFormat(@"%@", KStringIsNull(detainfo.price));
    }
    self.addressL.text = KStringIsNull(detainfo.province);
    if (detainfo.images.count > 0) {
        [self.logoImg sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(detainfo.images[0])]];
    }
    if (_infoModel.groupinfo.count > 0) {
        BC_GroupInfoModel *groupModel = _infoModel.groupinfo.firstObject;
        self.rowGroupNameL.text = KStringIsNull(groupModel.groupName);
        [self.rowGroupLogo sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(groupModel.FaceUrl)]];
        self.groupNameL.text = KStringIsNull(groupModel.groupName);
        self.groupNumL.text = KStringIsNull(self.groupid);
        [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(groupModel.FaceUrl)]];
    }
}

- (void)getGroupInfo {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/getimdetail/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"groupid":KStringIsNull(self.groupid)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dic1 = (NSDictionary *)resdic[@"data"];
            NSDictionary *infoDic = (NSDictionary *)dic1[@"data"];
            self.infoModel = [BC_GroupInfoModel mj_objectWithKeyValues:infoDic];
            [self configUIData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)uploadImg:(UIImage *)avatar {
    [self.view endEditing:YES];
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/uploadimage/",KMainUrl];
    [PPNetworkHelper uploadImagesWithURL:url parameters:@{} name:@"file" images:@[avatar] fileNames:nil imageScale:0.8 imageType:@"png" progress:^(NSProgress *progress) {
        
    } success:^(id responseObject) {
        KLog(@"上传成功===%@", responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            if (arr.count > 0) {
                
                V2TIMGroupInfo *info = [[V2TIMGroupInfo alloc] init];
                info.groupID = KStringIsNull(self.groupid);
                info.faceURL = arr[0];
                [[V2TIMManager sharedInstance] setGroupInfo:info succ:^{
                } fail:^(int code, NSString *desc) {
                }];
                [self.rowGroupLogo sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(arr[0])] placeholderImage:kAvatarPlaceHolderImg];
                [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(arr[0])] placeholderImage:kAvatarPlaceHolderImg];
            }
            [BaseFunction showAlert:@"Upload success"];
        }
    } failure:^(NSError *error) {
        KLog(@"上传失败===%@", error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
    }];
}
@end
