//
//  BC_MyRealEstateViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_MyRealEstateViewController.h"
#import "BC_AddHouseViewController.h"
#import "BC_HouseDetailViewController.h"
#import "BC_AddMyGroupViewController.h"
#import "BC_HouseGroupChatListViewController.h"

#import "BC_MyHouseListView.h"

#import "BC_HouseInfoModel.h"

@import ImSDK_Plus;
@interface BC_MyRealEstateViewController ()<BC_MyHouseListViewDelegate>
@property (nonatomic, strong) UIButton *houseBtn;
@property (nonatomic, strong) UIButton *chatBtn;
@property (nonatomic, strong) UIImageView *houseImg;
@property (nonatomic, strong) UIImageView *chatImg;
@property (nonatomic, strong) UILabel *houseL;
@property (nonatomic, strong) UILabel *chatL;
@property (nonatomic, strong) BC_MyHouseListView *houseListView;
@property (nonatomic, strong) BC_HouseGroupChatListViewController *house;
@property (nonatomic, assign) NSInteger pageType;
@end

@implementation BC_MyRealEstateViewController
- (void)dealloc {
    [kNotificationCenter removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Real estate";
    UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithImage:[[UIImage imageNamed:@"我的房产-增加"] imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal]  style:UIBarButtonItemStylePlain target:self action:@selector(addAction)];
    self.navigationItem.rightBarButtonItem = item;
    self.pageType = 1;
    [kNotificationCenter addObserver:self selector:@selector(refreshHouseList) name:@"refreshListAfterSubmit" object:nil];
    [self setupHouseView];
    [self setupTabbar];
    [self setupListView];
}

- (void)setupTabbar {
    UIView *tabbar = [JLFactoryUtil view];
    tabbar.backgroundColor = UIColor.whiteColor;
    [self.view addSubview:tabbar];
    [tabbar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.view);
        make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT);
        make.height.mas_equalTo(49);
    }];
    [tabbar setLayerShadow:[UIColor colorWithRed:0/255.0 green:0/255.0 blue:0/255.0 alpha:0.1] offset:CGSizeMake(0, -2) radius:6];

    UIView *houseView = [JLFactoryUtil view];
    [tabbar addSubview:houseView];
    [houseView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(tabbar.mas_left).offset((kScreenWidth - 200)/3);
        make.centerY.equalTo(tabbar);
        make.size.mas_equalTo(CGSizeMake(100, 49));
    }];
    UITapGestureRecognizer *tapHouse = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(chooseHouse)];
    [houseView addGestureRecognizer:tapHouse];
    
    self.houseImg = [JLFactoryUtil imageViewWithImg:@"我的房产-房产"];
    self.houseImg.userInteractionEnabled = YES;
    [houseView addSubview:self.houseImg];
    [self.houseImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(houseView);
        make.top.equalTo(houseView.mas_top).offset(5);
        make.size.mas_equalTo(CGSizeMake(22, 22));
    }];
    
    self.houseL = [JLFactoryUtil labelWithText:@"House" font:MEDIUMFONT(11) textColor:kMainTextColor];
    self.houseL.userInteractionEnabled = YES;
    [houseView addSubview:self.houseL];
    [self.houseL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(houseView);
        make.top.equalTo(self.houseImg.mas_bottom).offset(2);
    }];
    
    UIView *chatView = [JLFactoryUtil view];
    [tabbar addSubview:chatView];
    [chatView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(houseView.mas_right).offset((kScreenWidth - 200)/3);
        make.centerY.equalTo(tabbar);
        make.size.mas_equalTo(CGSizeMake(100, 49));
    }];
    UITapGestureRecognizer *tapChat = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(chooseChat)];
    [chatView addGestureRecognizer:tapChat];
    
    self.chatImg = [JLFactoryUtil imageViewWithImg:@"我的房产-群聊"];
    self.chatImg.userInteractionEnabled = YES;
    [chatView addSubview:self.chatImg];
    [self.chatImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(chatView);
        make.top.equalTo(chatView.mas_top).offset(5);
        make.size.mas_equalTo(CGSizeMake(22, 22));
    }];
    
    self.chatL = [JLFactoryUtil labelWithText:@"Chat" font:REGULARFONT(11) textColor:kMainTextColor];
    self.chatL.userInteractionEnabled = YES;
    [chatView addSubview:self.chatL];
    [self.chatL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(chatView);
        make.top.equalTo(self.chatImg.mas_bottom).offset(2);
    }];
}

- (void)setupHouseView {
    self.houseListView = [[BC_MyHouseListView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - kTabBarHeight)];
    self.houseListView.delegate = self;
    [self.view addSubview:self.houseListView];
    self.houseListView.hidden = NO;
}

- (void)setupListView {
    self.house = [BC_HouseGroupChatListViewController new];
    [self addChildViewController:self.house];
    self.house.view.frame = CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - kTabBarHeight);
    [self.view addSubview:self.house.view];
    self.house.view.hidden = YES;
}

#pragma mark  Delegate
- (void)clickHouseItem:(BC_HouseInfoModel *)model {
    BC_HouseDetailViewController *detail = [BC_HouseDetailViewController new];
    detail.houseid = KStringIsNull(model.itemid);
    [self.navigationController pushViewController:detail animated:YES];
}

- (void)addAction {
    if (_pageType == 1) {
        [self.navigationController pushViewController:[BC_AddHouseViewController new] animated:YES];
    } else {
        [self.navigationController pushViewController:[BC_AddMyGroupViewController new] animated:YES];
    }
}

- (void)chooseHouse {
    self.pageType = 1;
    self.houseImg.image = [UIImage imageNamed:@"我的房产-房产"];
    self.chatImg.image = [UIImage imageNamed:@"我的房产-群聊"];
    self.houseListView.hidden = NO;
    self.house.view.hidden = YES;
}

- (void)chooseChat {
    self.pageType = 2;
    self.houseImg.image = [UIImage imageNamed:@"房产未选中"];
    self.chatImg.image = [UIImage imageNamed:@"群聊选中"];
    self.houseListView.hidden = YES;
    self.house.view.hidden = NO;
}

- (void)refreshHouseList {
    [self.houseListView refreshList];
}

@end
