//
//  BC_HouseDetailTeneTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_HouseDetailTeneTableViewCell.h"
#import "BC_HouseInfoModel.h"
@implementation BC_HouseDetailTeneTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        [self setupUI];
    }
    return self;
}

- (void)setupUI  {
    UIView *line = [JLFactoryUtil view];
    line.backgroundColor = kColorWithHex(@"#F5F5F5");
    [self.contentView addSubview:line];
    [line mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.bottom.equalTo(self.contentView.mas_bottom);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:20 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(20);
        make.size.mas_equalTo(CGSizeMake(40, 40));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.avatar);
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    UIImageView *phoneImg = [JLFactoryUtil imageViewWithImg:@"租户手机"];
    [self.contentView addSubview:phoneImg];
    [phoneImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.avatar.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake(20, 20));
    }];
    
    self.phoneL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.phoneL];
    [self.phoneL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(phoneImg);
        make.left.equalTo(phoneImg.mas_right).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_centerX).offset(-5);
    }];
    
    UIImageView *emailImg = [JLFactoryUtil imageViewWithImg:@"租户邮箱"];
    [self.contentView addSubview:emailImg];
    [emailImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(phoneImg.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake(20, 20));
    }];
    
    self.emailL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.emailL];
    [self.emailL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(emailImg);
        make.left.equalTo(emailImg.mas_right).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_centerX).offset(-5);
    }];
    
    UIImageView *jobImg = [JLFactoryUtil imageViewWithImg:@"租户职业"];
    [self.contentView addSubview:jobImg];
    [jobImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_centerX).offset(5);
        make.centerY.equalTo(phoneImg);
        make.size.mas_equalTo(CGSizeMake(20, 20));
    }];
    
    self.jobL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.jobL];
    [self.jobL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(jobImg);
        make.left.equalTo(jobImg.mas_right).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-5);
    }];
    
    UIImageView *creImg = [JLFactoryUtil imageViewWithImg:@"租户信誉"];
    [self.contentView addSubview:creImg];
    [creImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_centerX).offset(5);
        make.centerY.equalTo(emailImg);
        make.size.mas_equalTo(CGSizeMake(20, 20));
    }];
    
    self.creditL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.creditL];
    [self.creditL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(creImg);
        make.left.equalTo(creImg.mas_right).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-5);
    }];
}

- (void)setModel:(BC_HouseInfoModel *)model {
    _model = model;
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(model.avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(model.name);
    self.phoneL.text = KStringIsNull(model.mobile);
    self.creditL.text = KStringIsNull(model.score);
    self.emailL.text = KStringIsNull(model.email);
    self.jobL.text = KStringIsNull(model.vocation);
}

@end
