//
//  BC_MyHouseListView.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_MyHouseListView.h"
#import "BC_MyHouseListTableViewCell.h"
#import "BC_HouseInfoModel.h"

@interface BC_MyHouseListView()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, assign) NSInteger pageNum;
@property (nonatomic, strong) NSMutableArray *dataArr;
@end

@implementation BC_MyHouseListView

- (id)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.pageNum = 1;
        self.dataArr = [NSMutableArray array];
        [self setupUI];
        [self loadData];
    }
    return self;
}

- (void)setupUI {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, self.bounds.size.height)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_MyHouseListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_MyHouseListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    [cell setModel:_dataArr[indexPath.row]];
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 130;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if (self.delegate && [self.delegate respondsToSelector:@selector(clickHouseItem:)]) {
        [self.delegate clickHouseItem:self.dataArr[indexPath.row]];
    }
}

- (void)refreshList {
    [self loadData];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getListData];
}

- (void)loadMoreData {
    self.pageNum ++;
    [self getListData];
}

- (void)getListData {
    NSString *url = [NSString stringWithFormat:@"%@/api/app/gethouselist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_HouseInfoModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_HouseInfoModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}

@end
