//
//  BC_PersonInfoViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/20.
//

#import "BC_PersonInfoViewController.h"
#import "BC_EditNicknameViewController.h"
#import "BC_EditPhoneViewController.h"
#import "BC_ChangePWViewController.h"

#import <TZImagePickerController.h>
@import ImSDK_Plus;

@interface BC_PersonInfoViewController ()<TZImagePickerControllerDelegate>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;

@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) UILabel *phoneL;

@end

@implementation BC_PersonInfoViewController

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(umodel.member_avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(umodel.member_nickname);
    NSString *phone = KStringIsNull(umodel.member_phone);
    if (phone.length >= 5) {
        self.phoneL.text = kStringFormat(@"%@***%@", [phone substringToIndex:2], [phone substringFromIndex:phone.length - 2]);
    } else {
        self.phoneL.text = phone;
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navigationItem.title = @"Information";
    [self setupUI];
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    UILabel *label1 = [JLFactoryUtil labelWithText:@"Avatar" font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:label1];
    [label1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(30);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:24 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label1);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    UIView *line1 = [JLFactoryUtil view];
    line1.backgroundColor = kColorWithHex(@"#F2F2F2");
    [self.contentView addSubview:line1];
    [line1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(label1.mas_bottom).offset(30);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
    UILabel *label2 = [JLFactoryUtil labelWithText:@"Name" font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:label2];
    [label2 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(line1.mas_bottom).offset(20);
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label2);
        make.right.equalTo(self.contentView.mas_right).offset(-40);
        make.left.greaterThanOrEqualTo(self.contentView.mas_centerX).offset(5);
    }];
    
    UIImageView *arrow1 = [JLFactoryUtil imageViewWithImg:@"更多"];
    [self.contentView addSubview:arrow1];
    [arrow1 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label2);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(15, 15));
    }];
    
    UIView *line2 = [JLFactoryUtil view];
    line2.backgroundColor = kColorWithHex(@"#F2F2F2");
    [self.contentView addSubview:line2];
    [line2 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(label2.mas_bottom).offset(20);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
    UILabel *label3 = [JLFactoryUtil labelWithText:@"Phone" font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:label3];
    [label3 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(line2.mas_bottom).offset(20);
    }];
    
    self.phoneL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.phoneL];
    [self.phoneL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label3);
        make.right.equalTo(self.contentView.mas_right).offset(-40);
        make.left.greaterThanOrEqualTo(self.contentView.mas_centerX).offset(5);
    }];
    
    UIImageView *arrow2 = [JLFactoryUtil imageViewWithImg:@"更多"];
    [self.contentView addSubview:arrow2];
    [arrow2 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label3);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(15, 15));
    }];
    
    UIView *line3 = [JLFactoryUtil view];
    line3.backgroundColor = kColorWithHex(@"#F2F2F2");
    [self.contentView addSubview:line3];
    [line3 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(label3.mas_bottom).offset(20);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, 1));
    }];
    
    UILabel *label4 = [JLFactoryUtil labelWithText:@"Change Password" font:REGULARFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:label4];
    [label4 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(line3.mas_bottom).offset(20);
    }];
    
    UIImageView *arrow3 = [JLFactoryUtil imageViewWithImg:@"更多"];
    [self.contentView addSubview:arrow3];
    [arrow3 mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(label4);
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.size.mas_equalTo(CGSizeMake(15, 15));
    }];
    
    UIView *avatarClear = [JLFactoryUtil view];
    avatarClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:avatarClear];
    [avatarClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(label1);
        make.height.mas_equalTo(50);
    }];
    UITapGestureRecognizer *tapAvatar = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(chooseAvatar)];
    [avatarClear addGestureRecognizer:tapAvatar];
    
    UIView *nicknameClear = [JLFactoryUtil view];
    nicknameClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:nicknameClear];
    [nicknameClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(label2);
        make.height.mas_equalTo(50);
    }];
    UITapGestureRecognizer *tapNickname = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(editNickname)];
    [nicknameClear addGestureRecognizer:tapNickname];
    
    UIView *phoneClear = [JLFactoryUtil view];
    phoneClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:phoneClear];
    [phoneClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(label3);
        make.height.mas_equalTo(50);
    }];
    UITapGestureRecognizer *tapPhone = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(editPhonenum)];
    [phoneClear addGestureRecognizer:tapPhone];
    
    UIView *pwClear = [JLFactoryUtil view];
    pwClear.backgroundColor = UIColor.clearColor;
    [self.contentView addSubview:pwClear];
    [pwClear mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.contentView);
        make.centerY.equalTo(label4);
        make.height.mas_equalTo(50);
    }];
    UITapGestureRecognizer *tapPw = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(editPW)];
    [pwClear addGestureRecognizer:tapPw];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(label4.mas_bottom).offset(30);
    }];
    
}

#pragma mark  选择头像
- (void)chooseAvatar {
    TZImagePickerController *imagePickerVc = [[TZImagePickerController alloc] initWithMaxImagesCount:1 delegate:self];
    imagePickerVc.barItemTextColor = [UIColor whiteColor];
    [imagePickerVc.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName : [UIColor whiteColor]}];
    imagePickerVc.navigationBar.tintColor = [UIColor whiteColor];
    if (@available(iOS 15.0, *)) {
        UINavigationBarAppearance *barApp = [UINavigationBarAppearance new];
        barApp.backgroundColor = [UIColor whiteColor];
        barApp.shadowColor = [UIColor clearColor];
        imagePickerVc.navigationController.navigationBar.scrollEdgeAppearance = barApp;
        imagePickerVc.navigationController.navigationBar.standardAppearance = barApp;
    } else {
        imagePickerVc.navigationBar.barTintColor = [UIColor whiteColor];
    }
    imagePickerVc.oKButtonTitleColorDisabled = [UIColor lightGrayColor];
    imagePickerVc.navigationBar.translucent = NO;
    imagePickerVc.iconThemeColor = kMainThemeColor;
    imagePickerVc.showSelectedIndex = YES;
    imagePickerVc.allowTakePicture = YES;
    imagePickerVc.allowTakeVideo = NO;   // 在内部显示拍视频按
    imagePickerVc.showPhotoCannotSelectLayer = YES;
    imagePickerVc.cannotSelectLayerColor = [[UIColor whiteColor] colorWithAlphaComponent:0.8];
    imagePickerVc.allowPickingVideo = NO;
    imagePickerVc.allowPickingOriginalPhoto = YES;
    imagePickerVc.allowPickingImage = YES;
    imagePickerVc.allowPickingGif = NO;
    imagePickerVc.allowCrop = YES;
    imagePickerVc.cropRect = CGRectMake(0, (kScreenHeight - kScreenWidth)/2, kScreenWidth, kScreenWidth);
    imagePickerVc.statusBarStyle = UIStatusBarStyleDefault;
    imagePickerVc.preferredLanguage = @"en";
    imagePickerVc.modalPresentationStyle = UIModalPresentationFullScreen;
    [self presentViewController:imagePickerVc animated:YES completion:nil];
}

- (void)tz_imagePickerControllerDidCancel:(TZImagePickerController *)picker {
}

- (void)imagePickerController:(TZImagePickerController *)picker didFinishPickingPhotos:(NSArray<UIImage *> *)photos sourceAssets:(NSArray *)assets isSelectOriginalPhoto:(BOOL)isSelectOriginalPhoto {
    if (photos.count > 0) {
        [self uploadImg:photos[0]];
    }
}

- (void)editNickname {
    BC_EditNicknameViewController *editVC = [BC_EditNicknameViewController new];
    editVC.nickname = KStringIsNull(self.nicknameL.text);
    [self.navigationController pushViewController:editVC animated:YES];
}

- (void)editPhonenum {
    [self.navigationController pushViewController:[BC_EditPhoneViewController new] animated:YES];
}

- (void)editPW {
    [self.navigationController pushViewController:[BC_ChangePWViewController new] animated:YES];
}

- (void)uploadImg:(UIImage *)avatar {
    [self.view endEditing:YES];
    
    [MBProgressHUD showHUDAddTo:kAppWindow alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/changeavatar/",KMainUrl];
    [PPNetworkHelper uploadImagesWithURL:url parameters:@{} name:@"file" images:@[avatar] fileNames:nil imageScale:0.8 imageType:@"png" progress:^(NSProgress *progress) {
        
    } success:^(id responseObject) {
        KLog(@"上传成功===%@", responseObject);
        [MBProgressHUD hideHUDForView:kAppWindow animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            NSArray *arr = (NSArray *)dataDic[@"data"];
            if (arr.count > 0) {
                
                V2TIMUserFullInfo *info = [[V2TIMUserFullInfo alloc] init];
                info.faceURL = KStringIsNull(arr[0]);
                [[V2TIMManager sharedInstance] setSelfInfo:info succ:^{
                    KLog(@"设置im头像成功");
                } fail:^(int code, NSString *desc) {
                }];
                
                [self.avatar sd_setImageWithURL:[NSURL URLWithString:arr[0]] placeholderImage:kAvatarPlaceHolderImg];
                BaseUserModel *umodel = [UserManager shareManager].getUserInfo;
                umodel.member_avatar = KStringIsNull(arr[0]);
                [[UserManager shareManager] saveUserInfoWithModel:umodel];
            }
            [BaseFunction showAlert:@"Upload success"];
        }
    } failure:^(NSError *error) {
        KLog(@"上传失败===%@", error);
        [MBProgressHUD hideHUDForView:kAppWindow animated:YES];
    }];
}



@end
