//
//  BC_SupervisorExecuteViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/27.
//

#import "BC_SupervisorExecuteViewController.h"
#import "BC_SupervisorExecuteSubListViewController.h"
#import "BC_SupervisorSearchExecuteViewController.h"

#import <JXCategoryView.h>
#import "TUILogin.h"

@interface BC_SupervisorExecuteViewController ()<JXCategoryViewDelegate, JXCategoryListContainerViewDelegate>

@property (nonatomic, strong) JXCategoryTitleView *categoryView;
@property (nonatomic, strong) JXCategoryListContainerView *listContainerView;

@end

@implementation BC_SupervisorExecuteViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.navigationItem.title = @"Project record";
    
    UIBarButtonItem *item = [[UIBarButtonItem alloc] initWithTitle:@"Sign out" style:UIBarButtonItemStylePlain target:self action:@selector(signoutBtnClick)];
    self.navigationItem.rightBarButtonItem = item;
    
    [self setupUI];
        
}

#pragma mark  退出
- (void)signoutBtnClick {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure to log out?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            
    }]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [TUILogin logout:^{
            dispatch_async(dispatch_get_main_queue(), ^{
                [[UserManager shareManager] cleanUserInfo];
                [PPNetworkHelper setValue:@"" forHTTPHeaderField:@"Authorization"];
                [UserManager shareManager].isLogin = NO;
                [BaseFunction showAlert:@"Success"];
                [kAppDelegate setLoginRoot];
            });
        } fail:^(int code, NSString *msg) {
            
        }];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)setupUI {
    UIView *topView = [JLFactoryUtil view];
    topView.backgroundColor = UIColor.whiteColor;
    topView.frame = CGRectMake(0, 0, kScreenWidth, 50);
    [self.view addSubview:topView];
    
    UIView *searchView = [JLFactoryUtil view];
    searchView.backgroundColor = kColorWithHex(@"#F7F7F7");
    [searchView setCorner:12 withType:JLCornerTypeAllCorner];
    [topView addSubview:searchView];
    [searchView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(topView.mas_left).offset(16);
        make.centerY.equalTo(topView);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 44));
    }];
    UITapGestureRecognizer *tapSearch = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushSearchVC)];
    [searchView addGestureRecognizer:tapSearch];
    
    UILabel *sLabel = [JLFactoryUtil labelWithText:@"Search" font:REGULARFONT(14) textColor:kColorWithHex(@"#AAAAAA")];
    sLabel.userInteractionEnabled = YES;
    [searchView addSubview:sLabel];
    [sLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.left.equalTo(searchView.mas_left).offset(20);
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [searchView addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.right.equalTo(searchView.mas_right).offset(-20);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.categoryView = [[JXCategoryTitleView alloc] initWithFrame:CGRectMake(15, 55, kScreenWidth - 30, 40)];
    self.categoryView.delegate = self;
    self.categoryView.backgroundColor = kColorWithHex(@"#F5F5F5");
    self.categoryView.cellSpacing = 0;
    self.categoryView.titleColor = kMainTextColor;
    self.categoryView.titleSelectedColor = UIColor.whiteColor;
    self.categoryView.titleFont = REGULARFONT(15);
    self.categoryView.titleSelectedFont = REGULARFONT(15);
    self.categoryView.defaultSelectedIndex = 0;
    self.categoryView.titleColorGradientEnabled = YES;
    self.categoryView.averageCellSpacingEnabled = YES;
    self.categoryView.titles = @[@"Process", @"Completed"];
    self.categoryView.cellWidth = (kScreenWidth - 30)/2;
    self.categoryView.layer.masksToBounds = YES;
    self.categoryView.layer.cornerRadius = 20;
    
    JXCategoryIndicatorBackgroundView *backgroundView = [[JXCategoryIndicatorBackgroundView alloc] init];
    backgroundView.indicatorWidth = (kScreenWidth - 30)/2-10;
    backgroundView.indicatorHeight = 40;
    backgroundView.indicatorCornerRadius = 20;
    backgroundView.indicatorColor = kMainThemeColor;
    [backgroundView setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 0) radius:8];
    self.categoryView.indicators = @[backgroundView];
    [self.view addSubview:self.categoryView];
    
    self.listContainerView = [[JXCategoryListContainerView alloc] initWithType:JXCategoryListContainerType_ScrollView delegate:self];
    self.listContainerView.frame = CGRectMake(0, 100, kScreenWidth, kScreenHeight - kTopHeight - 100 - kTabBarHeight);
    [self.view addSubview:self.listContainerView];
    self.categoryView.listContainer = self.listContainerView;
    
}

#pragma mark - JXCategoryViewDelegate
- (id<JXCategoryListContentViewDelegate>)listContainerView:(JXCategoryListContainerView *)listContainerView initListForIndex:(NSInteger)index {
    BC_SupervisorExecuteSubListViewController *list = [BC_SupervisorExecuteSubListViewController new];
    list.index = index;
    return list;
}

- (NSInteger)numberOfListsInlistContainerView:(JXCategoryListContainerView *)listContainerView {
    return 2;
}

- (void)pushSearchVC {
    [self.navigationController pushViewController:[BC_SupervisorSearchExecuteViewController new] animated:NO];
}

@end
