//
//  BC_SupervisorSearchExecuteViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/27.
//

#import "BC_SupervisorSearchExecuteViewController.h"
#import "BC_ProjectDetailViewController.h"

#import "BC_SearchProjectListTableViewCell.h"
#import "BC_ExecuteModuleModel.h"
@interface BC_SupervisorSearchExecuteViewController ()<UITextFieldDelegate, UITableViewDelegate, UITableViewDataSource, DZNEmptyDataSetSource, DZNEmptyDataSetDelegate, BC_SearchProjectListTableViewCellDelegate>
@property (nonatomic, strong) UITextField *searchTF;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@end

@implementation BC_SupervisorSearchExecuteViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.isHidenNaviBar = YES;
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupUI];
}

- (void)setupUI {
    UIView *sview = [JLFactoryUtil view];
    sview.backgroundColor = kColorWithHex(@"#F7F7F7");
    [sview setCorner:12 withType:JLCornerTypeAllCorner];
    [self.view addSubview:sview];
    [sview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(15);
        make.top.equalTo(self.view.mas_top).offset(kStatusBarHeight);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 95, 44));
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [sview addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_left).offset(15);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.searchTF = [JLFactoryUtil textFieldWithFont:REGULARFONT(14) place:@"Search"];
    self.searchTF.textColor = kMainTextColor;
    self.searchTF.delegate = self;
    self.searchTF.returnKeyType = UIReturnKeySearch;
    [sview addSubview:self.searchTF];
    [self.searchTF mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sImg.mas_right).offset(10);
        make.right.equalTo(sview.mas_right).offset(-10);
        make.height.mas_equalTo(30);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainTextColor];
    [cancelBtn addTarget:self action:@selector(cancelBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(sview);
        make.left.equalTo(sview.mas_right);
        make.right.equalTo(self.view.mas_right);
        make.height.mas_equalTo(30);
    }];
    
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, kTopHeight, kScreenWidth, kScreenHeight - kTopHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.backgroundColor = UIColor.whiteColor;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_SearchProjectListTableViewCell class] forCellReuseIdentifier:@"cellid"];
    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if ([KStringIsNull(textField.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Please enter content"];
        return NO;
    }
    [self loadData];
    return YES;
}

- (void)cancelBtnClick {
    [self.navigationController popViewControllerAnimated:NO];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellid = @"cellid";
    BC_SearchProjectListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
    if (!cell) {
        cell = [[BC_SearchProjectListTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
    }
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    [self configCell:cell andIndexP:indexPath];
    return cell;
}

- (void)configCell:(BC_SearchProjectListTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return [tableView fd_heightForCellWithIdentifier:@"cellid" cacheByIndexPath:indexPath configuration:^(id cell) {
        [self configCell:cell andIndexP:indexPath];
    }];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    BC_ExecuteModuleModel *model = self.dataArr[indexPath.row];
    BC_ProjectDetailViewController *detail = [BC_ProjectDetailViewController new];
    detail.pid = KStringIsNull(model.itemid);
    [self.navigationController pushViewController:detail animated:YES];
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_ExecuteModuleModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_ExecuteModuleModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_ExecuteModuleModel *)model {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"2"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.dataArr removeObject:model];
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}


- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    return [UIImage imageNamed:@"空数据"];
}

- (BOOL)emptyDataSetShouldAllowScroll:(UIScrollView *)scrollView {
    return YES;
}

- (void)getPageData {
    if ([KStringIsNull(self.searchTF.text) isEqualToString:@""]) return;
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/myprogramlist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10", @"status":@"", @"search":KStringIsNull(self.searchTF.text)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_ExecuteModuleModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
        [BaseFunction showAlert:@"Error"];
    }];
}

@end
