//
//  BC_SupervisorHomePageViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/25.
//

#import "BC_SupervisorHomePageViewController.h"
#import "BC_SupervisorHomeSubListViewController.h"
#import "BC_SupervisorSearchViewController.h"

#import <JXCategoryView.h>
@import ImSDK_Plus;

@interface BC_SupervisorHomePageViewController ()<JXCategoryViewDelegate, JXCategoryListContainerViewDelegate>

@property (nonatomic, strong) JXCategoryTitleView *categoryView;
@property (nonatomic, strong) JXCategoryListContainerView *listContainerView;

@end

@implementation BC_SupervisorHomePageViewController
- (void)dealloc {
    [kNotificationCenter removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.isHidenNaviBar = YES;
    [self setupUI];
    [kNotificationCenter addObserver:self selector:@selector(refreshPage) name:@"refreshPageAfterAudit" object:nil];
    
    V2TIMUserFullInfo *info = [[V2TIMUserFullInfo alloc] init];
    info.nickName = @"Supervisor";
    [[V2TIMManager sharedInstance] setSelfInfo:info succ:^{
        KLog(@"设置督导成功");
    } fail:^(int code, NSString *desc) {
    }];
    
}

- (void)refreshPage {
    [self.categoryView reloadData];
}

- (void)setupUI {
    UIView *topView = [JLFactoryUtil view];
    topView.backgroundColor = UIColor.whiteColor;
    topView.frame = CGRectMake(0, 0, kScreenWidth, kTopHeight);
    [self.view addSubview:topView];
    
    UIView *searchView = [JLFactoryUtil view];
    searchView.backgroundColor = kColorWithHex(@"#F7F7F7");
    [searchView setCorner:12 withType:JLCornerTypeAllCorner];
    [topView addSubview:searchView];
    [searchView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(topView.mas_left).offset(16);
        make.bottom.equalTo(topView.mas_bottom).offset(-2);
        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 32, 44));
    }];
    UITapGestureRecognizer *tapSearch = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(pushSearchVC)];
    [searchView addGestureRecognizer:tapSearch];
    
    UILabel *sLabel = [JLFactoryUtil labelWithText:@"Search" font:REGULARFONT(14) textColor:kColorWithHex(@"#AAAAAA")];
    sLabel.userInteractionEnabled = YES;
    [searchView addSubview:sLabel];
    [sLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.left.equalTo(searchView.mas_left).offset(20);
    }];
    
    UIImageView *sImg = [JLFactoryUtil imageViewWithImg:@"搜索"];
    sImg.userInteractionEnabled = YES;
    [searchView addSubview:sImg];
    [sImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(searchView);
        make.right.equalTo(searchView.mas_right).offset(-20);
        make.size.mas_equalTo(CGSizeMake(18, 18));
    }];
    
    self.categoryView = [[JXCategoryTitleView alloc] initWithFrame:CGRectMake(15, kTopHeight+5, kScreenWidth - 30, 40)];
    self.categoryView.delegate = self;
    self.categoryView.backgroundColor = kColorWithHex(@"#F5F5F5");
    self.categoryView.cellSpacing = 0;
    self.categoryView.titleColor = kMainTextColor;
    self.categoryView.titleSelectedColor = UIColor.whiteColor;
    self.categoryView.titleFont = REGULARFONT(15);
    self.categoryView.titleSelectedFont = REGULARFONT(15);
    self.categoryView.defaultSelectedIndex = 0;
    self.categoryView.titleColorGradientEnabled = YES;
    self.categoryView.averageCellSpacingEnabled = YES;
    self.categoryView.titles = @[@"Audit", @"Passed"];
    self.categoryView.cellWidth = (kScreenWidth - 30)/2;
    self.categoryView.layer.masksToBounds = YES;
    self.categoryView.layer.cornerRadius = 20;
    
    JXCategoryIndicatorBackgroundView *backgroundView = [[JXCategoryIndicatorBackgroundView alloc] init];
    backgroundView.indicatorWidth = (kScreenWidth - 30)/2-10;
    backgroundView.indicatorHeight = 40;
    backgroundView.indicatorCornerRadius = 20;
    backgroundView.indicatorColor = kMainThemeColor;
    [backgroundView setLayerShadow:[UIColor colorWithRed:232/255.0 green:37/255.0 blue:37/255.0 alpha:0.36] offset:CGSizeMake(0, 0) radius:8];
//    backgroundView.indicatorWidthIncrement = 60;
    self.categoryView.indicators = @[backgroundView];
    [self.view addSubview:self.categoryView];
    
    self.listContainerView = [[JXCategoryListContainerView alloc] initWithType:JXCategoryListContainerType_ScrollView delegate:self];
    self.listContainerView.frame = CGRectMake(0, kTopHeight + 50, kScreenWidth, kScreenHeight - kTopHeight - 50 - kTabBarHeight);
    [self.view addSubview:self.listContainerView];
    self.categoryView.listContainer = self.listContainerView;
}

#pragma mark - JXCategoryViewDelegate
- (id<JXCategoryListContentViewDelegate>)listContainerView:(JXCategoryListContainerView *)listContainerView initListForIndex:(NSInteger)index {
    BC_SupervisorHomeSubListViewController *list = [BC_SupervisorHomeSubListViewController new];
    list.index = index;
    return list;
}

- (NSInteger)numberOfListsInlistContainerView:(JXCategoryListContainerView *)listContainerView {
    return 2;
}

- (void)categoryView:(JXCategoryBaseView *)categoryView didSelectedItemAtIndex:(NSInteger)index {
}

- (void)pushSearchVC {
    BC_SupervisorSearchViewController *search = [BC_SupervisorSearchViewController new];
    [self.navigationController pushViewController:search animated:NO];
}
@end
