//
//  BC_SupervisorHomeSubListViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/25.
//

#import "BC_SupervisorHomeSubListViewController.h"
#import "BC_SupervisorPicDetailViewController.h"
#import "BC_SupervisorVideoDetailViewController.h"

#import "BC_SupervisorHomeHasBtnTableViewCell.h"
#import "BC_SupervisorHomeHasBtnPicTableViewCell.h"
#import "BC_SupervisorHomeHasBtnVideoTableViewCell.h"

#import "BC_PostListModel.h"
@interface BC_SupervisorHomeSubListViewController ()<UITableViewDelegate, UITableViewDataSource, DZNEmptyDataSetSource, DZNEmptyDataSetDelegate, BC_SupervisorHomeHasBtnTableViewCellDelegate, BC_SupervisorHomeHasBtnPicTableViewCellDelegate, BC_SupervisorHomeHasBtnVideoTableViewCellDelegate>
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) NSMutableArray *dataArr;
@property (nonatomic, assign) NSInteger pageNum;
@end

@implementation BC_SupervisorHomeSubListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.dataArr = [NSMutableArray array];
    self.pageNum = 1;
    [self setupListView];
    [self loadData];
}

- (void)setupListView {
    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - kTopHeight - 50 - kTabBarHeight)];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.tableFooterView = [UIView new];
    self.tableView.showsVerticalScrollIndicator = NO;
    self.tableView.emptyDataSetSource = self;
    self.tableView.emptyDataSetDelegate = self;
    self.tableView.backgroundColor = UIColor.whiteColor;
    [self.view addSubview:self.tableView];
    self.tableView.contentInsetAdjustmentBehavior = UIScrollViewContentInsetAdjustmentNever;
    if (@available(iOS 15.0, *)) {
        self.tableView.sectionHeaderTopPadding = 0;
    }
    [self.tableView registerClass:[BC_SupervisorHomeHasBtnTableViewCell class] forCellReuseIdentifier:@"btncellid"];
    [self.tableView registerClass:[BC_SupervisorHomeHasBtnPicTableViewCell class] forCellReuseIdentifier:@"piccellid"];
    [self.tableView registerClass:[BC_SupervisorHomeHasBtnVideoTableViewCell class] forCellReuseIdentifier:@"videocellid"];

    self.tableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingTarget:self refreshingAction:@selector(loadData)];
    self.tableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingTarget:self refreshingAction:@selector(loadMoreData)];
}

- (void)loadData {
    self.pageNum = 1;
    [self.tableView.mj_footer resetNoMoreData];
    [self getPageData];
}

- (void)loadMoreData {
    self.pageNum++;
    [self getPageData];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    BC_PostListModel *model = self.dataArr[indexPath.row];
    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        //视频
        static NSString *cellid = @"videocellid";
        BC_SupervisorHomeHasBtnVideoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
        if (!cell) {
            cell = [[BC_SupervisorHomeHasBtnVideoTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
        }
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.delegate = self;
        [self configVideoCell:cell andIndexP:indexPath];
        return cell;
    } else {
        if (model.files.count == 0) {
            //无图
            static NSString *cellid = @"btncellid";
            BC_SupervisorHomeHasBtnTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
            if (!cell) {
                cell = [[BC_SupervisorHomeHasBtnTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
            }
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            cell.delegate = self;
            [self configBtnCell:cell andIndexP:indexPath];
            return cell;
        } else {
            static NSString *cellid = @"piccellid";
            BC_SupervisorHomeHasBtnPicTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellid];
            if (!cell) {
                cell = [[BC_SupervisorHomeHasBtnPicTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellid];
            }
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            cell.delegate = self;
            [self configPicCell:cell andIndexP:indexPath];
            return cell;
        }
    }
}

- (void)configBtnCell:(BC_SupervisorHomeHasBtnTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    cell.pageIndex = self.index;
    [cell setModel:_dataArr[indexp.row]];
}

- (void)configPicCell:(BC_SupervisorHomeHasBtnPicTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    cell.pageIndex = self.index;
    [cell setModel:_dataArr[indexp.row]];
}

- (void)configVideoCell:(BC_SupervisorHomeHasBtnVideoTableViewCell *)cell andIndexP:(NSIndexPath *)indexp {
    cell.pageIndex = self.index;
    [cell setModel:_dataArr[indexp.row]];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    BC_PostListModel *model = self.dataArr[indexPath.row];
    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        //视频
        return [tableView fd_heightForCellWithIdentifier:@"videocellid" cacheByIndexPath:indexPath configuration:^(id cell) {
            [self configVideoCell:cell andIndexP:indexPath];
        }];
    } else {
        if (model.files.count == 0) {
            //无图
            return [tableView fd_heightForCellWithIdentifier:@"btncellid" cacheByIndexPath:indexPath configuration:^(id cell) {
                [self configBtnCell:cell andIndexP:indexPath];
            }];
        } else {
            return [tableView fd_heightForCellWithIdentifier:@"piccellid" cacheByIndexPath:indexPath configuration:^(id cell) {
                [self configPicCell:cell andIndexP:indexPath];
            }];
        }
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArr.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    BC_PostListModel *model = self.dataArr[indexPath.row];

    if ([KStringIsNull(model.file_type) isEqualToString:@"1"]) {
        BC_SupervisorVideoDetailViewController *video = [BC_SupervisorVideoDetailViewController new];
        video.infoModel = model;
        if (_index == 0) {
            video.ifNeedAudit = YES;
        }
        [self.navigationController pushViewController:video animated:YES];
    } else {
        BC_SupervisorPicDetailViewController *pic = [BC_SupervisorPicDetailViewController new];
        pic.infoModel = model;
        if (_index == 0) {
            pic.ifNeedAudit = YES;
        }
        [self.navigationController pushViewController:pic animated:YES];
    }
    
}

#pragma mark  屏蔽相关
- (void)shieldItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)shieldPicItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)shieldVideoItem:(BC_PostListModel *)model {
    [self showShieldAlert:model];
}

- (void)showShieldAlert:(BC_PostListModel *)model {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:@"Are you sure you want to block?" preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Confirm" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self shieldFromService:model];
    }]];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)shieldFromService:(BC_PostListModel *)model {
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/shield/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(model.itemid), @"type":@"1"} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [BaseFunction showAlert:@"Success"];
            [self.dataArr removeObject:model];
            [self.tableView reloadData];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (UIImage *)imageForEmptyDataSet:(UIScrollView *)scrollView {
    return [UIImage imageNamed:@"空数据"];
}

- (BOOL)emptyDataSetShouldAllowScroll:(UIScrollView *)scrollView {
    return YES;
}

#pragma mark - JXCategoryListContentViewDelegate
- (UIView *)listView {
    return self.view;
}

- (void)getPageData {
    NSString *status = @"";
    if (self.index == 0) {
        status = @"10";
    } else {
        status = @"20";
    }
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/poststatuslist/", KMainUrl];
    [PPNetworkHelper GET:url parameters:@{@"page":@(self.pageNum), @"limit":@"10", @"status":status} success:^(id responseObject) {
        KLog(@"帖子成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
            if (self.pageNum == 1) {
                self.dataArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                [self.tableView.mj_header endRefreshing];
            } else {
                NSMutableArray *tempArr = [BC_PostListModel mj_objectArrayWithKeyValuesArray:dataDic[@"data"]];
                if (tempArr.count > 0) {
                    [self.dataArr addObjectsFromArray:tempArr];
                    [self.tableView.mj_footer endRefreshing];
                } else {
                    [self.tableView.mj_footer endRefreshingWithNoMoreData];
                }
            }
        } else {
            [self.tableView.mj_header endRefreshing];
            [self.tableView.mj_footer endRefreshing];
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
        [self.tableView reloadData];
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
        [self.tableView.mj_header endRefreshing];
        [self.tableView.mj_footer endRefreshing];
    }];
}

@end
