//
//  BC_SupervisorPicDetailViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/25.
//

#import "BC_SupervisorPicDetailViewController.h"
#import "ZlinkPhotoBrowserView.h"

#import "BC_PostListModel.h"
@interface BC_SupervisorPicDetailViewController ()
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UILabel *titleL;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) UILabel *timeL;
@property (nonatomic, strong) UILabel *contentL;
@property (nonatomic, strong) UIView *areaView;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) YYTextView *reasonTV;

@end

@implementation BC_SupervisorPicDetailViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [self setupUI];
}

- (void)setupUI {
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, kScreenWidth, kScreenHeight - HOME_INDICATOR_HEIGHT - 60 - kTopHeight)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    self.titleL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(18) textColor:kMainTextColor];
    self.titleL.numberOfLines = 0;
    [self.contentView addSubview:self.titleL];
    [self.titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(20);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:18 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.titleL.mas_bottom).offset(20);
        make.size.mas_equalTo(CGSizeMake(36, 36));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-1);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(11) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(self.avatar.mas_centerY).offset(1);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    self.contentL.numberOfLines = 0;
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.avatar.mas_bottom).offset(15);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    CGFloat imgTotalH = 0;
    if (self.infoModel.files.count > 0) {
        NSArray *imgarr = self.infoModel.files;
        CGFloat imgH = (kScreenWidth - 30)*240/345;
        imgTotalH = (5 + imgH) * imgarr.count;
        for (int i = 0; i < imgarr.count; i++) {
            UIImageView *img = [JLFactoryUtil imageView];
            img.tag = 200 + i;
            [img setCorner:4 withType:JLCornerTypeAllCorner];
            [self.contentView addSubview:img];
            [img mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.contentView.mas_left).offset(15);
                make.top.equalTo(self.contentL.mas_bottom).offset(5 + i*(imgH + 5));
                make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, imgH));
            }];
            [img sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(imgarr[i])] placeholderImage:[UIImage imageWithColor:kCOLOR(130, 151, 218, 1)]];
            img.userInteractionEnabled = YES;
            UITapGestureRecognizer *tapImg = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showBigImg:)];
            [img addGestureRecognizer:tapImg];
        }
    } else {
        imgTotalH = 5;
    }
    
    self.areaView = [JLFactoryUtil view];
    self.areaView.backgroundColor = kColorWithHex(@"#ECF8EF");
    [self.areaView setCorner:12 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.areaView];
    [self.areaView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentL.mas_bottom).offset(15 + imgTotalH);
        make.size.mas_equalTo(CGSizeMake(30, 24));
    }];

    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.areaView addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(self.areaView.mas_left).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];

    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.areaView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(areaImg.mas_right).offset(4);
        make.width.mas_lessThanOrEqualTo(kScreenWidth - 70);
    }];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.areaView.mas_bottom).offset(20);
    }];
    
    if (_ifNeedAudit) {
        UIButton *passBtn = [JLFactoryUtil buttonWithTitle:@"Pass" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
        [passBtn addTarget:self action:@selector(passbtnClick) forControlEvents:UIControlEventTouchUpInside];
        passBtn.backgroundColor = kMainThemeColor;
        [passBtn setCorner:22 withType:JLCornerTypeAllCorner];
        [self.view addSubview:passBtn];
        [passBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT - 8);
            make.right.equalTo(self.view.mas_right).offset(-15);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 70)/2, 44));
        }];
        
        UIButton *refuseBtn = [JLFactoryUtil buttonWithTitle:@"Refuse" font:MEDIUMFONT(16) textColor:kMainThemeColor];
        [refuseBtn addTarget:self action:@selector(refuseBtnClick) forControlEvents:UIControlEventTouchUpInside];
        refuseBtn.backgroundColor = UIColor.whiteColor;
        refuseBtn.layer.borderWidth = 1;
        refuseBtn.layer.borderColor = kMainThemeColor.CGColor;
        [refuseBtn setCorner:22 withType:JLCornerTypeAllCorner];
        [self.view addSubview:refuseBtn];
        [refuseBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT - 8);
            make.right.equalTo(passBtn.mas_left).offset(-40);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 70)/2, 44));
        }];
        
    }
    
    self.areaL.text = KStringIsNull(self.infoModel.city);
    CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
    if (size.width >= kScreenWidth - 70) {
        size.width = kScreenWidth - 70;
    }
    [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.contentL.mas_bottom).offset(15 + imgTotalH);
        make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
    }];
    
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(self.infoModel.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(self.infoModel.user_name).length == 0 ? @"User":KStringIsNull(self.infoModel.user_name);
    self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
    self.titleL.text = KStringIsNull(self.infoModel.name);
    self.contentL.text = KStringIsNull(self.infoModel.desc);
    
}

- (void)passbtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"The post will be published successfully after passing" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmPassAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)confirmPassAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postsaudit/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(self.infoModel.itemid), @"action":@"pass", @"reason":@""} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"refreshPageAfterAudit" object:nil];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popViewControllerAnimated:YES];

        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)cancelAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)refuseBtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 200)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Reason" font:MEDIUMFONT(15) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(15);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *dview = [JLFactoryUtil view];
    [dview setCorner:4 withType:JLCornerTypeAllCorner];
    dview.backgroundColor = kColorWithHex(@"#F5F5F5");
    [contentView addSubview:dview];
    [dview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.top.equalTo(titleL.mas_bottom).offset(10);
        make.size.mas_equalTo(CGSizeMake(260, 100));
    }];
    
    self.reasonTV = [YYTextView new];
    self.reasonTV.placeholderFont = REGULARFONT(14);
    self.reasonTV.placeholderText = @"Reason for rejection";
    self.reasonTV.placeholderTextColor = kLightGrayTextColor;
    self.reasonTV.textColor = kMainTextColor;
    self.reasonTV.font = REGULARFONT(14);
    [dview addSubview:self.reasonTV];
    [self.reasonTV mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(dview).insets(UIEdgeInsetsMake(10, 10, 10, 10));
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmRefuseAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    popup.shouldHandleKeyboard = YES;
    [popup showWithLayout:layout];
}

- (void)confirmRefuseAction:(UIButton *)sender {
    if ([KStringIsNull(self.reasonTV.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Please input reason"];
        return;
    }
    
    [sender dismissPresentingPopup];
        
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postsaudit/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(self.infoModel.itemid), @"action":@"deny", @"reason":KStringIsNull(self.reasonTV.text)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"refreshPageAfterAudit" object:nil];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popViewControllerAnimated:YES];
        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)showBigImg:(UIGestureRecognizer *)ges {
    NSInteger tag = [ges view].tag - 200;
    NSArray *imgarr = self.infoModel.files;
    NSMutableArray *itemarr = [NSMutableArray array];
    for (NSString *url in imgarr) {
        ZlinkPhotoGroupItem *item = [ZlinkPhotoGroupItem new];
        item.thumbView = [ges view];
        item.largeImageURL = [NSURL URLWithString:KStringIsNull(url)];
        [itemarr addObject:item];
    }
    ZlinkPhotoBrowserView *browser = [[ZlinkPhotoBrowserView alloc] initWithGroupItems:itemarr];
    [browser presentFromImageView:[ges view] toContainer:self.navigationController.view animated:YES index:tag completion:^{
    }];
}

//- (void)getPageData {
//    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
//    NSString *url = [NSString stringWithFormat:@"%@/api/app/getpostsdetail/", KMainUrl];
//    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.postid)} success:^(id responseObject) {
//        KLog(@"详情成功======%@",responseObject);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//        NSDictionary *resdic = (NSDictionary *)responseObject;
//        if ([resdic[@"code"] integerValue] == 2000) {
//            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
//            self.infoModel = [BC_PostListModel mj_objectWithKeyValues:dataDic[@"data"]];
//            CGFloat imgTotalH = 0;
//            if (![KStringIsNull(self.infoModel.files) isEqualToString:@""]) {
//                NSArray *imgarr = [KStringIsNull(self.infoModel.files) componentsSeparatedByString:@","];
//                CGFloat imgH = (kScreenWidth - 30)*240/345;
//                imgTotalH = (5 + imgH) * imgarr.count;
//                for (int i = 0; i < imgarr.count; i++) {
//                    UIImageView *img = [JLFactoryUtil imageView];
//                    img.tag = 200 + i;
//                    [img setCorner:4 withType:JLCornerTypeAllCorner];
//                    [self.contentView addSubview:img];
//                    [img mas_makeConstraints:^(MASConstraintMaker *make) {
//                        make.left.equalTo(self.contentView.mas_left).offset(15);
//                        make.top.equalTo(self.contentL.mas_bottom).offset(5 + i*(imgH + 5));
//                        make.size.mas_equalTo(CGSizeMake(kScreenWidth - 30, imgH));
//                    }];
//                    [img sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(imgarr[i])] placeholderImage:[UIImage imageWithColor:kCOLOR(130, 151, 218, 1)]];
//                    img.userInteractionEnabled = YES;
//                    UITapGestureRecognizer *tapImg = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showBigImg:)];
//                    [img addGestureRecognizer:tapImg];
//                }
//            } else {
//                imgTotalH = 5;
//            }
//
//            self.areaL.text = KStringIsNull(self.infoModel.city);
//            CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
//            if (size.width >= kScreenWidth - 70) {
//                size.width = kScreenWidth - 70;
//            }
//            [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
//                make.top.equalTo(self.contentL.mas_bottom).offset(15 + imgTotalH);
//                make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
//            }];
//
//            [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(self.infoModel.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
//            self.nicknameL.text = KStringIsNull(self.infoModel.user_name).length == 0 ? @"User":KStringIsNull(self.infoModel.user_name);
//            self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
//            self.titleL.text = KStringIsNull(self.infoModel.name);
//            self.contentL.text = KStringIsNull(self.infoModel.desc);
//
//        } else {
//            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
//        }
//    } failure:^(NSError *error) {
//        KLog(@"失败======%@",error);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//        [BaseFunction showAlert:@"Error"];
//    }];
//}

@end
