//
//  BC_SupervisorVideoDetailViewController.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/25.
//

#import "BC_SupervisorVideoDetailViewController.h"
#import "FullScreenHelperViewController.h"
#import "LandscapeRightViewController.h"
#import "LandscapeLeftViewController.h"
#import "EnterFullScreenTransition.h"
#import "ExitFullScreenTransition.h"
#import "WMPlayer.h"
#import "BC_PostListModel.h"
@interface BC_SupervisorVideoDetailViewController ()<WMPlayerDelegate, UIViewControllerTransitioningDelegate>
@property (nonatomic, strong) UIScrollView *scrollView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong)WMPlayer  *wmPlayer;
@property (nonatomic, strong) UILabel *titleL;
@property (nonatomic, strong) UIImageView *avatar;
@property (nonatomic, strong) UILabel *nicknameL;
@property (nonatomic, strong) UILabel *timeL;
@property (nonatomic, strong) UILabel *contentL;
@property (nonatomic, strong) UIView *areaView;
@property (nonatomic, strong) UILabel *areaL;
@property (nonatomic, strong) UIImageView *coverImg;
@property (nonatomic, strong) YYTextView *reasonTV;
@end

@implementation BC_SupervisorVideoDetailViewController
- (void)dealloc{
    [self.wmPlayer pause];
    [self.wmPlayer removeFromSuperview];
    self.wmPlayer = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    KLog(@"DetailViewController dealloc");
}

//全屏的时候hidden底部homeIndicator
- (BOOL)prefersHomeIndicatorAutoHidden{
    return self.wmPlayer.isFullscreen;
}
- (UIStatusBarStyle)preferredStatusBarStyle{
    return UIStatusBarStyleDefault;
}
- (BOOL)prefersStatusBarHidden{
    return NO;
}
//视图控制器实现的方法
- (BOOL)shouldAutorotate{
     return YES;
}
- (UIInterfaceOrientationMask)supportedInterfaceOrientations{
    return UIInterfaceOrientationMaskAllButUpsideDown;
}

-(UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    return UIInterfaceOrientationPortrait;
}
//播放器CloseButton
- (void)wmplayer:(WMPlayer *)wmplayer clickedCloseButton:(UIButton *)closeBtn{
    if (wmplayer.isFullscreen) {
        [self exitFullScreen];
    }else{
        if (self.presentingViewController) {
            [self dismissViewControllerAnimated:YES completion:^{

            }];
        }else{
            [self.navigationController popViewControllerAnimated:YES];
        }
    }
}

///全屏按钮
- (void)wmplayer:(WMPlayer *)wmplayer clickedFullScreenButton:(UIButton *)fullScreenBtn{
    if (self.wmPlayer.viewState==PlayerViewStateSmall) {
         [self enterFullScreen];
    } else {
        [self exitFullScreen];
    }
}

- (void)enterFullScreen {
    if (self.wmPlayer.viewState!=PlayerViewStateSmall) {
        return;
    }
    LandscapeRightViewController *rightVC = [[LandscapeRightViewController alloc] init];
    [self presentToVC:rightVC];
}

- (void)exitFullScreen {
    if (self.wmPlayer.viewState!=PlayerViewStateFullScreen) {
        return;
    }
    self.wmPlayer.isFullscreen = NO;
    self.wmPlayer.viewState = PlayerViewStateAnimating;
    [self dismissViewControllerAnimated:YES completion:^{
       self.wmPlayer.viewState  = PlayerViewStateSmall;
    }];
}

- (void)presentToVC:(FullScreenHelperViewController *)aHelperVC{
     self.wmPlayer.viewState = PlayerViewStateAnimating;
       self.wmPlayer.beforeBounds = self.wmPlayer.bounds;
       self.wmPlayer.beforeCenter = self.wmPlayer.center;
       self.wmPlayer.parentView = self.wmPlayer.superview;
       self.wmPlayer.isFullscreen = YES;

       aHelperVC.wmPlayer = self.wmPlayer;
       aHelperVC.modalPresentationStyle = UIModalPresentationFullScreen;
       aHelperVC.transitioningDelegate = self;
       [self presentViewController:aHelperVC animated:YES completion:^{
           self.wmPlayer.viewState = PlayerViewStateFullScreen;
       }];
}

/**
 *  旋转屏幕通知
 */
- (void)onDeviceOrientationChange:(NSNotification *)notification{
    if (self.wmPlayer.viewState!=PlayerViewStateSmall) {
        return;
    }
    if (self.wmPlayer.isLockScreen){
        return;
    }
    UIDeviceOrientation orientation = [UIDevice currentDevice].orientation;
    UIInterfaceOrientation interfaceOrientation = (UIInterfaceOrientation)orientation;
    switch (interfaceOrientation) {
        case UIInterfaceOrientationPortraitUpsideDown:{
        }
            break;
        case UIInterfaceOrientationPortrait:{

        }
            break;
        case UIInterfaceOrientationLandscapeLeft:{
            [self presentToVC:[LandscapeLeftViewController new]];
        }
            break;
        case UIInterfaceOrientationLandscapeRight:{
            [self presentToVC:[LandscapeRightViewController new]];
        }
            break;
        default:
            break;
    }
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
     self.wmPlayer.delegate = self;
    [IQKeyboardManager sharedManager].enable = NO;
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [IQKeyboardManager sharedManager].enable = YES;
}

- (void)wmplayer:(WMPlayer *)wmplayer clickedPlayOrPauseButton:(UIButton *)playOrPauseBtn {
    KLog(@"wmplayer.state===%zd", wmplayer.state);
//    if (wmplayer.state == WMPlayerStatePlaying) {
//        [wmplayer pause];
//    } else {
//        [wmplayer play];
//    }
}

- (void)wmplayer:(WMPlayer *)wmplayer doubleTaped:(UITapGestureRecognizer *)doubleTap {
    if (wmplayer.state == WMPlayerStatePlaying) {
        [wmplayer pause];
    } else {
        [wmplayer play];
    }
}

- (void)backBtnClicked {
    [self.wmPlayer pause];
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.wmPlayer = [[WMPlayer alloc] initWithFrame:CGRectMake(0, 0, kScreenWidth, 224)];
    self.wmPlayer.tintColor = UIColor.whiteColor;
    self.wmPlayer.backBtnStyle = BackBtnStylePop;
    self.wmPlayer.loopPlay = NO;
    self.wmPlayer.delegate = self;
    self.wmPlayer.enableAirPlay = NO;
    [self.view addSubview:self.wmPlayer];
    
    self.coverImg = [JLFactoryUtil imageView];
    self.coverImg.userInteractionEnabled = YES;
    self.coverImg.frame = CGRectMake(0, 0, kScreenWidth, 224);
    [self.view addSubview:self.coverImg];
    UITapGestureRecognizer *tapCover = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(playVideo)];
    [self.coverImg addGestureRecognizer:tapCover];
    UIImageView *playimg = [JLFactoryUtil imageViewWithImg:@"播放icon"];
    playimg.userInteractionEnabled = YES;
    [self.coverImg addSubview:playimg];
    [playimg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.centerX.equalTo(self.coverImg);
        make.size.mas_equalTo(CGSizeMake(48, 48));
    }];
    
    //旋转屏幕通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onDeviceOrientationChange:) name:UIDeviceOrientationDidChangeNotification object:nil];
            
    [self setupHeader];
    
}

- (void)setupHeader {
    
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 224, kScreenWidth, kScreenHeight - HOME_INDICATOR_HEIGHT - 60 - kTopHeight - 224)];
    self.scrollView.showsVerticalScrollIndicator = NO;
    [self.view addSubview:self.scrollView];
    
    self.contentView = [UIView new];
    self.contentView.backgroundColor = [UIColor whiteColor];
    [self.scrollView addSubview:self.contentView];
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(self.scrollView);
        make.width.equalTo(self.scrollView);
    }];
    
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:18 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(15);
        make.size.mas_equalTo(CGSizeMake(36, 36));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.contentView.mas_centerX).offset(-2);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-1);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(11) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(8);
        make.right.lessThanOrEqualTo(self.contentView.mas_centerX).offset(-2);
        make.top.equalTo(self.avatar.mas_centerY).offset(1);
    }];
    
    self.areaView = [JLFactoryUtil view];
    self.areaView.backgroundColor = kColorWithHex(@"#ECF8EF");
    [self.areaView setCorner:12 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.areaView];
    [self.areaView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.centerY.equalTo(self.avatar);
        make.size.mas_equalTo(CGSizeMake(30, 24));
    }];

    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.areaView addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(self.areaView.mas_left).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];

    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.areaView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(areaImg.mas_right).offset(4);
        make.width.mas_lessThanOrEqualTo(kScreenWidth/2 - 57);
    }];
    
    self.titleL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(18) textColor:kMainTextColor];
    self.titleL.numberOfLines = 0;
    [self.contentView addSubview:self.titleL];
    [self.titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.avatar.mas_bottom).offset(20);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    self.contentL.numberOfLines = 0;
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.titleL.mas_bottom).offset(12);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.contentL.mas_bottom).offset(20);
    }];
    
    if (_ifNeedAudit) {
        UIButton *passBtn = [JLFactoryUtil buttonWithTitle:@"Pass" font:MEDIUMFONT(16) textColor:UIColor.whiteColor];
        [passBtn addTarget:self action:@selector(passbtnClick) forControlEvents:UIControlEventTouchUpInside];
        passBtn.backgroundColor = kMainThemeColor;
        [passBtn setCorner:22 withType:JLCornerTypeAllCorner];
        [self.view addSubview:passBtn];
        [passBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT - 8);
            make.right.equalTo(self.view.mas_right).offset(-15);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 70)/2, 44));
        }];
        
        UIButton *refuseBtn = [JLFactoryUtil buttonWithTitle:@"Refuse" font:MEDIUMFONT(16) textColor:kMainThemeColor];
        [refuseBtn addTarget:self action:@selector(refuseBtnClick) forControlEvents:UIControlEventTouchUpInside];
        refuseBtn.backgroundColor = UIColor.whiteColor;
        refuseBtn.layer.borderWidth = 1;
        refuseBtn.layer.borderColor = kMainThemeColor.CGColor;
        [refuseBtn setCorner:22 withType:JLCornerTypeAllCorner];
        [self.view addSubview:refuseBtn];
        [refuseBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(self.view.mas_bottom).offset(-HOME_INDICATOR_HEIGHT - 8);
            make.right.equalTo(passBtn.mas_left).offset(-40);
            make.size.mas_equalTo(CGSizeMake((kScreenWidth - 70)/2, 44));
        }];
        
    }
    
    self.areaL.text = KStringIsNull(self.infoModel.city);
    CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
    if (size.width >= kScreenWidth/2 - 57) {
        size.width = kScreenWidth/2 - 57;
    }
    [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
    }];
    
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(self.infoModel.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
    self.nicknameL.text = KStringIsNull(self.infoModel.user_name).length == 0 ? @"User":KStringIsNull(self.infoModel.user_name);
    self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
    self.titleL.text = KStringIsNull(self.infoModel.name);
    self.contentL.text = KStringIsNull(self.infoModel.desc);
    
    //@"http://static.tripbe.com/videofiles/20121214/9533522808.f4v.mp4"
    if (self.infoModel.files.count > 0) {
        NSString *encodedString = [KStringIsNull(self.infoModel.files[0]) stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
        WMPlayerModel *playerModel = [WMPlayerModel new];
        playerModel.videoURL = [NSURL URLWithString:KStringIsNull(encodedString)];
        playerModel.title = @"";
        self.wmPlayer.playerModel = playerModel;
    }
    

}

- (nullable id <UIViewControllerAnimatedTransitioning>)animationControllerForPresentedController:(UIViewController *)presented presentingController:(UIViewController *)presenting sourceController:(UIViewController *)source{
    return [[EnterFullScreenTransition alloc] initWithPlayer:self.wmPlayer];
}

- (nullable id <UIViewControllerAnimatedTransitioning>)animationControllerForDismissedController:(UIViewController *)dismissed{
    return [[ExitFullScreenTransition alloc] initWithPlayer:self.wmPlayer];
}

//准备播放的代理方法
- (void)wmplayerReadyToPlay:(WMPlayer *)wmplayer WMPlayerStatus:(WMPlayerState)state {
    
}

- (void)playVideo {
    [UIView animateWithDuration:0.3 animations:^{
        self.coverImg.alpha = 0;
    } completion:^(BOOL finished) {
        [self.wmPlayer play];
    }];
}

- (void)passbtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 150)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"The post will be published successfully after passing" font:REGULARFONT(16) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(30);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmPassAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    [popup showWithLayout:layout];
    
}

- (void)confirmPassAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
    
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postsaudit/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(self.infoModel.itemid), @"action":@"pass", @"reason":@""} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"refreshPageAfterAudit" object:nil];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popViewControllerAnimated:YES];

        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

- (void)cancelAction:(UIButton *)sender {
    [sender dismissPresentingPopup];
}

- (void)refuseBtnClick {
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 290, 200)];
    contentView.backgroundColor = UIColor.whiteColor;
    [contentView setCorner:12 withType:JLCornerTypeAllCorner];
    
    UILabel *titleL = [JLFactoryUtil labelWithText:@"Reason" font:MEDIUMFONT(15) textColor:kMainTextColor];
    titleL.textAlignment = NSTextAlignmentCenter;
    titleL.numberOfLines = 0;
    [contentView addSubview:titleL];
    [titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(contentView.mas_top).offset(15);
        make.width.mas_lessThanOrEqualTo(260);
    }];
    
    UIView *dview = [JLFactoryUtil view];
    [dview setCorner:4 withType:JLCornerTypeAllCorner];
    dview.backgroundColor = kColorWithHex(@"#F5F5F5");
    [contentView addSubview:dview];
    [dview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.top.equalTo(titleL.mas_bottom).offset(10);
        make.size.mas_equalTo(CGSizeMake(260, 100));
    }];
    
    self.reasonTV = [YYTextView new];
    self.reasonTV.placeholderFont = REGULARFONT(14);
    self.reasonTV.placeholderText = @"Reason for rejection";
    self.reasonTV.placeholderTextColor = kLightGrayTextColor;
    self.reasonTV.textColor = kMainTextColor;
    self.reasonTV.font = REGULARFONT(14);
    [dview addSubview:self.reasonTV];
    [self.reasonTV mas_makeConstraints:^(MASConstraintMaker *make) {
        make.edges.equalTo(dview).insets(UIEdgeInsetsMake(10, 10, 10, 10));
    }];
    
    UIView *hline = [UIView new];
    hline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:hline];
    [hline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(contentView);
        make.bottom.equalTo(contentView.mas_bottom).offset(-40);
        make.height.mas_equalTo(1);
    }];
    
    UIView *vline = [UIView new];
    vline.backgroundColor = kColorWithHex(@"#EEEEEE");
    [contentView addSubview:vline];
    [vline mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(contentView);
        make.top.equalTo(hline.mas_bottom);
        make.bottom.equalTo(contentView.mas_bottom);
        make.width.mas_equalTo(1);
    }];
    
    UIButton *cancelBtn = [JLFactoryUtil buttonWithTitle:@"Cancel" font:REGULARFONT(14) textColor:kMainThemeColor];
    [cancelBtn addTarget:self action:@selector(cancelAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:cancelBtn];
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(contentView.mas_left).offset(15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.right.equalTo(vline.mas_left);
    }];
    
    UIButton *confirmBtn = [JLFactoryUtil buttonWithTitle:@"OK" font:REGULARFONT(14) textColor:kMainThemeColor];
    [confirmBtn addTarget:self action:@selector(confirmRefuseAction:) forControlEvents:UIControlEventTouchUpInside];
    [contentView addSubview:confirmBtn];
    [confirmBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(contentView.mas_right).offset(-15);
        make.bottom.equalTo(contentView.mas_bottom);
        make.top.equalTo(hline.mas_bottom);
        make.left.equalTo(vline.mas_right);
    }];
    
    KLCPopupLayout layout = KLCPopupLayoutMake(KLCPopupHorizontalLayoutCenter, KLCPopupVerticalLayoutCenter);
    KLCPopup *popup = [KLCPopup popupWithContentView:contentView showType:KLCPopupShowTypeShrinkIn dismissType:KLCPopupDismissTypeShrinkOut maskType:KLCPopupMaskTypeDimmed dismissOnBackgroundTouch:NO dismissOnContentTouch:NO];
    popup.shouldHandleKeyboard = YES;
    [popup showWithLayout:layout];
}

- (void)confirmRefuseAction:(UIButton *)sender {
    if ([KStringIsNull(self.reasonTV.text) isEqualToString:@""]) {
        [BaseFunction showAlert:@"Please input reason"];
        return;
    }
    
    [sender dismissPresentingPopup];
        
    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
    NSString *url = [NSString stringWithFormat:@"%@/api/app/postsaudit/", KMainUrl];
    [PPNetworkHelper POST:url parameters:@{@"id":KStringIsNull(self.infoModel.itemid), @"action":@"deny", @"reason":KStringIsNull(self.reasonTV.text)} success:^(id responseObject) {
        KLog(@"成功======%@",responseObject);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        NSDictionary *resdic = (NSDictionary *)responseObject;
        if ([resdic[@"code"] integerValue] == 2000) {
            [kNotificationCenter postNotificationName:@"refreshPageAfterAudit" object:nil];
            [BaseFunction showAlert:@"Success"];
            [self.navigationController popViewControllerAnimated:YES];

        } else {
            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
        }
    } failure:^(NSError *error) {
        KLog(@"失败======%@",error);
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [BaseFunction showAlert:@"Error"];
    }];
}

//- (void)getPageData {
//    [MBProgressHUD showHUDAddTo:self.view alertMsg:@"" animated:YES];
//    NSString *url = [NSString stringWithFormat:@"%@/api/app/getpostsdetail/", KMainUrl];
//    [PPNetworkHelper GET:url parameters:@{@"id":KStringIsNull(self.postid)} success:^(id responseObject) {
//        KLog(@"详情成功======%@",responseObject);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//        NSDictionary *resdic = (NSDictionary *)responseObject;
//        if ([resdic[@"code"] integerValue] == 2000) {
//            NSDictionary *dataDic = (NSDictionary *)resdic[@"data"];
//            self.infoModel = [BC_PostListModel mj_objectWithKeyValues:dataDic[@"data"]];
//
//            self.areaL.text = KStringIsNull(self.infoModel.city);
//            CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
//            if (size.width >= kScreenWidth/2 - 57) {
//                size.width = kScreenWidth/2 - 57;
//            }
//            [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
//                make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
//            }];
//
//            [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(self.infoModel.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
//            self.nicknameL.text = KStringIsNull(self.infoModel.user_name).length == 0 ? @"User":KStringIsNull(self.infoModel.user_name);
//            self.timeL.text = KStringIsNull(self.infoModel.create_datetime);
//            self.titleL.text = KStringIsNull(self.infoModel.name);
//            self.contentL.text = KStringIsNull(self.infoModel.desc);
//
//            //@"http://static.tripbe.com/videofiles/20121214/9533522808.f4v.mp4"
//            WMPlayerModel *playerModel = [WMPlayerModel new];
//            playerModel.videoURL = [NSURL URLWithString:KStringIsNull(self.infoModel.files)];
//            playerModel.title = @"";
//            self.wmPlayer.playerModel = playerModel;
//        } else {
//            [BaseFunction showAlert:kStringFormat(@"%@", resdic[@"msg"])];
//        }
//    } failure:^(NSError *error) {
//        KLog(@"失败======%@",error);
//        [MBProgressHUD hideHUDForView:self.view animated:YES];
//        [BaseFunction showAlert:@"Error"];
//    }];
//}


@end
