//
//  BC_SupervisorHomeHasBtnTableViewCell.m
//  BuildingComApp
//
//  Created by Jiangl on 2021/12/25.
//

#import "BC_SupervisorHomeHasBtnTableViewCell.h"
#import "BC_PostListModel.h"
@implementation BC_SupervisorHomeHasBtnTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.contentView.backgroundColor = UIColor.whiteColor;
        [self setupUI];
    }
    return self;
}

- (void)setupUI {
    self.avatar = [JLFactoryUtil imageView];
    [self.avatar setCorner:20 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.avatar];
    [self.avatar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentView.mas_top).offset(15);
        make.size.mas_equalTo(CGSizeMake(40, 40));
    }];
    
    self.nicknameL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(15) textColor:kMainTextColor];
    [self.contentView addSubview:self.nicknameL];
    [self.nicknameL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.bottom.equalTo(self.avatar.mas_centerY).offset(-1);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-10);
    }];
    
    self.timeL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(12) textColor:kLightGrayTextColor];
    [self.contentView addSubview:self.timeL];
    [self.timeL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.avatar.mas_right).offset(10);
        make.top.equalTo(self.avatar.mas_centerY).offset(1);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-10);
    }];
    
    self.titleL = [JLFactoryUtil labelWithText:@" " font:MEDIUMFONT(14) textColor:kMainTextColor];
    self.titleL.numberOfLines = 0;
    [self.contentView addSubview:self.titleL];
    [self.titleL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.avatar.mas_bottom).offset(12);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.contentL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(14) textColor:kColorWithHex(@"#777777")];
    self.contentL.numberOfLines = 2;
    [self.contentView addSubview:self.contentL];
    [self.contentL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.titleL.mas_bottom).offset(8);
        make.right.lessThanOrEqualTo(self.contentView.mas_right).offset(-15);
    }];
    
    self.areaView = [JLFactoryUtil view];
    self.areaView.backgroundColor = kColorWithHex(@"#ECF8EF");
    [self.areaView setCorner:12 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.areaView];
    [self.areaView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(15);
        make.top.equalTo(self.contentL.mas_bottom).offset(8);
        make.size.mas_equalTo(CGSizeMake(30, 24));
        make.bottom.equalTo(self.contentView.mas_bottom).offset(-20);
    }];
    
    UIImageView *areaImg = [JLFactoryUtil imageViewWithImg:@"位置"];
    [self.areaView addSubview:areaImg];
    [areaImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(self.areaView.mas_left).offset(10);
        make.size.mas_equalTo(CGSizeMake(16, 16));
    }];
    
    self.areaL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(13) textColor:kColorWithHex(@"#05A833")];
    [self.areaView addSubview:self.areaL];
    [self.areaL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.areaView);
        make.left.equalTo(areaImg.mas_right).offset(3);
        make.width.mas_lessThanOrEqualTo(200);
    }];
    
    UIButton *repBtn = [JLFactoryUtil buttonWithImage:@"屏蔽"];
    [repBtn setTitle:@"shield" forState:0];
    repBtn.titleLabel.font = REGULARFONT(14);
    [repBtn setTitleColor:kMainTextColor forState:0];
    repBtn.titleEdgeInsets = UIEdgeInsetsMake(0, 5, 0, 0);
    [repBtn addTarget:self action:@selector(repBtnClick) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:repBtn];
    [repBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-10);
        make.centerY.equalTo(self.areaView);
        make.size.mas_equalTo(CGSizeMake(90, 20));
    }];
    
    self.passBtn = [JLFactoryUtil buttonWithTitle:@"Pass" font:REGULARFONT(15) textColor:kMainThemeColor];
//    [self.passBtn addTarget:self action:@selector(passBtnClick) forControlEvents:UIControlEventTouchUpInside];
    self.passBtn.enabled = NO;
    self.passBtn.layer.borderWidth = 1;
    self.passBtn.layer.borderColor = kMainThemeColor.CGColor;
    [self.passBtn setCorner:16 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.passBtn];
    [self.passBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.top.equalTo(self.areaView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake(80, 32));
    }];
    
    self.refuseBtn = [JLFactoryUtil buttonWithTitle:@"Refuse" font:REGULARFONT(15) textColor:kMainTextColor];
//    [self.refuseBtn addTarget:self action:@selector(refuseBtnClick) forControlEvents:UIControlEventTouchUpInside];
    self.refuseBtn.layer.borderWidth = 1;
    self.refuseBtn.enabled = NO;
    self.refuseBtn.layer.borderColor = kColorWithHex(@"#AAAAAA").CGColor;
    [self.refuseBtn setCorner:16 withType:JLCornerTypeAllCorner];
    [self.contentView addSubview:self.refuseBtn];
    [self.refuseBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.passBtn.mas_left).offset(-15);
        make.top.equalTo(self.areaView.mas_bottom).offset(12);
        make.size.mas_equalTo(CGSizeMake(80, 32));
    }];
    self.passBtn.hidden = YES;
    self.refuseBtn.hidden = YES;
    
    self.statusL = [JLFactoryUtil labelWithText:@" " font:REGULARFONT(15) textColor:kColorWithHex(@"#777777")];
    [self.contentView addSubview:self.statusL];
    [self.statusL mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView.mas_right).offset(-15);
        make.centerY.equalTo(self.nicknameL);
    }];
        
}

- (void)repBtnClick {
    if (self.delegate && [self.delegate respondsToSelector:@selector(shieldItem:)]) {
        [self.delegate shieldItem:_model];
    }
}

- (void)setPageIndex:(NSInteger)pageIndex {
//    if (pageIndex == 0) {
//        self.statusL.hidden = YES;
//        self.refuseBtn.hidden = NO;
//        self.passBtn.hidden = NO;
//        [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
//            make.bottom.equalTo(self.contentView.mas_bottom).offset(-60);
//        }];
//    } else {
//        self.statusL.hidden = NO;
//        self.refuseBtn.hidden = YES;
//        self.passBtn.hidden = YES;
//        [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
//            make.bottom.equalTo(self.contentView.mas_bottom).offset(-20);
//        }];
//        self.statusL.text = @"through";//Refused
//    }
}

- (void)setModel:(BC_PostListModel *)model {
    _model = model;
    [self.avatar sd_setImageWithURL:[NSURL URLWithString:KStringIsNull(model.user_avatar)] placeholderImage:kAvatarPlaceHolderImg];
    
    self.nicknameL.text = KStringIsNull(model.user_name).length == 0 ? @"User":KStringIsNull(model.user_name);
    self.timeL.text = KStringIsNull(model.create_datetime);
    self.titleL.text = KStringIsNull(model.name);
    self.contentL.text = KStringIsNull(model.desc);
    self.areaL.text = KStringIsNull(model.city);
    CGSize size = [self.areaL sizeThatFits:CGSizeMake(kScreenWidth, CGFLOAT_MAX)];
    if (size.width >= 200) {
        size.width = 200;
    }
    
    if ([KStringIsNull(model.audit_status) isEqualToString:@"10"]) {
        self.statusL.hidden = YES;
        self.refuseBtn.hidden = NO;
        self.passBtn.hidden = NO;
        [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
            make.bottom.equalTo(self.contentView.mas_bottom).offset(-60);
        }];
    } else {
        self.statusL.hidden = NO;
        self.refuseBtn.hidden = YES;
        self.passBtn.hidden = YES;
        [self.areaView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(CGSizeMake(40+size.width, 24));
            make.bottom.equalTo(self.contentView.mas_bottom).offset(-20);
        }];
        if ([KStringIsNull(model.audit_status) isEqualToString:@"20"]) {
            self.statusL.text = @"through";
        } else {
            self.statusL.text = @"Refused";
        }
    }
    
}

- (void)refuseBtnClick {
    
}

- (void)passBtnClick {
    
}

@end
