//
//  BaseFunction.m
//  YiKaoTianXia
//
//  Created by Jiangl on 2019/7/20.
//  Copyright © 2019 姜磊. All rights reserved.
//

#import "BaseFunction.h"
#import <sys/utsname.h>
#import <AVFoundation/AVFoundation.h>
@implementation BaseFunction
+ (NSDictionary *)dictionaryWithJsonString:(NSString *)jsonString {
    if (jsonString == nil) {
        return nil;
    }

    NSData *jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:jsonData
                                                        options:NSJSONReadingMutableContainers
                                                          error:&err];
    if(err) {

        return nil;
    }
    return dic;
}

// 设置圆角
+ (void)changeCorner: (UIView*)changeView  withRadius:(CGFloat)changeradius {
    changeView.layer.masksToBounds = YES;
    changeView.layer.cornerRadius = changeradius;
}

//验证手机号
+(BOOL)validatePhone:(NSString *)phone {
    NSString *phoneRegex = @"^1(3[0-9]|4[579]|5[0-35-9]|6[6]|7[0-35-9]|8[0-9]|9[89])\\d{8}$";
    NSPredicate *phoneTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@",phoneRegex];
    return [phoneTest evaluateWithObject:phone];
}

+(void)showAlert:(NSString*)msg
{
    if([msg length]>0){
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
            hud.mode = MBProgressHUDModeText;
            hud.label.numberOfLines = 0;
            hud.label.text = msg;
            hud.label.textColor = UIColor.whiteColor;
            hud.label.font = REGULARFONT(14);
            hud.margin = 15.f;
            hud.bezelView.style = MBProgressHUDBackgroundStyleSolidColor;
            hud.bezelView.backgroundColor = UIColor.blackColor;
            hud.bezelView.alpha = 0.8;
            hud.layer.cornerRadius = 8.f;
            hud.removeFromSuperViewOnHide = YES;
            [hud hideAnimated:YES afterDelay:1.5];
        });
    }
}

+(void)showAlertMsg:(NSString *)msg andDelay:(NSTimeInterval)delay {
    if([msg length]>0){
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
            hud.mode = MBProgressHUDModeText;
            hud.label.text = msg;//自动换行
            hud.label.textColor = kColorWithHex(@"#021A12");
            hud.label.font = REGULARFONT(15);
            hud.margin = 15.f;
            hud.bezelView.style = MBProgressHUDBackgroundStyleSolidColor;
            hud.bezelView.backgroundColor = kColorWithHex(@"#CFDCD8");
            hud.bezelView.alpha = 1;
            hud.layer.cornerRadius = 8.f;
            hud.removeFromSuperViewOnHide = YES;
            [hud hideAnimated:YES afterDelay:delay];
        });
    }
}

+ (BOOL)checkPassword:(NSString*) password
{
    NSString*pattern =@"^(?![0-9]+$)(?![a-zA-Z]+$)[a-zA-Z0-9]{8,16}";

    NSPredicate*pred = [NSPredicate predicateWithFormat:@"SELF MATCHES %@",pattern];

    BOOL isMatch = [pred evaluateWithObject:password];

    return isMatch;
}

//验证邮箱
+ (BOOL)isValidateEmail:(NSString *)email {
    NSString *emailRegex = @"[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}";
    NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex];
    return [emailTest evaluateWithObject:email];
}

+ (UIColor *)colorWithHexString:(NSString *)stringToConvert
{
    NSString *cString = [[stringToConvert stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] uppercaseString];

    // String should be 6 or 8 characters
    if ([cString length] < 6) {
        return [UIColor clearColor];
    }

    // strip 0X if it appears
    if ([cString hasPrefix:@"0X"])
        cString = [cString substringFromIndex:2];
    if ([cString hasPrefix:@"#"])
        cString = [cString substringFromIndex:1];
    if ([cString length] != 6)
        return [UIColor clearColor];

    // Separate into r, g, b substrings
    NSRange range;
    range.location = 0;
    range.length = 2;

    //r
    NSString *rString = [cString substringWithRange:range];

    //g
    range.location = 2;
    NSString *gString = [cString substringWithRange:range];

    //b
    range.location = 4;
    NSString *bString = [cString substringWithRange:range];

    // Scan values
    unsigned int r, g, b;
    [[NSScanner scannerWithString:rString] scanHexInt:&r];
    [[NSScanner scannerWithString:gString] scanHexInt:&g];
    [[NSScanner scannerWithString:bString] scanHexInt:&b];

    return [UIColor colorWithRed:((float) r / 255.0f) green:((float) g / 255.0f) blue:((float) b / 255.0f) alpha:1.0f];
}

//去掉字符串中的空格和换行
+(NSString *)handleStringWithWhitespaceAndNewline:(NSString *)content
{
    //去除掉首尾的空白字符和换行字符
    content= [content stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    content = [content stringByReplacingOccurrencesOfString:@"\r" withString:@""];
    content = [content stringByReplacingOccurrencesOfString:@"\n" withString:@""];
    content = [content stringByReplacingOccurrencesOfString:@"\t" withString:@""];
    content=[content stringByReplacingOccurrencesOfString:@"　　" withString:@""];
    content=[content stringByReplacingOccurrencesOfString:@"  " withString:@""];//@#

    content = [content stringByReplacingOccurrencesOfString:@"&nbsp;" withString:@""];

    return content;
}

/**字符串如果为空 ,则返回 @"" */
+ (NSString *)stringIsNull:(NSString *)string {

    NSString *string1 = [NSString stringWithFormat:@"%@",string];
    if ([BaseFunction isNull:string1]) {
        return @"";
    }else {
        return string1;
    }

}

/**判断字符串是否为空*/
+ (BOOL)isNull:(NSString *)string {
    if(string == nil || string == NULL || [string  isEqual: @""] || [string  isEqual: @"(null)"] ||
       [string  isEqual: @"null"] || [string  isEqual: @"NULL"] ||

       [string  isEqual: @"<null>"] || [string  isEqual: @"<NULL>"] ||

       [string isKindOfClass:[NSNull class]]){

        return YES;

    }else{
        return NO;

    }
}

+ (NSString *)filterHTML:(NSString *)str
{
    NSScanner * scanner = [NSScanner scannerWithString:str];
    NSString * text = nil;
    while([scanner isAtEnd]==NO)
    {
        //找到标签的起始位置
        [scanner scanUpToString:@"<" intoString:nil];
        //找到标签的结束位置
        [scanner scanUpToString:@">" intoString:&text];
        //替换字符
        str = [str stringByReplacingOccurrencesOfString:[NSString stringWithFormat:@"%@>",text] withString:@""];
    }
    return str;
}

+ (NSString *)filterHTMLTag:(NSString *)str {
    //替换字符
    str  =  [str  stringByReplacingOccurrencesOfString:@"&mdash;" withString:@"-"];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&ldquo;" withString:@"\""];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&rdquo;" withString:@"\""];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&nbsp;" withString:@" "];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&rsquo;" withString:@"’"];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&lsquo;" withString:@"‘"];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&middot;" withString:@"·"];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&quot;" withString:@"\""];
    str  =  [str  stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
    str  =  [str  stringByReplacingOccurrencesOfString:@"<strong>" withString:@""];
    str  =  [str  stringByReplacingOccurrencesOfString:@"</strong>" withString:@""];
    str  =  [str  stringByReplacingOccurrencesOfString:@"\n" withString:@" "];
    return str;

}

+ (NSString *)appendHtmlBody:(NSString *)content {
    NSString *first = @"<!DOCTYPE html><html><head><meta charset='utf-8' /><meta name='viewport' content='width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0'><meta name='apple-mobile-web-app-capable' content='yes'><title></title><style>  body{background-color:#fff;margin:0px !important;padding:0px !important} img{display:block;width:100% !important;height:auto} *{word-break: break-all;color:#000} p{margin-left:-10px;margin:0px !important;padding:0px !important} </style></head><body>";
    NSString *second = @"</body><html>";
    NSString *temp = [first stringByAppendingString:KStringIsNull(content)];
    NSString *final = [temp stringByAppendingString:second];
    return final;
}

+ (UIImage *)thumbnailImageForVideo:(NSURL *)videoURL atTime:(NSTimeInterval)time {
    AVURLAsset *asset = [[AVURLAsset alloc] initWithURL:videoURL options:nil];
        NSParameterAssert(asset);
    AVAssetImageGenerator *assetImageGenerator =[[AVAssetImageGenerator alloc] initWithAsset:asset];
    assetImageGenerator.appliesPreferredTrackTransform = YES;
    assetImageGenerator.apertureMode = AVAssetImageGeneratorApertureModeEncodedPixels;
    assetImageGenerator.maximumSize = CGSizeMake(kScreenWidth - 30, 184);

    CGImageRef thumbnailImageRef = NULL;
    CFTimeInterval thumbnailImageTime = time;
    NSError *thumbnailImageGenerationError = nil;
    thumbnailImageRef = [assetImageGenerator copyCGImageAtTime:CMTimeMake(thumbnailImageTime, 60)actualTime:NULL error:&thumbnailImageGenerationError];
    
    if(!thumbnailImageRef)
    NSLog(@"thumbnailImageGenerationError %@",thumbnailImageGenerationError);
    
    UIImage*thumbnailImage = thumbnailImageRef ? [[UIImage alloc]initWithCGImage: thumbnailImageRef] : nil;
    
    CGImageRelease(thumbnailImageRef);
    
    return thumbnailImage;

}

@end
